package com.hive.sample


import android.app.Activity
import android.app.AlertDialog
import android.view.View
import android.widget.ArrayAdapter
import android.widget.EditText
import android.widget.Spinner
import android.widget.Toast
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.*
import com.hive.Auth.Account
import com.hive.Auth.AuthInitResult
import com.hive.Auth.AuthMaintenanceInfo
import com.hive.Auth.LoginType
import com.hive.Promotion.EngagementEventState
import com.hive.Promotion.EngagementEventType
import com.hive.Promotion.EngagementListener
import com.hive.base.Android
import com.hive.sample.game.*
import com.hive.userengagement.UserEngagementCoupon
import com.hive.sample.uihelper.ViewPagerBase
import org.json.JSONException
import org.json.JSONObject
import java.util.*


/**
 * HIVE SDK 의 Auth (인증) 기능 테스트 뷰
 *
 * @author hife
 */
class AuthTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {


    private var recentCurrentAccountVid: String? = null
    private var recentUsedAccountVid: String? = null


    // SDK 가 특정한 조건에서 클라이언트에 개입 (Engagement) 하기 위한 이벤트 리스너
    private val engagementListener = object : EngagementListener {

        override fun onEngagement(result: ResultAPI, engagementEventType: EngagementEventType, eventState: EngagementEventState, param: JSONObject?) {

            addLog(String.format("AuthInitializeListener.onEngagement() Callback\nresult = %s\n", result.toString()))
            addLog(String.format("engagementEventType = %s , %s\n", engagementEventType.name, eventState.name))
            addLog(String.format("param = %s\n", param?.toString() ?: "null"))


            when (engagementEventType) {
                Promotion.EngagementEventType.EVENT_TYPE -> {
                    // Engagement의 전체 시작과 끝.
                    if (eventState === EngagementEventState.BEGIN) {
                        // Engagement의 모든 동작이 수행됨.
                    } else if (eventState === EngagementEventState.FINISH) {
                        // Engagement의 모든 동작이 수행 완료됨.
                    }
                }

                Promotion.EngagementEventType.AUTH_LOGIN_VIEW -> {

                    if (eventState === EngagementEventState.START) {
                        // HIVE 멤버십 로그인 페이지 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // HIVE 멤버십 로그인 수행 완료. (로그인 완료 / 창닫음 수행 이후 전달됨)

                        var loginTypeName: String? = null
                        try {
                            loginTypeName = param!!.getString("loginType")
                        } catch (e: JSONException) {
                            e.printStackTrace()
                        }

                        if (LoginType.SELECT.name == loginTypeName) {

                            // 사용자 선택 창, test
                            val currentVidData = HashMap<String, Any>()
                            val jCurrentAccount = param!!.optJSONObject("currentAccount")
                            currentVidData["vid"] = jCurrentAccount.optString("vid")

                            val currentVidGameData = HashMap<String, Any>()
                            currentVidGameData["Name"] = "Nam"
                            currentVidGameData["Level"] = 20

                            currentVidData["data"] = currentVidGameData


                            val usedVidData = HashMap<String, Any>()
                            val jUsedAccount = param.optJSONObject("usedAccount")
                            usedVidData["vid"] = jUsedAccount.optString("vid")

                            val usedVidGameData = HashMap<String, Any>()
                            usedVidGameData["Name"] = "An"
                            usedVidGameData["Level"] = 10

                            usedVidData["data"] = usedVidGameData


                            addLog("Auth.selectLogin(currentVidData, usedVidData, listener) Called\n")
                            addLog(String.format("currentVidData = %s\n", currentVidData.toString()))
                            addLog(String.format("usedVidData = %s\n", usedVidData.toString()))

                            com.hive.Auth.showLoginSelection(currentVidData, usedVidData, object : com.hive.Auth.AuthLoginListener {

                                override fun onAuthLogin(result: ResultAPI, loginType: LoginType?, currentAccount: Account?, usedAccount: Account?) {

                                    val msg = String.format("AuthLoginListener.onAuthLogin() Callback\nresult = %s\nloginType = %s\ncurrentAccount = %s\nusedAccount = %s\n", result.toString(), loginType!!.name, currentAccount?.toString()
                                            ?: "null", usedAccount?.toString() ?: "null")

                                    addLog(msg)
                                    showAlertDialog("Auth SelectLogin Result", msg)


                                    if (result.isSuccess) {
                                        GameUser.login(null)
                                    }
                                }
                            })


                        } else {

                            addLog(String.format("로그인이 완료되지 않았습니다.\n"))

                            val toast = Toast.makeText(Configuration.context, "로그인이 완료되지 않았습니다.\n", Toast.LENGTH_SHORT)
                            toast.show()
                        }
                    }
                }

                Promotion.EngagementEventType.SOCIAL_INQUIRY_VIEW -> {
                    if (eventState === EngagementEventState.START) {
                        // 1:1 문의 페이지 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // 1:1 문의 페이지 노출 완료 (1:1문의 창 닫은 이후 전달)
                    }
                }

                Promotion.EngagementEventType.PROMOTION_VIEW -> {
                    if (eventState === EngagementEventState.START) {
                        // Promotion View 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // Promotion View 노출 완료 (Promotion 창 닫은 이후 전달)
                    }


                }

                Promotion.EngagementEventType.COUPON -> {
                    if (eventState === EngagementEventState.START) {
                        // 쿠폰 소모 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // 쿠폰 소모 요청 완료 및 응답 전달
                        var couponConsumeResult: UserEngagementCoupon.CouponConsumeResult? = null
                        var message: String? = null

                        try {
                            couponConsumeResult = UserEngagementCoupon.CouponConsumeResult.getCouponConsumeResult(param!!.getInt("code"))
                            message = param.getString("message")
                        } catch (e: JSONException) {
                            e.printStackTrace()
                        }

                        addLog(String.format("Coupon Code = %s\nCoupon Message = %s\n", couponConsumeResult!!.name, message))
                        val toast = Toast.makeText(Configuration.context, "쿠폰 처리 결과입니다.\nCode : $couponConsumeResult\nmMessage : $message", Toast.LENGTH_SHORT)
                        toast.show()
                    }
                }

                Promotion.EngagementEventType.OFFERWALL_VIEW -> {
                    if (eventState === EngagementEventState.START) {
                        // Offerwall 페이지 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // Offerwall 페이지 노출 완료 (Offerwall 창 닫은 이후 전달)
                    }
                }

                Promotion.EngagementEventType.EVENT -> {
                    if (eventState === EngagementEventState.START) {
                        // host가 game인 경우 해당 데이터 전달
                    } else if (eventState === EngagementEventState.END) {
                        // host가 game인 경우 해당 데이터 전달
                    }

                    var api: String? = null
                    var parameter: String? = null
                    try {
                        api = param!!.getString("api")
                        parameter = param.getString("param")
                    } catch (e: JSONException) {
                        e.printStackTrace()
                    }

                    addLog(String.format("apiName = %s\napiParam = %s\n", api, parameter))

                    val toast = Toast.makeText(Configuration.context, "이벤트를 전달받았습니다.\nAPI : $api, Param : $parameter", Toast.LENGTH_SHORT)
                    toast.show()
                }


                Promotion.EngagementEventType.IAP_UPDATED -> {
                    if (eventState === EngagementEventState.START) {
                        // 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
                    } else if (eventState === EngagementEventState.END) {
                        // 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
                    }
                    val toast = Toast.makeText(Configuration.context, "IAP Purchase Updated.", Toast.LENGTH_SHORT)
                    toast.show()
                }

                //4.5.0 new engagement Callback parameter.
                Promotion.EngagementEventType.IAP_PROMOTE -> {
                    if (eventState === EngagementEventState.START) {
                        // IAP Promote 데이터 존재 여부 전달
                    } else if (eventState === EngagementEventState.END) {
                        // IAP Promote 데이터 존재 여부 전달
                    }
                }

                Promotion.EngagementEventType.IAP_PURCHASE -> {
                    if (eventState === EngagementEventState.START) {
                        // IAP Purhcase 구매 시작
                    } else if (eventState === EngagementEventState.END) {
                        // IAP Purhcase 구매 완료 (+영수증데이터)
                        var iapV4Receipt: IAPV4.IAPV4Receipt? = null

                        try {
                            iapV4Receipt = param!!.get("iapV4Receipt") as IAPV4.IAPV4Receipt
                        } catch (e: JSONException) {
                            e.printStackTrace()
                        }

                        val builder = StringBuilder()
                        builder.append(String.format("IAPV4PurchaseListener.onIAPV4Purchase() Callback - result = %s\n", result))
                                .append("IAPV4Receipt : ").append(iapV4Receipt).append("\n")


                        //                                                forLog.addLog(builder.toString());
                        addLog(builder.toString())

                        if (result.isSuccess && iapV4Receipt != null) {

                            GameNetwork.receiptVerify(iapV4Receipt, object : IGameListener {
                                override fun onGameListener(result: Boolean, response: Response?) {
                                    if (result) {
                                        if (response?.isSuccess == true) {

                                            IAPV4.transactionFinish(iapV4Receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                    Toast.makeText(activity, "Purchase Complete", Toast.LENGTH_SHORT).show()
                                                }
                                            })
                                        }
                                    }
                                }
                            })

                        }
                        showAlertDialog("Purchase Result", result.toString() + "\n" + builder.toString())
                    }
                }
                else -> {
                }
            }
        }
    }


    init {


        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)


        val spAdapter: ArrayAdapter<CharSequence> = ArrayAdapter(activity, android.R.layout.simple_spinner_item)

        this.view.findViewById<View>(R.id.btnAuthRequestPermissionViewData).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthInitialize).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthGetLoginType).setOnClickListener(this)

        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spAdapter.add("Guest")
        spAdapter.add("Account")
        spAdapter.add("Select")
        spAdapter.add("Auto")

        (this.view.findViewById<View>(R.id.spLoginType) as Spinner).adapter = spAdapter

        this.view.findViewById<View>(R.id.btnAuthLogin).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthSelectLogin).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthLogout).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthGetAccount).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthCheckMaintenance).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthShowTerms).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthShowAdultConfirm).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthSetEmergencyMode).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthReset).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnUserEngagementSetEngagementListener).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnUserEngagementSetEngagementReady).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPlatformHelperAuthRequestUserPermissions).setOnClickListener(this)
    }


    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {

            R.id.btnAuthRequestPermissionViewData -> {

                addLog("\n\n=== HIVE SDK 권한 고지 팝업 데이터 요청 ===\n");
                addLog("Auth.requestPermissionViewData(PermissionViewDataListener) Called\n")

                Auth.requestPermissionViewData(object : Permission.PermissionViewDataListener {
                    override fun onPermissionViewData(result: ResultAPI, data: Permission.PermissionViewData?) {
                        if (result.isSuccess && data != null) {
                            if (result.code === ResultAPI.Code.AuthV4SkipPermissionView) {
                                addLog(String.format("Can skip permission notice"))
                            } else {
                                addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()))

                                val popupTitle = data.contents
                                val permissionViewUnitDatas = data.permissions
                                val sb = StringBuilder()
                                for (permissionViewUnitData in permissionViewUnitDatas) {
                                    sb.append("Title : ").append(permissionViewUnitData.title).append("\n")
                                    sb.append("Description : ").append(permissionViewUnitData.contents).append("\n")
                                    sb.append("Permission Name : ").append(permissionViewUnitData.nativePermissionName).append("\n\n")
                                }

                                val commonDatas = data.commons
                                for (commonUnitData in commonDatas) {
                                    sb.append("Title : ").append(commonUnitData.title).append("\n")
                                    sb.append("Description : ").append(commonUnitData.contents).append("\n")
                                    sb.append("Permission Name : ").append(commonUnitData.nativePermissionName).append("\n\n")
                                }

                                val builder = AlertDialog.Builder(activity)
                                builder.setTitle(popupTitle)
                                builder.setMessage(sb.toString())
                                builder.setPositiveButton("OK") { dialog, which -> }
                                builder.create().show()
                            }
                        } else {
                            addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()))
                        }
                    }
                })

            }


            R.id.btnAuthInitialize -> {

                addLog("\n\n=== HIVE SDK 초기화 요청 ===\n")
                addLog("Auth.initialize(authInitializeListener, engagementListener) Called\n")

                com.hive.Auth.initialize(object : com.hive.Auth.AuthInitializeListener {

                    override fun onAuthInitialize(result: ResultAPI, authInitResult: AuthInitResult?) {

                        addLog(String.format("AuthInitializeListener.onAuthInitialize() Callback\nresult = %s\n", result.toString()))

                        if (result.needExit()) {
                            //							System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        } else {
                            if (authInitResult != null)
                                addLog(String.format("authInitResult = %s\n", authInitResult.toString()))
                        }


                        // HIVE SDK 초기화 결과에 따라서 로그인 선택 스피너 UI 변경
                        if (result.isSuccess) {
                            val spinner = this@AuthTestView.view.findViewById<View>(R.id.spLoginType) as Spinner
                            spinner.setSelection(authInitResult!!.loginType.ordinal)
                        }

                        val msg = result.toString() + if (authInitResult != null) "\nLoginType : " + authInitResult.loginType else ""
                        showAlertDialog("Auth Initialize Result", msg)
                    }
                })

            }


            R.id.btnAuthGetLoginType -> {

                val loginType = com.hive.Auth.loginType

                if (loginType == null) {
                    addLog("\n\n=== 로그인 가능 형태 조회 (GUEST, ACCOUNT, AUTO, SELECT) ===\nAuth.getLoginType() Called\nloginType is null, need initialize")

                    showAlertDialog("Auth getLoginType Failed", "loginType is null, need initialize")
                } else {

                    addLog(String.format("\n\n=== 로그인 가능 형태 조회 (GUEST, ACCOUNT, AUTO, SELECT) ===\nAuth.getLoginType() Called\nloginType = %s\n", loginType.name))
                    val spinner = this.view.findViewById<View>(R.id.spLoginType) as Spinner
                    spinner.setSelection(loginType.ordinal)

                    val toast = Toast.makeText(Configuration.context, "=== getLoginType : $loginType ===", Toast.LENGTH_SHORT)
                    toast.show()
                }

            }


            R.id.btnAuthLogin -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spLoginType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String
                val loginType = LoginType.valueOf(selectedValue.toUpperCase(Locale.US))


                addLog("\n\n=== 선택된 로그인 타입으로 로그인을 요청 ===\n")
                addLog(String.format("Auth.login(loginType = %s, listener) Called\n", loginType.name))

                com.hive.Auth.login(loginType, object : com.hive.Auth.AuthLoginListener {

                    override fun onAuthLogin(result: ResultAPI, loginType: LoginType?, currentAccount: Account?, usedAccount: Account?) {

                        val msg = String.format("AuthLoginListener.onAuthLogin() Callback\nresult = %s\nloginType = %s\ncurrentAccount = %s\nusedAccount = %s\n", result.toString(), loginType!!.name, currentAccount?.toString()
                                ?: "null", usedAccount?.toString() ?: "null")

                        addLog(msg)

                        /// http://its.withhive.com/issues/394 이슈처리. 정상적인 상황이 아닌경우.
                        if (result.isSuccess && loginType === LoginType.SELECT) {
                            recentCurrentAccountVid = currentAccount?.vid
                            recentUsedAccountVid = usedAccount?.vid
                        }


                        showAlertDialog("Auth Login Result", msg)


                        if (result.isSuccess) {
                            GameUser.login(null)
                        }


                    }
                })
            }


            R.id.btnAuthSelectLogin -> {

                recentCurrentAccountVid?.let { recentCurrentAccountVid ->
                    recentUsedAccountVid?.let { recentUsedAccountVid ->

                        // 사용자 선택 창, test
                        val currentVidData = HashMap<String, Any>()
                        currentVidData["vid"] = recentCurrentAccountVid
                        val currentVidGameData = HashMap<String, Any>()
                        currentVidGameData["Name"] = "Nam"
                        currentVidGameData["Level"] = 20

                        currentVidData["data"] = currentVidGameData


                        val usedVidData = HashMap<String, Any>()
                        usedVidData["vid"] = recentUsedAccountVid
                        val usedVidGameData = HashMap<String, Any>()
                        usedVidGameData["Name"] = "An"
                        usedVidGameData["Level"] = 10

                        usedVidData["data"] = usedVidGameData


                        addLog("\n\n=== Auth.login() 메서드를 호출 후 결과 값이 LoginType.SELECT 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n")
                        addLog("Auth.selectLogin(currentVidData, usedVidData, listener) Called\n")
                        addLog(String.format("currentVidData = %s\n", currentVidData.toString()))
                        addLog(String.format("usedVidData = %s\n", usedVidData.toString()))

                        com.hive.Auth.showLoginSelection(currentVidData, usedVidData, object : com.hive.Auth.AuthLoginListener {

                            override fun onAuthLogin(result: ResultAPI, loginType: LoginType?, currentAccount: Account?, usedAccount: Account?) {

                                val msg = String.format("AuthLoginListener.onAuthLogin() Callback\nresult = %s\nloginType = %s\ncurrentAccount = %s\nusedAccount = %s\n", result.toString(), loginType!!.name, currentAccount?.toString()
                                        ?: "null", usedAccount?.toString() ?: "null")

                                addLog(msg)
                                showAlertDialog("Auth SelectLogin Result", msg)


                                if (result.isSuccess) {
                                    GameUser.login(null)
                                }
                            }
                        })

                    }
                }

            }


            R.id.btnAuthLogout -> {

                addLog("\n\n=== 로그 아웃 요청 ===\n")
                addLog("Auth.logout(listener) Called\n")

                com.hive.Auth.logout(object : com.hive.Auth.AuthLogoutListener {

                    override fun onAuthLogout(result: ResultAPI) {

                        addLog(String.format("AuthLogoutListener.onAuthLogout() Callback\nresult = %s\n", result.toString()))

                        showAlertDialog("Auth Logout Result", result.toString())


                        GameUser.logout(null)
                    }
                })
            }


            R.id.btnAuthGetAccount -> {

                val account = com.hive.Auth.account

                addLog("\n\n=== HIVE 인증된 사용자의 정보 조회 ===\n")

                val msg = String.format("Auth.getAccount() Called\naccount = %s\n", account?.toString() ?: "null")
                addLog(msg)
                showAlertDialog("Auth Logout Result", msg)
            }


            R.id.btnAuthCheckMaintenance -> {

                addLog("\n\n=== 점검 상태 표시 및 데이터 반환 ===\n")
                addLog("Auth.checkMaintenance() Called\n")

                com.hive.Auth.checkMaintenance(true, object : com.hive.Auth.AuthMaintenanceListener {

                    override fun onAuthMaintenance(result: ResultAPI, authMaintenanceInfo: AuthMaintenanceInfo?) {

                        addLog(String.format("AuthMaintenanceListener.onAuthMaintenance() Callback\nresult = %s\nauthMaintenanceInfo = %s\n", result.toString(), authMaintenanceInfo?.toString()
                                ?: "null"))


                        if (result.needExit()) {
                            //							System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        }

                        var msg: String? = null

                        if (result.isSuccess && authMaintenanceInfo != null) {
                        } else if (result.isSuccess && authMaintenanceInfo == null) {
                            msg = "$result\n공지가 없습니다."
                        } else {
                            msg = result.toString()
                        }

                        showAlertDialog("Auth CheckMaintenance Result", msg)
                    }
                })

            }


            R.id.btnAuthShowTerms -> {

                addLog("\n\n=== 약관 정보 표시 ===\n")
                addLog(String.format("Auth.showTerms(listener) Called\n"))

                com.hive.Auth.showTerms(object : com.hive.Auth.AuthShowTermsListener {

                    override fun onAuthShowTerms(result: ResultAPI) {
                        addLog(String.format("AuthShowTermsListener.onAuthShowTerms() Callback\nresult = %s\n", result.toString()))
                    }
                })
            }


            R.id.btnAuthSetEmergencyMode -> {

                addLog("\n\n=== 긴급모드 사용 준비 ===\n")
                addLog("Auth.setEmergencyMode() Called\n")

                val ret = com.hive.Auth.setEmergencyMode()

                val toast = Toast.makeText(Configuration.context, "=== 긴급모드 설정 결과 === : $ret", Toast.LENGTH_SHORT)
                toast.show()

                addLog("\n\n=== 긴급모드 설정 결과 === : $ret\n")

                if (ret)
                    GameUser.login(null)
            }


            R.id.btnAuthReset -> {

                addLog("\n\n=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===\n")
                addLog("Auth.reset() Called\n")

                com.hive.Auth.reset()

                val toast = Toast.makeText(Configuration.context, "=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===", Toast.LENGTH_SHORT)
                toast.show()


                GameUser.logout(null)
            }

            R.id.btnUserEngagementSetEngagementListener -> {

                addLog("\n\n=== Engagement 리스너 등록 ===\n")
                addLog("UserEngagement.setEngagementListener(listener) Called.\n")

                Promotion.setEngagementListener(this.engagementListener)

                addLog("\n\n=== Engagement 리스너 등록 완료 ===\n")
                addLog("UserEngagement.setEngagementListener(listener) complete.")

                val toast = Toast.makeText(Configuration.context,
                        "=== Engagement 리스너 등록 완료 ===", Toast.LENGTH_SHORT)
                toast.show()
            }

            R.id.btnUserEngagementSetEngagementReady -> {

                addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 ===\n")
                addLog("UserEngagement.setEngagementReady() Called.\n")

                val retVal = Promotion.setEngagementReady(true)

                addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 결과 ===\n")
                addLog("UserEngagement.setEngagementReady() result : $retVal")

                val toast = Toast.makeText(Configuration.context,
                        if (retVal.isSuccess)
                            "=== Engagement 작업 수행이 가능함으로 설정 ==="
                        else
                            "=== Engagement 작업 수행이 가능함으로 설정되지 않음 ===", Toast.LENGTH_SHORT)
                toast.show()

            }

            R.id.btnPlatformHelperAuthRequestUserPermissions -> {
                var editText = this.view.findViewById<View>(R.id.editTextAuthPermissionList) as EditText
                var permissionArray = editText.text.toString().split("\n");
                var requests = ArrayList<String>(permissionArray)

                PlatformHelper.requestUserPermissions(requests, object: PlatformHelper.RequestUserPermissionsListener {
                    override fun onRequestUserPermissions(resultApi: ResultAPI, granted: List<String>, denied: List<String>) {
                        addLog("\n\n=== OS 권한동의 팝업 재노출 ===\n")
                        addLog("resultApi :" + resultApi.toString() + "\n")

                        addLog("동의한 권한 목록 : \n")
                        for (name in granted) {
                            addLog(name + '\n')
                        }
                        addLog("\n거절한 권한 목록: \n")
                        for (name in denied) {
                            addLog(name + '\n')
                        }
                    }
                })
            }

            R.id.btnAuthShowAdultConfirm -> {
                addLog("\n\n=== 성인 인증 화면 호출 ===\n")
                addLog(String.format("Auth.showAdultConfirm(listener) Called\n"))

                Auth.showAdultConfirm(object : com.hive.Auth.AuthAdultConfirmListener {
                    override fun onAuthAdultConfirm(result: ResultAPI) {
                        val builder = StringBuilder()
                        builder.append(String.format("Auth.showAdultConfirm() Callback - result = %s\n", result))
                        addLog(builder.toString())
                    }
                })
            }
        }
    }
}



