package com.hive.sample

import android.app.Activity
import android.view.View
import android.widget.ArrayAdapter
import android.widget.Spinner
import android.widget.Toast
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.AuthV4
import com.hive.Configuration
import com.hive.ResultAPI
import com.hive.sample.game.GameUser
import com.hive.sample.uihelper.ViewPagerBase
import java.util.*


/**
 * HIVE SDK 의 AuthV4 (인증) 기능 테스트 뷰
 *
 * @author hife
 *
 * Created by hife on 2017. 4. 11..
 */
class AuthV4HelperTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {


    private var recentCurrentPlayerInfo: AuthV4.PlayerInfo? = AuthV4.PlayerInfo()
    private var recentConflictPlayerInfo: AuthV4.PlayerInfo? = AuthV4.PlayerInfo()

    private var achivementsScore = 0
    private var leaderboardsScore = 0L

    init {


        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)


        val spProviderAdapter: ArrayAdapter<CharSequence>

        this.view.findViewById<View>(R.id.btnAuthV4HelperSetup).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperGetSignInType).setOnClickListener(this)

        this.view.findViewById<View>(R.id.btnAuthV4HelperSyncAccount).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperSignIn).setOnClickListener(this)

        spProviderAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spProviderAdapter.add("GUEST")
        spProviderAdapter.add("HIVE")
        spProviderAdapter.add("FACEBOOK")
        spProviderAdapter.add("GOOGLE")
        spProviderAdapter.add("QQ")
        spProviderAdapter.add("WEIBO")
        spProviderAdapter.add("VK")
        spProviderAdapter.add("WECHAT")
        spProviderAdapter.add("APPLE")
        spProviderAdapter.add("SIGNIN_APPLE")
        spProviderAdapter.add("LINE")
        spProviderAdapter.add("TWITTER")
        spProviderAdapter.add("WEVERSE")
        spProviderAdapter.add("AUTO")

        (this.view.findViewById<View>(R.id.spProviderTypeHelper) as Spinner).adapter = spProviderAdapter

        this.view.findViewById<View>(R.id.btnAuthV4HelperSignOut).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperConnect).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperDisconnect).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperAchievementsReveal).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperAchievementsUnlock).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperAchievementsIncrement).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperShowAchievements).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperLeaderboardsSubmitScore).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperShowLeaderboard).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperShowConflict1).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperShowConflict2).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperResolveConflict).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperSwitchAccount).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4HelperGetIDPList).setOnClickListener(this)

    }


    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {

            R.id.btnAuthV4HelperSetup -> {

                addLog("\n\n=== HIVE SDK 초기화 요청 ===\n")
                addLog("AuthV4.setup(authV4SetupListener, engagementListener) Called\n")

                AuthV4.setup(object : AuthV4.AuthV4SetupListener {
                    override fun onAuthV4Setup(result: ResultAPI, isAutoSignIn: Boolean, did: String?, providerTypeList: ArrayList<AuthV4.ProviderType>?) {

                        addLog(String.format("AuthV4SetupListener.onAuthV4Setup() Callback\nresult = %s\n", result.toString()))
                        addLog(String.format("signInType = %b\n", isAutoSignIn))

                        // HIVE SDK 초기화 결과에 따라서 로그인 선택 스피너 UI 변경
                        if (result.isSuccess) {
                            val spinner = this@AuthV4HelperTestView.view.findViewById<View>(R.id.spProviderTypeHelper) as Spinner
                            spinner.setSelection(if (isAutoSignIn) AuthV4.ProviderType.AUTO.ordinal else AuthV4.ProviderType.GUEST.ordinal)
                        }
                        var typeList = ""
                        if (providerTypeList != null) {
                            for (type in providerTypeList) {
                                typeList = "$typeList\n  $type"
                            }
                        }

                        val msg = (result.toString()
                                + "\nisAutoSignIn: " + isAutoSignIn
                                + "\ndid: " + did
                                + "\ntypeList: " + typeList)

                        showAlertDialog("AuthV4 Setup Result", msg)
                    }
                })

            }


            R.id.btnAuthV4HelperGetSignInType -> {

                val isAutoSignIn = AuthV4.isAutoSignIn()

                addLog(String.format("\n\n=== 현재 사인-인 형태 조회 (GUEST, AUTO) ===\n" + "AuthV4.getSignInType() Called\nsignInType = %s\n", if (isAutoSignIn) "AUTO" else "GUEST"))
                val spinner = this.view.findViewById<View>(R.id.spProviderTypeHelper) as Spinner
                spinner.setSelection(if (isAutoSignIn) AuthV4.ProviderType.AUTO.ordinal else AuthV4.ProviderType.GUEST.ordinal)

                val toast = Toast.makeText(Configuration.context, "=== isAutoSignIns: $isAutoSignIn ===", Toast.LENGTH_SHORT)
                toast.show()
            }

            R.id.btnAuthV4HelperSyncAccount -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderTypeHelper) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    AuthV4.Helper.syncAccount(providerType, object : AuthV4.Helper.AuthV4HelperListener {
                        override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {
                            val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                    + "\n" + result
                                    + "\n" + playerInfo)

                            addLog(msg)

                            showAlertDialog("AuthV4 Helper syncAccount Result", msg)

                            if (result.isSuccess) {
                                GameUser.login(null)
                            }
                        }
                    })
                }

            }


            R.id.btnAuthV4HelperSignIn -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderTypeHelper) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                val providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))

                addLog("\n\n=== 자동으로 사인-인을 요청 ===\n")
                addLog(String.format("AuthV4.Helper.signIn(listener) Called\n"))

                AuthV4.Helper.signIn(object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {
                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result
                                + "\n" + playerInfo)

                        addLog(msg)

                        showAlertDialog("AuthV4 Helper SignIn Result", msg)

                        if (result.isSuccess) {
                            GameUser.login(null)
                        }
                    }
                })
            }


            R.id.btnAuthV4HelperSignOut -> {

                addLog("\n\n=== 사인 아웃 요청 ===\n")
                addLog("AuthV4.Helper.signOut(listener) Called\n")

                AuthV4.Helper.signOut(object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {
                        addLog(String.format("AuthV4HelperListener.onAuthV4Helper() Callback\nresult = %s\n", result.toString()))

                        showAlertDialog("AuthV4 Helper SignOut Result", result.toString())

                        GameUser.logout(null)
                    }
                })
            }


            R.id.btnAuthV4HelperConnect -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderTypeHelper) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    addLog("\n\n=== Provider Connect 요청 ===\n")
                    addLog(String.format("AuthV4.Helper.connect(providerType = %s, listener) Called\n", providerType.name))

//                    AuthV4.connect(providerType, new AuthV4.AuthV4ConnectListener() {
//                        @Override
//                        public void onAuthV4Connect(ResultAPI result, AuthV4.PlayerInfo conflictPlayer) {
//
//                            String msg = "AuthV4ConnectListener.onAuthV4Connect() Callback"
//                                    + "\n" + result.toString()
//                                    + "\nCONFLICT: \n" + conflictPlayer;
//
//                            addLog(msg);
//
//                            showAlertDialog("AuthV4 connect Result", msg);
//
//                            if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && conflictPlayer != null) {
//                                recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
//                                recentConflictPlayerInfo = conflictPlayer;
//                            }
//
//                        }
//                    });

                    AuthV4.Helper.connect(providerType, object : AuthV4.Helper.AuthV4HelperListener {
                        override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                            val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                    + "\n" + result.toString()
                                    + "\nplayerInfo: \n" + playerInfo)

                            addLog(msg)

                            showAlertDialog("AuthV4 Helper connect Result", msg)

                            if (result.code === ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                                recentCurrentPlayerInfo = AuthV4.getPlayerInfo()
                                recentConflictPlayerInfo = playerInfo
                            }
                        }
                    })
                }

            }


            R.id.btnAuthV4HelperDisconnect -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderTypeHelper) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    addLog("\n\n=== Provider Disconnect 요청 ===\n")
                    addLog(String.format("AuthV4.Helper.disconnect(providerType = %s, listener) Called\n", providerType.name))


//                    AuthV4.Helper.disconnect(providerType, new AuthV4.AuthV4DisconnectListener() {
//                        @Override
//                        public void onAuthV4Disconnect(ResultAPI result) {
//
//                            String msg = "AuthV4ConnectListener.onAuthV4Connect() Callback"
//                                    + "\n" + result.toString();
//
//                            addLog(msg);
//
//                            showAlertDialog("AuthV4 connect Result", msg);
//                        }
//                    });

                    AuthV4.Helper.disconnect(providerType, object : AuthV4.Helper.AuthV4HelperListener {
                        override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                            val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                    + "\n" + result.toString())

                            addLog(msg)

                            showAlertDialog("AuthV4 Helper disconnect Result", msg)
                        }
                    })
                }

            }

            R.id.btnAuthV4HelperAchievementsReveal -> {

                addLog("\n\n=== 업적 숨김 해제 요청 ===\n")
                addLog("AuthV4.Helper.achievementsReveal() Called\n")

                AuthV4.Helper.achievementsReveal("CgkIzev2g9MJEAIQAQ", object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val title = "AuthV4HelperAchievementsReveal"
                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString())
                        showAlertDialog(title, msg)
                    }
                })

            }

            R.id.btnAuthV4HelperAchievementsUnlock -> {

                addLog("\n\n=== 업적 획득 요청 ===\n")
                addLog("AuthV4.Helper.achievementsUnlock() Called\n")

                AuthV4.Helper.achievementsUnlock("CgkIzev2g9MJEAIQAw", object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val title = "AuthV4HelperAchievementsUnlock"
                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString())
                        showAlertDialog(title, msg)
                    }
                })
            }

            R.id.btnAuthV4HelperAchievementsIncrement -> {

                addLog("\n\n=== 업적 단계 증가 (+1) / 5 ===\n")
                addLog("AuthV4.Helper.achievementsIncrement() Called\n")

                achivementsScore++

                AuthV4.Helper.achievementsIncrement("CgkIzev2g9MJEAIQAg", 1)
                AuthV4.Helper.achievementsIncrement("CgkIzev2g9MJEAIQBQ", 1)
                AuthV4.Helper.achievementsIncrement("CgkIzev2g9MJEAIQBg", 1, object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val title = "AuthV4HelperAchievementsIncrement"
                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString())
                        showAlertDialog(title, msg)
                    }
                })
            }


            R.id.btnAuthV4HelperShowAchievements -> {

                addLog("\n\n=== Show Achievements 요청 ===\n")
                addLog(String.format("AuthV4.Helper.showAchievements(listener) Called\n"))

                AuthV4.Helper.showAchievements(object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo)

                        addLog(msg)

                        showAlertDialog("AuthV4 Helper showAchievements Result", msg)

                        if (result.code === ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo()
                            recentConflictPlayerInfo = playerInfo
                        }
                    }
                })
            }

            R.id.btnAuthV4HelperLeaderboardsSubmitScore -> {

                addLog("\n\n=== 리더보드 점수 증가 (+10) ===\n")
                addLog("AuthV4.Helper.achievementsIncrement() Called\n")

                leaderboardsScore += 10

                AuthV4.Helper.leaderboardsSubmitScore("CgkIzev2g9MJEAIQBw", leaderboardsScore, object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val title = "AuthV4HelperLeaderboardsSubmitScore"
                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString())
                        showAlertDialog(title, msg)
                    }
                })
            }

            R.id.btnAuthV4HelperShowLeaderboard -> {

                addLog("\n\n=== Show Leaderboard 요청 ===\n")
                addLog(String.format("AuthV4.Helper.showAchievements(listener) Called\n"))

                AuthV4.Helper.showLeaderboard(object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo)

                        addLog(msg)

                        showAlertDialog("AuthV4 Helper showLeaderboards Result", msg)

                        if (result.code === ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo()
                            recentConflictPlayerInfo = playerInfo
                        }
                    }
                })
            }

            R.id.btnAuthV4HelperShowConflict1 -> {

                addLog("\n\n=== Show Conflict 요청 ===\n")
                addLog(String.format("AuthV4.Helper.showConflict(listener) Called\n"))

                AuthV4.Helper.showConflict(object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo)

                        addLog(msg)

                        showAlertDialog("AuthV4 Helper showConflict Result", msg)

                        if (result.code === ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo()
                            recentConflictPlayerInfo = playerInfo
                        }
                    }
                })
            }

            R.id.btnAuthV4HelperShowConflict2 -> {

                val viewInfo = AuthV4.Helper.ConflictSingleViewInfo(recentConflictPlayerInfo!!.playerId)
                viewInfo.setValue("Name", "ConflictPlayer-" + recentConflictPlayerInfo!!.playerId)
                viewInfo.setValue("Level", 10)

                addLog("\n\n=== Show Conflict 요청 ===\n")
                addLog(String.format("AuthV4.Helper.showConflict(currentPlayerData, conflictPlayerData, listener) Called\n"))

                AuthV4.Helper.showConflict(viewInfo, object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo)

                        addLog(msg)

                        showAlertDialog("AuthV4 Helper showConflict Result", msg)

                        if (result.code === ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo()
                            recentConflictPlayerInfo = playerInfo
                        }
                    }
                })
            }


            R.id.btnAuthV4HelperResolveConflict -> {

                addLog("AuthV4.Helper.switchAccount(listener) Called\n")

                AuthV4.Helper.resolveConflict(object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString())

                        addLog(msg)

                        showAlertDialog("AuthV4 Helper Resolve Conflict Result", msg)
                    }
                })

            }


            R.id.btnAuthV4HelperSwitchAccount -> {

                addLog("AuthV4.Helper.switchAccount(listener) Called\n")

                AuthV4.Helper.switchAccount(object : AuthV4.Helper.AuthV4HelperListener {
                    override fun onAuthV4Helper(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString())

                        addLog(msg)

                        showAlertDialog("AuthV4 Helper Switch Account Result", msg)
                    }
                })

            }


            R.id.btnAuthV4HelperGetIDPList -> {

                addLog("\n\n=== IDP List 조회 ===\n")

                val idpList = AuthV4.Helper.getIdpList()

                addLog(String.format("AuthV4.Helper.getIDPList() Called\n"))

                var msg = ""

                for (i in idpList.indices) {
                    msg = msg + idpList.get(i) + "\n"
                }

                addLog(msg)

                showAlertDialog("AuthV4 Get Info Result", msg)
            }
        }
    }
}



