package com.hive.sample

import android.app.Activity
import android.app.AlertDialog
import android.view.View
import android.widget.ArrayAdapter
import android.widget.EditText
import android.widget.Spinner
import android.widget.Toast
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.*
import com.hive.Promotion.EngagementEventState
import com.hive.base.Android
import com.hive.sample.game.GameNetwork
import com.hive.sample.game.GameUser
import com.hive.sample.game.IGameListener
import com.hive.sample.game.Response
import com.hive.sample.uihelper.ViewPagerBase
import com.hive.userengagement.UserEngagementCoupon
import org.json.JSONException
import org.json.JSONObject
import java.util.*


/**
 * HIVE SDK 의 AuthV4 (인증) 기능 테스트 뷰
 *
 * @author hife
 *
 * Created by hife on 2017. 4. 11..
 */
class AuthV4TestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {


    private var recentCurrentPlayerInfo: AuthV4.PlayerInfo? = AuthV4.PlayerInfo()
    private var recentConflictPlayerInfo: AuthV4.PlayerInfo? = AuthV4.PlayerInfo()


    // SDK 가 특정한 조건에서 클라이언트에 개입 (Engagement) 하기 위한 이벤트 리스너
    private val engagementListener = object : Promotion.EngagementListener {

        override fun onEngagement(result: ResultAPI, engagementEventType: Promotion.EngagementEventType, eventState: Promotion.EngagementEventState, param: JSONObject?) {

            addLog(String.format("AuthInitializeListener.onEngagement() Callback\nresult = %s\n", result.toString()))
            addLog(String.format("engagementEventType = %s , %s\n", engagementEventType.name, eventState.name))
            addLog(String.format("param = %s\n", param?.toString() ?: "null"))


            when (engagementEventType) {
                Promotion.EngagementEventType.EVENT_TYPE -> {
                    // Engagement의 전체 시작과 끝.
                    if (eventState === EngagementEventState.BEGIN) {
                        // Engagement의 모든 동작이 수행됨.
                    } else if (eventState === EngagementEventState.FINISH) {
                        // Engagement의 모든 동작이 수행 완료됨.
                    }
                }

                Promotion.EngagementEventType.AUTH_LOGIN_VIEW -> {

                    if (eventState === EngagementEventState.START) {
                        // HIVE 멤버십 로그인 페이지 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // HIVE 멤버십 로그인 수행 완료. (로그인 완료 / 창닫음 수행 이후 전달됨)
                    }
                }

                Promotion.EngagementEventType.SOCIAL_INQUIRY_VIEW -> {
                    if (eventState === EngagementEventState.START) {
                        // 1:1 문의 페이지 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // 1:1 문의 페이지 노출 완료 (1:1문의 창 닫은 이후 전달)
                    }
                }

                Promotion.EngagementEventType.PROMOTION_VIEW -> {
                    if (eventState === EngagementEventState.START) {
                        // Promotion View 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // Promotion View 노출 완료 (Promotion 창 닫은 이후 전달)
                    }

                }

                Promotion.EngagementEventType.COUPON -> {
                    if (eventState === EngagementEventState.START) {
                        // 쿠폰 소모 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // 쿠폰 소모 요청 완료 및 응답 전달
                        var couponConsumeResult: UserEngagementCoupon.CouponConsumeResult? = null
                        var message: String? = null

                        try {
                            couponConsumeResult = UserEngagementCoupon.CouponConsumeResult.getCouponConsumeResult(param!!.getInt("code"))
                            message = param.getString("message")
                        } catch (e: JSONException) {
                            e.printStackTrace()
                        }

                        addLog(String.format("Coupon Code = %s\nCoupon Message = %s\n", couponConsumeResult!!.name, message))
                        val toast = Toast.makeText(Configuration.context, "쿠폰 처리 결과입니다.\nCode : $couponConsumeResult\nmMessage : $message", Toast.LENGTH_SHORT)
                        toast.show()
                    }
                }

                Promotion.EngagementEventType.OFFERWALL_VIEW -> {
                    if (eventState === EngagementEventState.START) {
                        // Offerwall 페이지 노출 요청 시작
                    } else if (eventState === EngagementEventState.END) {
                        // Offerwall 페이지 노출 완료 (Offerwall 창 닫은 이후 전달)
                    }
                }

                Promotion.EngagementEventType.EVENT -> {
                    if (eventState === EngagementEventState.START) {
                        // host가 game인 경우 해당 데이터 전달
                    } else if (eventState === EngagementEventState.END) {
                        // host가 game인 경우 해당 데이터 전달
                    }

                    var api: String? = null
                    var parameter: String? = null
                    try {
                        api = param!!.getString("api")
                        parameter = param.getString("param")
                    } catch (e: JSONException) {
                        e.printStackTrace()
                    }

                    addLog(String.format("apiName = %s\napiParam = %s\n", api, parameter))

                    val toast = Toast.makeText(Configuration.context, "이벤트를 전달받았습니다.\nAPI : $api, Param : $parameter", Toast.LENGTH_SHORT)
                    toast.show()
                }


                Promotion.EngagementEventType.IAP_UPDATED -> {
                    if (eventState === EngagementEventState.START) {
                        // 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
                    } else if (eventState === EngagementEventState.END) {
                        // 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
                    }
                    val toast = Toast.makeText(Configuration.context, "IAP Purchase Updated.", Toast.LENGTH_SHORT)
                    toast.show()
                }

                //4.5.0 new engagement Callback parameter.
                Promotion.EngagementEventType.IAP_PROMOTE -> {
                    if (eventState === EngagementEventState.START) {
                        // IAP Promote 데이터 존재 여부 전달
                    } else if (eventState === EngagementEventState.END) {
                        // IAP Promote 데이터 존재 여부 전달
                    }
                }

                Promotion.EngagementEventType.IAP_PURCHASE -> {
                    if (eventState === EngagementEventState.START) {
                        // IAP Purhcase 구매 시작
                    } else if (eventState === EngagementEventState.END) {
                        // IAP Purhcase 구매 완료 (+영수증데이터)
                        var iapV4Receipt: IAPV4.IAPV4Receipt? = null

                        try {
                            iapV4Receipt = param!!.get("iapV4Receipt") as IAPV4.IAPV4Receipt
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }

                        val builder = StringBuilder()
                        builder.append(String.format("IAPV4PurchaseListener.onIAPV4Purchase() Callback - result = %s\n", result))
                                .append("IAPV4Receipt : ").append(iapV4Receipt).append("\n")


                        //                                                forLog.addLog(builder.toString());
                        addLog(builder.toString())

                        if (result.isSuccess && iapV4Receipt != null) {

                            GameNetwork.receiptVerify(iapV4Receipt, object : IGameListener {
                                override fun onGameListener(result: Boolean, response: Response?) {
                                    if (result) {
                                        if (response?.isSuccess == true) {

                                            IAPV4.transactionFinish(iapV4Receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                    Toast.makeText(activity, "Purchase Complete", Toast.LENGTH_SHORT).show()
                                                }
                                            })
                                        }
                                    }
                                }
                            })

                        }
                        showAlertDialog("Purchase Result", result.toString() + "\n" + builder.toString())
                    }
                }
                else -> {
                }
            }
        }
    }


    private val authV4CheckProviderListener = object : AuthV4.AuthV4CheckProviderListener {
        override fun onDeviceProviderInfo(result: ResultAPI, providerInfo: AuthV4.ProviderInfo?) {

            showAlertDialog("AuthV4 onDeviceProviderInfo Result", result.toString() + "\n" + providerInfo)
        }
    }


    init {


        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)


        val spProviderAdapter: ArrayAdapter<String> = ArrayAdapter(activity, android.R.layout.simple_spinner_item)

        this.view.findViewById<View>(R.id.btnAuthV4RequestPermissionViewData).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4Setup).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4GetSignInType).setOnClickListener(this)

        this.view.findViewById<View>(R.id.btnAuthV4SignIn).setOnClickListener(this)

        spProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spProviderAdapter.add("GUEST")
        spProviderAdapter.add("HIVE")
        spProviderAdapter.add("FACEBOOK")
        spProviderAdapter.add("GOOGLE")
        spProviderAdapter.add("QQ")
        spProviderAdapter.add("WEIBO")
        spProviderAdapter.add("VK")
        spProviderAdapter.add("WECHAT")
        spProviderAdapter.add("APPLE")
        spProviderAdapter.add("SIGNIN_APPLE")
        spProviderAdapter.add("LINE")
        spProviderAdapter.add("TWITTER")
        spProviderAdapter.add("WEVERSE")
        spProviderAdapter.add("AUTO")

        (this.view.findViewById<View>(R.id.spProviderType) as Spinner).adapter = spProviderAdapter

        this.view.findViewById<View>(R.id.btnAuthV4SignOut).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4Connect).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4Disconnect).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4SelectConflict).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ResolveConflict).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4GetInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4GetProfile).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowSignIn).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowConflictSelection).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowProfile).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowInquiry).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowMyInquiry).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowChatbotInquiry).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowTerms).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowAdultConfirm).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4CheckProvider).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4CheckMaintenance).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4CheckBlacklist).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4Reset).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnUserEngagementSetEngagementListener).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnUserEngagementSetEngagementReady).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4GetProviderFriendsList).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4GetAgeGateU13).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPlatformHelperAuthV4RequestUserPermissions).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnAuthV4ShowDeviceManagement).setOnClickListener(this)


        AuthV4.setProviderChangedListener(authV4CheckProviderListener)
    }


    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {

            R.id.btnAuthV4RequestPermissionViewData -> {
                addLog("\n\n=== HIVE SDK 권한 고지 팝업 데이터 요청 ===\n");
                addLog("AuthV4.requestPermissionViewData(PermissionViewDataListener) Called\n")

                AuthV4.requestPermissionViewData(object : Permission.PermissionViewDataListener {
                    override fun onPermissionViewData(result: ResultAPI, data: Permission.PermissionViewData?) {
                        if (result.isSuccess && data != null) {
                            if (result.code === ResultAPI.Code.AuthV4SkipPermissionView) {
                                addLog(String.format("Can skip permission notice"))
                            } else {
                                addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()))

                                val popupTitle = data.contents
                                val permissionViewUnitDatas = data.permissions
                                val sb = StringBuilder()
                                for (permissionViewUnitData in permissionViewUnitDatas) {
                                    sb.append("Title : ").append(permissionViewUnitData.title).append("\n")
                                    sb.append("Description : ").append(permissionViewUnitData.contents).append("\n")
                                    sb.append("Permission Name : ").append(permissionViewUnitData.nativePermissionName).append("\n\n")
                                }

                                val commonDatas = data.commons
                                for (commonUnitData in commonDatas) {
                                    sb.append("Title : ").append(commonUnitData.title).append("\n")
                                    sb.append("Description : ").append(commonUnitData.contents).append("\n")
                                    sb.append("Permission Name : ").append(commonUnitData.nativePermissionName).append("\n\n")
                                }

                                val builder = AlertDialog.Builder(activity)
                                builder.setTitle(popupTitle)
                                builder.setMessage(sb.toString())
                                builder.setPositiveButton("OK") { dialog, which -> }
                                builder.create().show()
                            }
                        } else {
                            addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()))
                        }
                    }
                })
            }


            R.id.btnAuthV4Setup -> {

                addLog("\n\n=== HIVE SDK 초기화 요청 ===\n")
                addLog("AuthV4.setup(authV4SetupListener, engagementListener) Called\n")

                AuthV4.setup(object : AuthV4.AuthV4SetupListener {
                    override fun onAuthV4Setup(result: ResultAPI, isAutoSignIn: Boolean, did: String?, providerTypeList: ArrayList<AuthV4.ProviderType>?) {

                        addLog(String.format("AuthV4SetupListener.onAuthV4Setup() Callback\nresult = %s\n", result.toString()))
                        addLog(String.format("signInType = %b\n", isAutoSignIn))

                        if (result.needExit()) {
                            //                            System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        }

                        // HIVE SDK 초기화 결과에 따라서 로그인 선택 스피너 UI 변경
                        if (result.isSuccess) {
                            val spinner = this@AuthV4TestView.view.findViewById<View>(R.id.spProviderType) as Spinner
                            spinner.setSelection(if (isAutoSignIn) AuthV4.ProviderType.AUTO.ordinal else AuthV4.ProviderType.GUEST.ordinal)
                        }

                        var typeList = ""
                        if (providerTypeList != null) {
                            for (type in providerTypeList) {
                                typeList = "$typeList\n  $type"
                            }
                        }

                        val msg = (result.toString()
                                + "\nisAutoSignIn: " + isAutoSignIn
                                + "\ndid: " + did
                                + "\ntypeList: " + typeList)

                        showAlertDialog("AuthV4 Setup Result", msg)
                    }
                })

            }


            R.id.btnAuthV4GetSignInType -> {

                val isAutoSignIn = AuthV4.isAutoSignIn()

                addLog(String.format("\n\n=== 현재 사인-인 형태 조회 (GUEST, AUTO) ===\n" + "AuthV4.getSignInType() Called\nsignInType = %s\n", if (isAutoSignIn) "AUTO" else "GUEST"))
                val spinner = this.view.findViewById<View>(R.id.spProviderType) as Spinner
                spinner.setSelection(if (isAutoSignIn) AuthV4.ProviderType.AUTO.ordinal else AuthV4.ProviderType.GUEST.ordinal)

                val toast = Toast.makeText(Configuration.context, "=== isAutoSignIns: $isAutoSignIn ===", Toast.LENGTH_SHORT)
                toast.show()
            }


            R.id.btnAuthV4SignIn -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    addLog("\n\n=== 자동으로 Guest 혹은 Auto 사인-인을 요청 ===\n")
                    addLog(String.format("AuthV4.signIn(providerType = %s, listener) Called\n", providerType.value))

                    AuthV4.signIn(providerType, object : AuthV4.AuthV4SignInListener {
                        override fun onAuthV4SignIn(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                            val msg = ("AuthV4SignInListener.onAuthV4SignIn() Callback"
                                    + "\n" + result
                                    + "\n" + "playerId: " + playerInfo?.playerId)

                            addLog(msg)

                            if (result.needExit()) {
                                //                            System.exit(0);
                                val builder = AlertDialog.Builder(activity)
                                builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                                builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                                builder.create().show()
                                return
                            }

                            showAlertDialog("AuthV4 SignIn Result", msg)

                            if (result.isSuccess) {
                                GameUser.login(null)
                            }
                        }
                    })
                }

            }


            R.id.btnAuthV4SignOut -> {

                addLog("\n\n=== 사인 아웃 요청 ===\n")
                addLog("AuthV4.signOut(listener) Called\n")

                AuthV4.signOut(object : AuthV4.AuthV4SignOutListener {
                    override fun onAuthV4SignOut(result: ResultAPI) {

                        addLog(String.format("AuthV4SignOutListener.onAuthV4SignOut() Callback\nresult = %s\n", result.toString()))

                        showAlertDialog("AuthV4 SignOut Result", result.toString())

                        GameUser.logout(null)

                    }
                })
            }


            R.id.btnAuthV4Connect -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    addLog("\n\n=== Provider Connect 요청 ===\n")
                    addLog(String.format("AuthV4.connect(providerType = %s, listener) Called\n", providerType.name))

                    AuthV4.connect(providerType, object : AuthV4.AuthV4ConnectListener {
                        override fun onAuthV4Connect(result: ResultAPI, conflictPlayer: AuthV4.PlayerInfo?) {

                            val msg = ("AuthV4ConnectListener.onAuthV4Connect() Callback"
                                    + "\n" + result.toString()
                                    + "\nCONFLICT: \n" + conflictPlayer)

                            addLog(msg)

                            if (result.needExit()) {
                                //                            System.exit(0);
                                val builder = AlertDialog.Builder(activity)
                                builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                                builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                                builder.create().show()
                                return
                            }

                            showAlertDialog("AuthV4 connect Result", msg)

                            if (result.code === ResultAPI.Code.AuthV4ConflictPlayer && conflictPlayer != null) {
                                recentCurrentPlayerInfo = AuthV4.getPlayerInfo()
                                recentConflictPlayerInfo = conflictPlayer
                            }

                        }
                    })
                }

            }


            R.id.btnAuthV4Disconnect -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                val providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    addLog("\n\n=== Provider Disconnect 요청 ===\n")
                    addLog(String.format("AuthV4.disconnect(providerType = %s, listener) Called\n", providerType.name))


                    AuthV4.disconnect(providerType, object : AuthV4.AuthV4DisconnectListener {
                        override fun onAuthV4Disconnect(result: ResultAPI) {

                            val msg = ("AuthV4ConnectListener.onAuthV4Disconnect() Callback"
                                    + "\n" + result.toString())

                            addLog(msg)

                            if (result.needExit()) {
                                //                            System.exit(0);
                                val builder = AlertDialog.Builder(activity)
                                builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                                builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                                builder.create().show()
                                return
                            }

                            showAlertDialog("AuthV4 connect Result", msg)
                        }
                    })
                }

            }


            R.id.btnAuthV4SelectConflict -> {

                addLog("\n\n=== AuthV4.connect() 메서드를 호출 후 결과 값이 ResultAPI.CONFLICT_PLAYER 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n")
                addLog("AuthV4.selectConflict(currentPlayerId, listener) Called\n")
                addLog("currentPlayerId = " + recentConflictPlayerInfo!!.playerId)

                AuthV4.selectConflict(recentConflictPlayerInfo!!.playerId, object : AuthV4.AuthV4SignInListener {
                    override fun onAuthV4SignIn(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result.toString()
                                + "\n" + playerInfo)

                        addLog(msg)

                        if (result.needExit()) {
                            //                            System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        }

                        showAlertDialog("AuthV4 Select Conflict Result", msg)

                        if (result.isSuccess) {
                            GameUser.login(null)
                        }
                    }
                })

            }


            R.id.btnAuthV4ResolveConflict -> {

                addLog("AuthV4.resolveConflict(listener) Called\n")

                AuthV4.resolveConflict(object : AuthV4.AuthV4ResolveConflictListener {
                    override fun onAuthV4ResolveConflict(result: ResultAPI) {

                        val msg = ("AuthV4ResolveConflictListener.onAuthV4ResolveConflict() Callback"
                                + "\n" + result.toString())

                        addLog(msg)

                        showAlertDialog("AuthV4 Resolve Conflict Result", msg)
                    }
                })

            }


            R.id.btnAuthV4GetInfo -> {

                addLog("\n\n=== HIVE 인증된 사용자의 정보 조회 ===\n")

                val playerInfo = AuthV4.getPlayerInfo()

                addLog(String.format("AuthV4.getPlayerInfo() Called\n"))

                val msg = ("AuthV4ConnectListener.onAuthV4Connect() Callback"
                        + "\n" + playerInfo)

                addLog(msg)

                showAlertDialog("AuthV4 Get Info Result", msg)
            }


            R.id.btnAuthV4GetProfile -> {

                addLog("\n\n=== 사용자의 프로필 정보 조회 ===\n")

                val playerInfo = AuthV4.getPlayerInfo()

                if (playerInfo == null) {
                    val msg = "Please Sign-In First."

                    addLog(msg)

                    showAlertDialog("AuthV4 Get Profile Result", msg)
                    return
                }

                addLog(String.format("AuthV4.getProfile() Called\n"))

                val playerIdList = ArrayList<Long>()
                playerIdList.add(playerInfo.playerId)

                AuthV4.getProfile(playerIdList, object : AuthV4.AuthV4GetProfileListener {
                    override fun onAuthV4GetProfile(result: ResultAPI, profileInfoList: ArrayList<AuthV4.ProfileInfo>?) {

                        var info: AuthV4.ProfileInfo? = null
                        if (profileInfoList != null) {
                            info = profileInfoList[0]
                        }


                        val msg = ("AuthV4GetProfileListener.onAuthV4GetProfile() Callback"
                                + "\n" + result + "\n\n" + info)

                        addLog(msg)

                        showAlertDialog("AuthV4 Get Profile Result", msg)
                    }
                })
            }


            R.id.btnAuthV4ShowSignIn -> {

                addLog("\n\n=== HIVE 사인-인 UI 호출 ===\n")
                addLog(String.format("AuthV4.showSignIn(listener) Called\n"))

                AuthV4.showSignIn(object : AuthV4.AuthV4SignInListener {
                    override fun onAuthV4SignIn(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result.toString()
                                + "\n" + playerInfo)

                        addLog(msg)

                        if (result.needExit()) {
                            //                            System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        }

                        if (result.isSuccess) {
                            GameUser.login(null)
                        }

                        showAlertDialog("AuthV4 Show SignIn Result", msg)

                    }
                })
            }


            R.id.btnAuthV4ShowConflictSelection -> {

                // 사용자 선택 창, test
                val currentPlayerData = HashMap<String, Any>()
                currentPlayerData["player_id"] = recentCurrentPlayerInfo!!.playerId
                val currentPlayerGameData = HashMap<String, Any>()
                currentPlayerGameData["Name"] = "CurrentPlayer-" + recentCurrentPlayerInfo!!.playerId
                currentPlayerGameData["Level"] = 20

                currentPlayerData["game_data"] = currentPlayerGameData


                val conflictPlayerData = HashMap<String, Any>()
                conflictPlayerData["player_id"] = recentConflictPlayerInfo!!.playerId
                val conflictPlayerGameData = HashMap<String, Any>()
                conflictPlayerGameData["Name"] = "ConflictPlayer-" + recentConflictPlayerInfo!!.playerId
                conflictPlayerGameData["Level"] = 10

                conflictPlayerData["game_data"] = conflictPlayerGameData


                addLog("\n\n=== AuthV4.connect() 메서드를 호출 후 결과 값이 ResultAPI.CONFLICT_PLAYER 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n")
                addLog("AuthV4.showConflictSelection(currentPlayerData, conflictPlayerData, listener) Called\n")
                addLog(String.format("currentPlayerData = %s\n", currentPlayerData.toString()))
                addLog(String.format("conflictPlayerData = %s\n", conflictPlayerData.toString()))

                AuthV4.showConflictSelection(currentPlayerData, conflictPlayerData, object : AuthV4.AuthV4SignInListener {
                    override fun onAuthV4SignIn(result: ResultAPI, playerInfo: AuthV4.PlayerInfo?) {

                        val msg = ("AuthV4SignInListener.onAuthV4SignIn() Callback"
                                + "\n" + result.toString()
                                + "\n" + playerInfo)

                        addLog(msg)

                        if (result.needExit()) {
                            //                            System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        }

                        showAlertDialog("AuthV4 Show Conflict Selection Result", msg)

                        if (result.isSuccess) {
                            GameUser.login(null)
                        }

                    }
                })
            }


            R.id.btnAuthV4ShowProfile -> {

                addLog("\n\n=== 프로필 정보 표시 ===\n")
                addLog(String.format("AuthV4.showProfile(listener) Called\n"))

                if (AuthV4.getPlayerInfo() != null) {

                    AuthV4.showProfile(AuthV4.getPlayerInfo()!!.playerId, object : AuthV4.AuthV4ShowProfileListener {
                        override fun onAuthV4ShowProfile(result: ResultAPI) {
                            addLog(String.format("AuthV4ShowProfileListener.onAuthV4ShowProfile() Callback\nresult = %s\n", result.toString()))
                            showAlertDialog("AuthV4 Show Profile Result", result.toString())
                        }
                    })

                } else {
                    showAlertDialog("AuthV4 Show Profile Result", "AuthV4 ID is null. Please Sign-In first.")
                }
            }


            R.id.btnAuthV4ShowInquiry -> {

                addLog("\n\n=== 1:1 문의 표시 ===\n")
                addLog(String.format("AuthV4.showInquiry(listener) Called\n"))

                AuthV4.showInquiry(object : AuthV4.AuthV4ShowInquiryListener {
                    override fun onAuthV4ShowInquiry(result: ResultAPI) {
                        addLog(String.format("AuthV4ShowInquiryListener.onAuthV4ShowInquiry() Callback\nresult = %s\n", result.toString()))
                        showAlertDialog("AuthV4 Show Inquiry Result", result.toString())
                    }
                })
            }


            R.id.btnAuthV4ShowMyInquiry -> {

                addLog("\n\n=== 내 문의 표시 ===\n")
                addLog(String.format("AuthV4.showMyInquiry(listener) Called\n"))

                AuthV4.showMyInquiry(object : AuthV4.AuthV4ShowMyInquiryListener {
                    override fun onAuthV4ShowMyInquiry(result: ResultAPI) {
                        addLog(String.format("AuthV4ShowMyInquiryListener.onAuthV4ShowMyInquiry() Callback\nresult = %s\n", result.toString()))
                        showAlertDialog("AuthV4 Show My Inquiry Result", result.toString())
                    }
                })
            }


            R.id.btnAuthV4ShowChatbotInquiry -> {

                addLog("\n\n=== 챗봇 문의 표시 ===\n")
                addLog(String.format("AuthV4.showChatbotInquiry(listener) Called\n"))

                val additionalInfo = JSONObject().apply {
                    this.put("evt_code",1000)
                    this.put("p1","random")
                }

                AuthV4.showChatbotInquiry(additionalInfo.toString(), object : AuthV4.AuthV4ShowChatbotInquiryListener{
                    override fun onAuthV4ShowChatbotInquiry(result: ResultAPI) {
                        addLog(String.format("AuthV4ShowChatbotInquiryListener.onAuthV4ShowChatbotInquiry() Callback\nresult = %s\n", result.toString()))
                        showAlertDialog("AuthV4 Show Chatbot Inquiry Result", result.toString())
                    }
                })
            }


            R.id.btnAuthV4ShowTerms -> {

                addLog("\n\n=== 약관 정보 표시 ===\n")
                addLog(String.format("AuthV4.showTerms(listener) Called\n"))

                AuthV4.showTerms(object : AuthV4.AuthV4ShowTermsListener {
                    override fun onAuthV4ShowTerms(result: ResultAPI) {
                        addLog(String.format("AuthV4ShowTermsListener.onAuthV4ShowTerms() Callback\nresult = %s\n", result.toString()))
                    }
                })
            }


            R.id.btnAuthV4ShowAdultConfirm -> {
                addLog("\n\n=== 성인 인증 화면 호출 ===\n")
                addLog("AuthV4.showAdultConfirm(listener) Called\n")

                AuthV4.showAdultConfirm(object : AuthV4.AuthV4AdultConfirmListener {
                    override fun onAuthV4AdultConfirm(result: ResultAPI) {

                        addLog(String.format("AuthV4AdultConfirmListener.onAuthV4AdultConfirm() Callback\nresult = %s\n", result.toString()))

                        showAlertDialog("AuthV4 Show AdultConfirm Result", result.toString())

                    }
                })
            }


            R.id.btnAuthV4CheckProvider -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    addLog("\n\n=== Check Provider 요청 ===\n")
                    addLog(String.format("AuthV4.checkProvider(providerType = %s, listener) Called\n", providerType.name))


                    AuthV4.checkProvider(providerType, object : AuthV4.AuthV4CheckProviderListener {
                        override fun onDeviceProviderInfo(result: ResultAPI, providerInfo: AuthV4.ProviderInfo?) {

                            val msg = ("AuthV4SignInListener.onAuthV4SignIn() Callback"
                                    + "\n" + result.toString()
                                    + "\n" + providerInfo)

                            addLog(msg)

                            showAlertDialog("AuthV4 Check Provider Result", msg)
                        }
                    })
                }

            }


            R.id.btnAuthV4CheckMaintenance -> {

                addLog("\n\n=== 점검 상태 표시 및 데이터 반환 ===\n")
                addLog("AuthV4.checkMaintenance() Called\n")


                AuthV4.checkMaintenance(true, object : AuthV4.AuthV4MaintenanceListener {
                    override fun onAuthV4Maintenance(result: ResultAPI, maintenanceInfo: ArrayList<AuthV4.AuthV4MaintenanceInfo>?) {

                        addLog(String.format("AuthV4MaintenanceListener.onAuthV4Maintenance() Callback\nresult = %s\n", result.toString()))

                        if (result.needExit()) {
                            //                            System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        }

                        var msg = String()

                        if (maintenanceInfo != null && maintenanceInfo.size > 0) {
                            for (authV4MaintenanceInfo in maintenanceInfo) {

                                msg = msg + authV4MaintenanceInfo.toString() + "\n\n"
                            }

                        } else if (result.isSuccess && maintenanceInfo!!.size <= 0) {
                            msg = "$result\n\n  공지가 없습니다."
                        } else {
                            msg = result.toString()
                        }

                        showAlertDialog("Auth Check Maintenance Result", msg)
                    }
                })
            }


            R.id.btnAuthV4CheckBlacklist -> {

                addLog("\n\n=== 제재 상태 표시 및 데이터 반환 ===\n")
                addLog("AuthV4.checkBlacklist() Called\n")

                AuthV4.checkBlacklist(true, object : AuthV4.AuthV4MaintenanceListener {
                    override fun onAuthV4Maintenance(result: ResultAPI, maintenanceInfo: ArrayList<AuthV4.AuthV4MaintenanceInfo>?) {

                        addLog(String.format("AuthV4MaintenanceListener.onAuthV4Maintenance() Callback\nresult = %s\n", result.toString()))

                        if (result.needExit()) {
                            //                            System.exit(0);
                            val builder = AlertDialog.Builder(activity)
                            builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button")
                            builder.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder.create().show()
                            return
                        }

                        var msg = String()

                        if (maintenanceInfo != null && !maintenanceInfo.isEmpty()) {
                            for (authV4MaintenanceInfo in maintenanceInfo) {

                                msg = msg + authV4MaintenanceInfo.toString() + "\n\n"
                            }

                        } else if (result.isSuccess && (maintenanceInfo == null || maintenanceInfo.isEmpty()) ) {
                            msg = "$result\n\n  공지가 없습니다."
                        } else {
                            msg = result.toString()
                        }

                        showAlertDialog("AuthV4 Check Blacklist Result", msg)

                    }
                })

            }


            R.id.btnAuthV4Reset -> {

                addLog("\n\n=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===\n")
                addLog("AuthV4.reset() Called\n")


                AuthV4.reset()


                val toast = Toast.makeText(Configuration.context, "=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===", Toast.LENGTH_SHORT)
                toast.show()


                GameUser.logout(null)
            }

            R.id.btnUserEngagementSetEngagementListener -> {

                addLog("\n\n=== Engagement 리스너 등록 ===\n")
                addLog("UserEngagement.setEngagementListener(listener) Called.\n")

                Promotion.setEngagementListener(this.engagementListener)

                addLog("\n\n=== Engagement 리스너 등록 완료 ===\n")
                addLog("UserEngagement.setEngagementListener(listener) complete.")

                val toast = Toast.makeText(Configuration.context,
                        "=== Engagement 리스너 등록 완료 ===", Toast.LENGTH_SHORT)
                toast.show()
            }


            R.id.btnUserEngagementSetEngagementReady -> {

                addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 ===\n")
                addLog("UserEngagement.setEngagementReady() Called.\n")

                val retVal = Promotion.setEngagementReady(true)

                addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 결과 ===\n")
                addLog("UserEngagement.setEngagementReady() result : $retVal")

                val toast = Toast.makeText(Configuration.context,
                        if (retVal.isSuccess)
                            "=== Engagement 작업 수행이 가능함으로 설정 ==="
                        else
                            "=== Engagement 작업 수행이 가능함으로 설정되지 않음 ===", Toast.LENGTH_SHORT)
                toast.show()

            }

            R.id.btnAuthV4GetProviderFriendsList -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spProviderType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                val providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US))?.let { providerType ->

                    addLog("\n\n=== $selectedValue 친구 조회 결과 ===\n")
                    addLog("AuthV4.getProviderFriendsList() Called\n")


                    AuthV4.getProviderFriendsList(providerType, object : AuthV4.AuthV4ProviderFriendsListener {
                        override fun onGetProviderFriendsList(result: ResultAPI, providerType: AuthV4.ProviderType, providerUserIdList: Map<String, Long>?) {
                            addLog(String.format("AuthV4ProviderFriendsListener.onGetProviderFriendsList() Callback\nresult = %s", result.toString()))
                            addLog(String.format("Provider = %s", providerType))
                            if (result.isSuccess && providerUserIdList != null) {
                                addLog(String.format("friend mathcing ID List = %s\n", providerUserIdList.toString()))
                            }

                            showAlertDialog("AuthV4 getProviderFriendsList Result", "ResultAPI : $result\nProviderUserIdList : ${providerUserIdList?.toString()}")
                        }
                    })
                }

            }

            R.id.btnAuthV4GetAgeGateU13 -> {

                addLog("\n\n=== 13세 나이제한 여부 조회 ===\n")
                addLog("AuthV4.getAgeGateU13() Called\n")

                val ageGateU13 = AuthV4.getAgeGateU13()

                addLog("AuthV4.getAgeGateU13() result : $ageGateU13")

                showAlertDialog("AuthV4 getAgeGateU13 Result", "13세 나이제한 여부 : $ageGateU13")
            }

            R.id.btnPlatformHelperAuthV4RequestUserPermissions -> {
                var editText = this.view.findViewById<View>(R.id.editTextAuthV4PermissionList) as EditText
                var permissionArray = editText.text.toString().split("\n");
                var requests = ArrayList<String>(permissionArray)

                PlatformHelper.requestUserPermissions(requests, object: PlatformHelper.RequestUserPermissionsListener {
                    override fun onRequestUserPermissions(resultApi: ResultAPI, granted: List<String>, denied: List<String>) {
                        addLog("\n\n=== OS 권한동의 팝업 재노출 ===\n")
                        addLog("resultApi :" + resultApi.toString() + "\n")

                        addLog("동의한 권한 목록 : \n")
                        for (name in granted) {
                            addLog(name + '\n')
                        }
                        addLog("\n거절한 권한 목록: \n")
                        for (name in denied) {
                            addLog(name + '\n')
                        }
                    }
                })
            }

            R.id.btnAuthV4ShowDeviceManagement -> {

                addLog("\n\n=== 기기 관리 서비스 표시 ===\n")
                addLog("AuthV4.showDeviceManagement() Called\n")

                AuthV4.showDeviceManagement(object : AuthV4.AuthV4ShowDeviceManagementListener {
                    override fun onAuthV4ShowDeviceManagement(result: ResultAPI) {
                        addLog("AuthV4.showDeviceManagement() result : $result")

                        showAlertDialog("AuthV4 showDeviceManagement", "Show Result : $result")
                    }
                })

            }
        }
    }
}



