package com.hive.sample

import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.*
import com.hive.IAP
import com.hive.IAP.IAPProduct
import com.hive.IAP.IAPShop
import com.hive.Logger
import com.hive.ResultAPI
import com.hive.base.Android
import com.hive.ui.Resource
import com.hive.sample.uihelper.ViewPagerBase
import org.json.JSONObject
import java.util.*


/**
 * HIVE SDK 의 IAP (In App Purchase) 상점 정보를 보여주는 대화 상자
 *
 * @author hife
 */
class IAPShopView(context: Context, iapShop: IAPShop, balance: Int, additionalInfo: String, internal var forLog: ViewPagerBase) : Dialog(context) {

    internal var textview_locationcode_msg: TextView
    internal var textview_shopid_msg: TextView
    internal var textview_shoptype_msg: TextView
    internal var textview_balance_msg: TextView
    internal var imageview_shop_badge: ImageView
    internal var textview_additional_info_msg: TextView

    internal var listview_products: ListView

    internal var listviewadapter_product: ListViewAdapter

    internal var additionalInfo: String? = null


    init {

        if (TextUtils.isEmpty(additionalInfo)) {
            this.additionalInfo = additionalInfo

        } else {
            try {
                val jsonObject = JSONObject()
                jsonObject.put("character_key", additionalInfo)
                this.additionalInfo = jsonObject.toString()

            } catch (e: Exception) {
                e.printStackTrace()
                this.additionalInfo = null
            }

        }

        this.requestWindowFeature(Window.FEATURE_NO_TITLE)
        this.setCancelable(true)

        setContentView(Resource.getLayoutId(context, "test_view_iap_shop"))

        textview_locationcode_msg = findViewById<View>(Resource.getViewId(context, "textview_locationcode_msg")) as TextView
        textview_shopid_msg = findViewById<View>(Resource.getViewId(context, "textview_shopid_msg")) as TextView
        textview_shoptype_msg = findViewById<View>(Resource.getViewId(context, "textview_shoptype_msg")) as TextView
        textview_balance_msg = findViewById<View>(Resource.getViewId(context, "textview_balance_msg")) as TextView
        imageview_shop_badge = findViewById<View>(Resource.getViewId(context, "imageview_shop_badge")) as ImageView
        textview_additional_info_msg = findViewById<View>(Resource.getViewId(context, "textview_additional_info_msg")) as TextView

        listview_products = findViewById<View>(Resource.getViewId(context, "listview_products")) as ListView

        textview_locationcode_msg.text = iapShop.locationCode
        textview_shopid_msg.text = iapShop.shopId.toString()
        textview_shoptype_msg.text = iapShop.shopType
        textview_balance_msg.text = balance.toString()
        textview_additional_info_msg.text = this.additionalInfo

        listviewadapter_product = ListViewAdapter()

        listview_products.adapter = listviewadapter_product

        iapShop.productList?.let {
            for (iapProduct in it) {
                listviewadapter_product.add(iapProduct)
            }
        }

        listview_products.layoutParams = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
    }


    override fun dismiss() {
        //TODO list bitmap recycle

        super.dismiss()
    }


    internal inner class ListViewAdapter : BaseAdapter() {

        private val iapProductList: ArrayList<IAPProduct>

        init {
            iapProductList = ArrayList()
        }

        override fun getCount(): Int {
            return iapProductList.size
        }

        override fun getItem(position: Int): Any {
            return iapProductList[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            var convertView = convertView

            val context = parent.context

            val iapProduct = iapProductList[position]
            val holder: ViewHolder


            if (convertView == null) {

                val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
                convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_iap_product"), parent, false)

                holder = ViewHolder()

                holder.imageview_product = convertView!!.findViewById<View>(Resource.getViewId(context, "imageview_product")) as ImageView
                holder.textview_product_displayprice = convertView.findViewById<View>(Resource.getViewId(context, "textview_product_displayprice")) as TextView
                holder.imageview_product_title = convertView.findViewById<View>(Resource.getViewId(context, "imageview_product_title")) as ImageView
                holder.textview_product_title = convertView.findViewById<View>(Resource.getViewId(context, "textview_product_title")) as TextView
                holder.imageview_product_badge = convertView.findViewById<View>(Resource.getViewId(context, "imageview_product_badge")) as ImageView
                holder.imageview_product_desc = convertView.findViewById<View>(Resource.getViewId(context, "imageview_product_desc")) as ImageView
                holder.textview_product_desc = convertView.findViewById<View>(Resource.getViewId(context, "textview_product_desc")) as TextView
                holder.iapProduct = iapProduct

                convertView.tag = holder
            } else {
                holder = convertView.tag as ViewHolder
            }

            //display price
            holder.textview_product_displayprice?.text = iapProduct.displayPrice

            //title
            val titleBuilder = StringBuilder()
            titleBuilder.append("PID: ").append(iapProduct.gamePid)

            holder.textview_product_title?.text = titleBuilder.toString()


            convertView.setOnClickListener {
                val pid = iapProduct.gamePid

                forLog.addLog("IAP.purchase(pid: " + iapProduct.gamePid + "), listener) Called\n")

                Logger.i("\n\n=== HIVE IAP 상품에 대한 구매를 요청. ===\n")
                Logger.i(String.format("IAP.purchase(pid = %s, onIAPPurchase) Called\n", pid))

                com.hive.IAP.purchase(pid, additionalInfo, object : IAP.IAPPurchaseReceiptListener {
                    override fun onIAPPurchaseReceipt(result: ResultAPI, iapReceipt: IAP.IAPReceipt?) {
                        val builder = StringBuilder()
                        builder.append(String.format("IAPPurchaseListener.onIAPPurchase() Callback - result = %s\n", result))
                        if (iapReceipt != null)
                            builder.append("IAPReceipt : ").append(iapReceipt).append("\n")

                        forLog.addLog(builder.toString())
                        Logger.i(builder.toString())

                        if (result.needExit()) {
                            //								System.exit(0);
                            val builder2 = AlertDialog.Builder(context)
                            builder2.setMessage("Exit Event Occur. App will be terminated when you press OK button")
                            builder2.setPositiveButton("OK") { dialog, which -> Android.finish() }
                            builder2.create().show()
                            return
                        }

                        AlertDialog.Builder(context)
                                .setTitle("Purchase Result")
                                .setMessage(result.toString() + "\n" + builder.toString())
                                .create().show()
                    }
                })
            }


            return convertView
        }

        fun add(iapProduct: IAPProduct) {
            iapProductList.add(iapProduct)
            this.notifyDataSetChanged()
        }

        fun remove(position: Int) {
            iapProductList.removeAt(position)
            this.notifyDataSetChanged()
        }
    }


    internal inner class ViewHolder {

        var imageview_product: ImageView? = null
        var textview_product_displayprice: TextView? = null
        var imageview_product_title: ImageView? = null
        var textview_product_title: TextView? = null
        var imageview_product_badge: ImageView? = null
        var imageview_product_desc: ImageView? = null
        var textview_product_desc: TextView? = null

        var iapProduct: IAPProduct? = null
    }

}
