package com.hive.sample


import android.app.Activity
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.view.View
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.Configuration
import com.hive.IAP
import com.hive.IAP.IAPType
import com.hive.ResultAPI
import com.hive.base.Property
import com.hive.sample.game.GamePostboxDialog
import com.hive.sample.game.GameUser
import com.hive.sample.game.IGameListener
import com.hive.sample.game.Response
import com.hive.sample.uihelper.ViewPagerBase
import java.util.*


/**
 * HIVE SDK 의 IAP (In App Purchase) 기능 테스트 뷰
 *
 * @author hife
 */
class IAPTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {


    init {

        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)

        this.view.findViewById<View>(R.id.btnPostbox).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPInitialize).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPGetShopInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPRestore).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPGetBalance).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPGetBadge).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPShowPayment).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPShowCharge).setOnClickListener(this)

        val iapProperty = Property("iap_property")
        val additionalInfo = iapProperty.getValue("iap.additionalInfo")

        if (!TextUtils.isEmpty(additionalInfo)) {
            val edtAdditionalInfo = this.view.findViewById<View>(R.id.edtAdditionalInfo) as EditText
            edtAdditionalInfo.setText(additionalInfo)
        }
    }


    fun onGameUserUiUpdate() {

        Handler(Looper.getMainLooper()).post {
            val tvServerId = this@IAPTestView.view.findViewById<View>(R.id.tvIapServerId) as TextView
            tvServerId.text = "Server ID: ${Configuration.serverId}"

            val tvIapNickname = this@IAPTestView.view.findViewById<View>(R.id.tvIapNickname) as TextView
            tvIapNickname.text = "Name: ${GameUser.nickname}"

            val tvIapGold = this@IAPTestView.view.findViewById<View>(R.id.tvIapGold) as TextView
            tvIapGold.text = "${GameUser.gold} GOLD"

            val tvIapGem = this@IAPTestView.view.findViewById<View>(R.id.tvIapGem) as TextView
            tvIapGem.text = "${GameUser.gem} GEM"
        }
    }


    override fun onUpdateView() {

        onGameUserUiUpdate()

        super.onUpdateView()
    }


    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {


            R.id.btnPostbox -> {

                if (GameUser.isLogin) {

                    Handler(Looper.getMainLooper()).post {
                        val gamePostboxDialog = GamePostboxDialog(activity, this@IAPTestView)
                        gamePostboxDialog.show()
                    }

                } else {

                    GameUser.login(object : IGameListener {

                        override fun onGameListener(result: Boolean, response: Response?) {

                            if (result && response?.isSuccess == true) {

                                Handler(Looper.getMainLooper()).post {
                                    onGameUserUiUpdate()

                                    val gamePostboxDialog = GamePostboxDialog(activity, this@IAPTestView)
                                    gamePostboxDialog.show()
                                }
                            } else {

                                val msg = "Postbox Error : GameUser Login Failed."
                                Toast.makeText(activity, msg, Toast.LENGTH_LONG).show()
                            }
                        }
                    })
                }

            }


            R.id.btnIAPInitialize -> {
                addLog("\n\n=== IAP 결재 API 초기화 요청 ===\n")
                addLog("IAP.initialize(listener) Called\n")

                com.hive.IAP.initialize(object : IAP.IAPMarketInfoListener {
                    override fun onIAPMarketInfo(result: ResultAPI, iapTypeList: ArrayList<IAPType>?) {

                        addLog(String.format("IAPMarketInfoListener.onIAPMarketInfo() Callback\nresult = %s\niapTypeList = %s\n", result.toString(), iapTypeList?.toString()
                                ?: "null"))

                        val txtSelectedMarket = this@IAPTestView.view.findViewById<View>(R.id.txtSelectedMarket) as TextView

                        if (iapTypeList != null) {
                            val buffer = StringBuffer()
                            for (iapType in iapTypeList) {
                                buffer.append(" " + iapType.name)
                            }
                            txtSelectedMarket.text = buffer.toString()
                        }

                        if (result.isFailure) {
                            showAlertDialog("IAP Initialize Failed", result.toString())
                        }

                    }
                })
            }


            R.id.btnIAPGetShopInfo -> {
                // 인자 설정
                val edtLocationCode = this.view.findViewById<View>(R.id.edtLocationCode) as EditText
                val locationCode = edtLocationCode.text.toString()

                val edtAdditionalInfo = this.view.findViewById<View>(R.id.edtAdditionalInfo) as EditText
                val additionalInfo = edtAdditionalInfo.text.toString()

                val iapProperty = Property("iap_property")
                iapProperty.setValue("iap.additionalInfo", additionalInfo)
                iapProperty.writeProperties()


                addLog("\n\n=== 상점 정보 조회 ===\n")
                addLog(String.format("IAP.getShopInfo(locationCode = %s, listener) Called\n", locationCode))

                com.hive.IAP.getShopInfo(locationCode, object : IAP.IAPShopInfoListener {
                    override fun onIAPShopInfo(result: ResultAPI, iapShop: IAP.IAPShop?, balance: Int) {

                        addLog(String.format("IAPShopInfoListener.onIAPShopInfo() Callback\nresult = %s\niapShop = %s\nbalance = %d\n", result.toString(), iapShop?.toString()
                                ?: "null", balance))

                        if (result.isSuccess && iapShop != null) {

                            val txtSelectedMarket = this@IAPTestView.view.findViewById<View>(R.id.txtSelectedMarket) as TextView

                            var selectedIAPType: IAPType? = null
                            for (iapType in IAPType.values()) {
                                if (iapType.value == IAP.getSelectedMarket()) {
                                    selectedIAPType = iapType
                                }
                            }
                            txtSelectedMarket.text = selectedIAPType?.name

                            Handler(Looper.getMainLooper()).post {
                                val iapShopView = IAPShopView(activity, iapShop, balance, additionalInfo, this@IAPTestView)
                                iapShopView.show()
                            }
                        } else {

                            showAlertDialog("IAP getShopInfo Failed", result.toString())
                        }

                    }
                })
            }


            R.id.btnIAPRestore -> {
                addLog("\n\n=== HIVE IAP 상품에 대한 미지급된 아이템 지급을 요청. ===\n")
                addLog("IAP.restore(listener) Called\n")

                com.hive.IAP.restoreReceipt(object : IAP.IAPRestoreReceiptListener {
                    override fun onIAPRestoreReceipt(result: ResultAPI, receiptList: ArrayList<IAP.IAPReceipt>?) {

                        val builder = StringBuilder()
                        builder.append(String.format("IAPRestoreListener.onIAPRestore() Callback\nresult = %s\n", result.toString()))

                        if (receiptList != null) {
                            for (product in receiptList) {
                                if (product != null)
                                    builder.append("\n").append(product.toString()).append("\n")
                            }
                        }

                        addLog(builder.toString())

                        showAlertDialog("IAP Restore result", result.toString() + builder.toString())

                    }
                })
            }


            R.id.btnIAPGetBalance -> {
                addLog("\n\n=== HIVE 러비 상점일 경우 잔액 정보 조회. ===\n")
                addLog("IAP.getBalance(listener) Called\n")

                com.hive.IAP.getBalanceInfo(object : IAP.IAPBalanceInfoListener {
                    override fun onIAPBalance(result: ResultAPI, balance: Int) {

                        addLog(String.format("IAPBalanceListener.onIAPBalance() Callback\nresult = %s\nbalance = %d\n", result.toString(), balance))

                        showAlertDialog("IAP getBalance result", result.toString() + if (result.isSuccess) "\n잔액 : $balance" else "")

                    }
                })
            }


            R.id.btnIAPShowPayment -> {
                addLog("\n\n=== 러비 상점이나 구글 상점을 선택하기 위한 창을 띄운다. ===\n")
                addLog("IAP.showPayment(listener) Called\n")

                com.hive.IAP.showMarketSelection(object : IAP.IAPMarketInfoListener {
                    override fun onIAPMarketInfo(result: ResultAPI, iapTypeList: ArrayList<IAPType>?) {

                        val log = String.format("IAPMarketInfoListener.onIAPMarketInfo() Callback\nresult = %s\niapTypeList = %s\n",
                                result.toString(), iapTypeList?.toString() ?: "null")

                        addLog(log)

                        val txtSelectedMarket = this@IAPTestView.view.findViewById<View>(R.id.txtSelectedMarket) as TextView

                        if (iapTypeList != null) {
                            val buffer = StringBuffer()
                            for (iapType in iapTypeList) {
                                buffer.append(" " + iapType.name)
                            }

                            txtSelectedMarket.text = buffer.toString()
                        }

                        showAlertDialog("IAP showPayment result", log)

                    }
                })
            }


            R.id.btnIAPShowCharge -> {
                addLog("\n\n=== 러비 충전 페이지 호출. ===\n")
                addLog("IAP.showCharge(listener) Called\n")

                com.hive.IAP.showCharge(object : IAP.IAPBalanceInfoListener {
                    override fun onIAPBalance(result: ResultAPI, balance: Int) {

                        val log = String.format(Locale.US, "IAPBalanceListener.onIAPBalance() Callback\nresult = %s\nbalance = %d\n", result.toString(), balance)

                        addLog(log)

                        showAlertDialog("IAP ShowCharge Result", log)

                    }
                })
            }
        }

    }
}



