package com.hive.sample

import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.graphics.Color
import android.text.Html
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.BaseAdapter
import android.widget.LinearLayout
import android.widget.ListView
import android.widget.TextView
import android.widget.Toast

import com.hive.IAPV4
import com.hive.IAPV4.IAPV4Product
import com.hive.ResultAPI
import com.hive.ui.Resource
import com.hive.sample.game.*
import com.hive.sample.uihelper.ViewPagerBase

import org.json.JSONObject
import java.text.SimpleDateFormat
import java.util.*


/**
 * HIVE SDK 의 IAP (In App Purchase) 상점 정보를 보여주는 대화 상자
 *
 * @author hife
 */
class IAPV4SubscriptionProductInfoView(context: Context, iapV4ProductList: List<IAPV4Product>, balance: Int, additionalInfo: String, internal var forLog: ViewPagerBase) : Dialog(context) {

    internal var textview_balance_msg: TextView
    internal var textview_additional_info_msg: TextView

    internal var additionalInfo: String? = null

    internal var presetSubscriptionGroup = ArrayList<ArrayList<*>>()        //	item type : ArrayList<String>

    init {

        //	HIVE SAMPLE Market PID PRESET
        //	Group 1
        //	sub30
        //	sub10
        //	sub01
        //	sub11
        //
        //	Group 2
        //	sub303
        //	sub103
        //	sub013
        //	sub113
        val group1 = ArrayList<String>()
        group1.add("sub30")
        group1.add("sub10")
        group1.add("sub01")
        group1.add("sub11")
        presetSubscriptionGroup.add(group1)
        val group2 = ArrayList<String>()
        group2.add("sub303")
        group2.add("sub103")
        group2.add("sub013")
        group2.add("sub113")
        presetSubscriptionGroup.add(group2)

        if (TextUtils.isEmpty(additionalInfo)) {
            this.additionalInfo = additionalInfo

        } else {
            try {
                val jsonObject = JSONObject()
                jsonObject.put("character_key", additionalInfo)
                this.additionalInfo = jsonObject.toString()

            } catch (e: Exception) {
                e.printStackTrace()
                this.additionalInfo = null
            }

        }

        this.requestWindowFeature(Window.FEATURE_NO_TITLE)
        this.setCancelable(true)

        setContentView(Resource.getLayoutId(context, "test_view_iapv4_subscription_product_info"))
        val rootLayout = findViewById<View>(Resource.getViewId(context, "iapv4_subscription_product_info_list")) as LinearLayout
        textview_balance_msg = findViewById<View>(Resource.getViewId(context, "iapv4_textview_balance_msg")) as TextView
        textview_additional_info_msg = findViewById<View>(Resource.getViewId(context, "iapv4_textview_additional_info_msg")) as TextView




        textview_balance_msg.text = balance.toString()
        textview_additional_info_msg.text = this.additionalInfo

        val listViewLayoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
        listViewLayoutParams.topMargin = 20
        listViewLayoutParams.bottomMargin = 20

        val listviewadapter_product_group1 = ListViewAdapter()
        val listview_products_group1 = ListView(context)
        listview_products_group1.layoutParams = listViewLayoutParams
        listview_products_group1.setBackgroundColor(Color.GREEN)
        listview_products_group1.adapter = listviewadapter_product_group1
        rootLayout.addView(listview_products_group1)

        val listviewadapter_product_group2 = ListViewAdapter()
        val listview_products_group2 = ListView(context)
        listview_products_group2.layoutParams = listViewLayoutParams
        listview_products_group2.setBackgroundColor(Color.GREEN)
        listview_products_group2.adapter = listviewadapter_product_group2
        rootLayout.addView(listview_products_group2)

        val listviewadapter_product_group3 = ListViewAdapter()
        val listview_products_group3 = ListView(context)
        //(ListView) findViewById(Resource.getViewId(context, "iapv4_listview_products_group1"));
        listview_products_group3.layoutParams = listViewLayoutParams
        listview_products_group3.setBackgroundColor(Color.GREEN)
        listview_products_group3.adapter = listviewadapter_product_group3
        rootLayout.addView(listview_products_group3)
        listview_products_group3.visibility = View.GONE

        //Divide Product Group


        for (iapProduct in iapV4ProductList) {
            if (presetSubscriptionGroup[0].contains(iapProduct.marketPid))
            //	compare first Group
                listviewadapter_product_group1.add(iapProduct)
            else if (presetSubscriptionGroup[1].contains(iapProduct.marketPid))
            //	compare second Group
                listviewadapter_product_group2.add(iapProduct)
            else {
                listviewadapter_product_group3.add(iapProduct)
                listview_products_group3.visibility = View.VISIBLE
            }

        }

    }


    override fun dismiss() {
        //TODO list bitmap recycle

        super.dismiss()
    }


    internal inner class ListViewAdapter : BaseAdapter() {

        private val iapProductList: ArrayList<IAPV4Product>

        init {
            iapProductList = ArrayList()
        }

        override fun getCount(): Int {
            return iapProductList.size
        }

        override fun getItem(position: Int): Any {
            return iapProductList[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            var convertView = convertView

            val context = parent.context

            val iapProduct = iapProductList[position]
            val holder: ViewHolder


            if (convertView == null) {

                val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
                convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_iapv4_subscription_product"), parent, false)

                holder = ViewHolder()

                holder.textview_product_displayprice = convertView!!.findViewById<View>(Resource.getViewId(context, "iapv4_textview_product_displayprice")) as TextView
                holder.textview_product_title = convertView.findViewById<View>(Resource.getViewId(context, "iapv4_textview_product_title")) as TextView
                holder.textview_product_desc = convertView.findViewById<View>(Resource.getViewId(context, "iapv4_textview_product_desc")) as TextView
                holder.iapProduct = iapProduct

                convertView.tag = holder
            } else {
                holder = convertView.tag as ViewHolder
            }

            //display price
            val text = "<strike><font color=\'#757575\'>" + iapProduct.displayOriginalPrice + "</font></strike><br>" + iapProduct.displayPrice
            holder.textview_product_displayprice!!.text = Html.fromHtml(text)


            //title
            val titleBuilder = StringBuilder()
            titleBuilder
                    .append("PID: ").append(iapProduct.marketPid)
                    .append("\n")
                    .append("타이틀: ")
                    .append(iapProduct.title)

            holder.textview_product_title!!.text = titleBuilder.toString()

            //description
            val descriptionBuilder = StringBuilder()
            descriptionBuilder.append("아이템 설명: ").append(iapProduct.description)
            holder.textview_product_desc!!.text = descriptionBuilder.toString()

            //			String oldMarketPid = GameUser.getInstance().subscrptionMarketPid;
            //			if(iapProduct.marketPid.equals(oldMarketPid))
            if (!GameUser.subscrptionMarketPidList.isEmpty() && GameUser.subscrptionMarketPidList.contains(iapProduct.marketPid)) {
                convertView.setBackgroundColor(Color.RED)
                convertView.setOnClickListener(null)
            }
            else {
                convertView.setBackgroundColor(Color.WHITE)

                convertView.setOnClickListener {
                    val pid = iapProduct.marketPid

                    forLog.addLog("IAPV4.purchase(pid: $pid), listener) Called\n")

                    Log.v("sample", "\n\n=== HIVE IAP 구독 상품에 대한 구매를 요청. ===\n")
                    Log.v("sample", String.format("IAP.purchase(pid = %s, additionalInfo = %s, onIAPV4Purchase) Called\n", pid, additionalInfo))

                    var oldMarketPid: String? = null
                    //						if (pid.startsWith("sub")) {
                    //Check ' is purchase pid's group is same group with subscrptionMarketPidList's group ?'
                    // For Set Old Market PID		//	iapProductList : Present Pid Group..

                    Log.v("sample", "\n\nsubscrptionMarketPidList : " + GameUser.subscrptionMarketPidList.toString() + "\n")
                    for (candidatePid in GameUser.subscrptionMarketPidList) {
                        for (product in iapProductList) {
                            Log.v("sample", "\n\niapProductList.product's PID :  : " + product.marketPid + "\n")
                            if (product.marketPid == candidatePid) {
                                oldMarketPid = candidatePid
                            }
                        }
                    }

                    IAPV4.purchaseSubscriptionUpdate(pid, oldMarketPid, additionalInfo, object : IAPV4.IAPV4PurchaseListener {
                        override fun onIAPV4Purchase(result: ResultAPI, iapV4Receipt: IAPV4.IAPV4Receipt?) {
                            val builder = StringBuilder()
                            builder.append(String.format("IAPV4PurchaseListener.onIAPV4Purchase() Callback - result = %s\n", result))
                                    .append("IAPV4Receipt : ").append(iapV4Receipt).append("\n")

                            forLog.addLog(builder.toString())
                            Log.v("sample", builder.toString())

                            if (result.isSuccess && iapV4Receipt != null) {

                                GameNetwork.receiptVerify(iapV4Receipt, object : IGameListener {

                                    override fun onGameListener(result: Boolean, response: Response?) {

                                        if (result) {

                                            if (response?.isSuccess == true) {

                                                IAPV4.transactionFinish(iapV4Receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                    override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                        Toast.makeText(context, "Purchase Complete", Toast.LENGTH_SHORT).show()
                                                    }
                                                })

                                            } else {

                                                AlertDialog.Builder(context)
                                                        .setTitle("Purchase Result")
                                                        .setMessage("Game Server Response Error\n" + response!!.resultMsg)
                                                        .setPositiveButton("OK") { dialog, which -> }
                                                        .setNegativeButton("Transaction Finish") { dialog, which ->
                                                            IAPV4.transactionFinish(iapV4Receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                                override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                                    Toast.makeText(context, "Purchase Complete", Toast.LENGTH_SHORT).show()
                                                                }
                                                            })
                                                        }
                                                        .create().show()
                                            }

                                        }
                                    }
                                })
                            }
                            AlertDialog.Builder(context)
                                    .setTitle("Purchase Result")
                                    .setMessage(result.toString() + "\n" + builder.toString())
                                    .create().show()
                        }
                    })
                    //						}
                }
            }

            return convertView
        }

        fun add(iapProduct: IAPV4Product) {
            iapProductList.add(iapProduct)
            this.notifyDataSetChanged()
        }

        fun remove(position: Int) {
            iapProductList.removeAt(position)
            this.notifyDataSetChanged()
        }
    }


    internal inner class ViewHolder {

        var textview_product_displayprice: TextView? = null
        var textview_product_title: TextView? = null
        var textview_product_desc: TextView? = null

        var iapProduct: IAPV4Product? = null
    }

}
