package com.hive.sample


import android.app.Activity
import android.app.AlertDialog
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.view.View
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.Configuration
import com.hive.IAP
import com.hive.IAPV4
import com.hive.IAPV4.IAPV4Type
import com.hive.ResultAPI
import com.hive.base.Property
import com.hive.sample.game.*
import com.hive.sample.uihelper.ViewPagerBase
import java.util.*
import kotlin.collections.ArrayList





/**
 * HIVE SDK 의 IAP (In App Purchase) 기능 테스트 뷰
 *
 * @author hife
 */
class IAPV4TestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {

    private val gamePostList = ArrayList<Post>()


    init {

        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)

        this.view.findViewById<View>(R.id.btnIAPV4Postbox).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4MarketConnect).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4GetProductInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4Restore).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4GetSubscriptionProductInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4RestoreSubscription).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4GetBalance).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4ShowPayment).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4ShowCharge).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnIAPV4CheckPromote).setOnClickListener(this)
        val iapProperty = Property("iapv4_property")
        val additionalInfo = iapProperty.getValue("iapv4.additionalInfo")

        if (!TextUtils.isEmpty(additionalInfo)) {
            val edtAdditionalInfo = this.view.findViewById<View>(R.id.edtIAPV4AdditionalInfo) as EditText
            edtAdditionalInfo.setText(additionalInfo)
        }
    }


    fun onGameUserUiUpdate() {

        Handler(Looper.getMainLooper()).post {
            val tvServerId = this@IAPV4TestView.view.findViewById<View>(R.id.tvIapV4ServerId) as TextView
            tvServerId.text = "Server ID: ${Configuration.serverId}"

            val tvIapNickname = this@IAPV4TestView.view.findViewById<View>(R.id.tvIapV4Nickname) as TextView
            tvIapNickname.text = "Name: ${GameUser.nickname}"

            val tvIapGold = this@IAPV4TestView.view.findViewById<View>(R.id.tvIapV4Gold) as TextView
            tvIapGold.text = "${GameUser.gold} GOLD"

            val tvIapGem = this@IAPV4TestView.view.findViewById<View>(R.id.tvIapV4Gem) as TextView
            tvIapGem.text = "${GameUser.gem} GEM"

            val tvIapSuvscription = this@IAPV4TestView.view.findViewById<View>(R.id.tvIapV4Subscription) as TextView
            tvIapSuvscription.text = "구독 여부 : " + !TextUtils.isEmpty(GameUser.subscrptionMarketPid)

        }
    }


    override fun onUpdateView() {

        onGameUserUiUpdate()

        super.onUpdateView()
    }

    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {


            R.id.btnIAPV4Postbox -> {
                Handler(Looper.getMainLooper()).post {
                    val gamePostboxDialog = GamePostboxDialog(activity, this@IAPV4TestView)
                    gamePostboxDialog.show()

//                    val gamePostboxDialog = GamePostboxDialog(activity, this@IAPV4TestView)
//                    gamePostboxDialog.show()
                }
            }


            R.id.btnIAPV4MarketConnect -> {
                addLog("\n\n=== IAP 결재 API 초기화 요청 ===\n")
                addLog("IAPV4.marketConnect(listener) Called\n")

                IAPV4.marketConnect(object : IAPV4.IAPV4MarketInfoListener {

                    override fun onIAPV4MarketInfo(result: ResultAPI, iapV4TypeList: ArrayList<IAPV4Type>?) {

                        addLog(String.format("IAPV4MarketInfoListener.onIAPV4MarketInfo() Callback\nresult = %s\niapTypeList = %s\n", result.toString(), iapV4TypeList?.toString()
                                ?: "null"))

                        val txtSelectedMarket = this@IAPV4TestView.view.findViewById<View>(R.id.txtIAPV4SelectedMarket) as TextView

                        if (iapV4TypeList != null) {
                            val buffer = StringBuffer()
                            for (iapType in iapV4TypeList) {
                                buffer.append(" " + iapType.name)
                            }
                            txtSelectedMarket.text = buffer.toString()
                        }

                        if (result.isFailure) {
                            showAlertDialog("IAPV4 marketConnect Failed", result.toString())
                        }
                    }

                })
            }


            R.id.btnIAPV4GetProductInfo -> {
                // 인자 설정
                val edtAdditionalInfo = this.view.findViewById<View>(R.id.edtIAPV4AdditionalInfo) as EditText
                val additionalInfo = edtAdditionalInfo.text.toString()

                val iapProperty = Property("iapv4_property")
                iapProperty.setValue("iapv4.additionalInfo", additionalInfo)
                iapProperty.writeProperties()


                addLog("\n\n=== 상점 정보 조회 ===\n")
                addLog(String.format("IAPV4.getProductInfo(listener) Called\n"))

                IAPV4.getProductInfo(object : IAPV4.IAPV4ProductInfoListener {

                    override fun onIAPV4ProductInfo(result: ResultAPI, iapV4ProductList: ArrayList<IAPV4.IAPV4Product>?, balance: Int) {

                        addLog(String.format("IAPV4ProductInfoListener.onIAPV4ProductInfo() Callback\nresult = %s\nbalance = %d\n", result.toString(), balance))

                        if (result.isSuccess && iapV4ProductList != null) {

                            val txtSelectedMarket = this@IAPV4TestView.view.findViewById<View>(R.id.txtIAPV4SelectedMarket) as TextView

                            txtSelectedMarket.text = IAPV4.getSelectedMarket().name

                            Handler(Looper.getMainLooper()).post {
                                val iapProductView = IAPV4ProductInfoView(activity, iapV4ProductList, balance, additionalInfo, this@IAPV4TestView)
                                iapProductView.show()
                            }
                        } else {

                            showAlertDialog("IAPV4 getProductInfo Failed", result.toString())
                        }

                    }
                })
            }


            R.id.btnIAPV4Restore -> {
                addLog("\n\n=== HIVE IAPV4 상품에 대한 미지급된 아이템 지급을 요청. ===\n")
                addLog("IAPV4.restore(listener) Called\n")

                IAPV4.restore(object : IAPV4.IAPV4RestoreListener {

                    override fun onIAPV4Restore(result: ResultAPI, iapv4ReceiptList: ArrayList<IAPV4.IAPV4Receipt>?) {

                        val builder = StringBuilder()
                        builder.append(String.format("IAPV4RestoreListener.onIAPV4Restore() Callback\nresult = %s\n", result.toString()))

                        if (result.isSuccess && iapv4ReceiptList != null) {

                            builder.append("\n" + iapv4ReceiptList.size + " item restored.\n\n")

//                            val marketPidList = ArrayList<String>()

                            for (receipt in iapv4ReceiptList) {
                                if (receipt != null) {
                                    builder.append("\n").append(receipt.toString()).append("\n\n------------\n\n")

//                                    marketPidList.add(receipt.product.marketPid)


                                    GameNetwork.receiptVerify(receipt, object : IGameListener{

                                        override fun onGameListener(result: Boolean, response: Response?) {

                                            if (result) {

                                                if (response?.isSuccess == true) {

                                                    IAPV4.transactionFinish(receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                        override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                            Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show()
                                                        }
                                                    })

                                                } else {

                                                    AlertDialog.Builder(activity)
                                                            .setTitle("Purchase Result")
                                                            .setMessage("Game Server Response Error\n" + response!!.resultMsg)
                                                            .setPositiveButton("OK") { dialog, which -> }
                                                            .setNegativeButton("Transaction Finish") { dialog, which ->
                                                                IAPV4.transactionFinish(receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                                    override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                                        Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show()
                                                                    }
                                                                })
                                                            }
                                                            .create().show()
                                                }

                                            }
                                        }
                                    })
                                }
                            }

//							IAPV4.transactionMultiFinish(marketPidList, new IAPV4.IAPV4TransactionMultiFinishListener() {
//								@Override
//								public void onIAPV4TransactionMultiFinish(List<ResultAPI> resultList, List<String> marketPidList) {
//
//									for (int i = 0; i < resultList.size(); i++) {
//										Logger.log("transactionFinish : " + resultList.get(i));
//									}
//								}
//							});
                        }

                        addLog(builder.toString())

                        showAlertDialog("IAPV4 Restore result", result.toString() + builder.toString())
                    }
                })
            }


            R.id.btnIAPV4GetSubscriptionProductInfo -> {
                // 인자 설정
                val edtAdditionalInfo = this.view.findViewById<View>(R.id.edtIAPV4AdditionalInfo) as EditText
                val additionalInfo = edtAdditionalInfo.text.toString()

                val iapProperty = Property("iapv4_property")
                iapProperty.setValue("iapv4.additionalInfo", additionalInfo)
                iapProperty.writeProperties()


                addLog("\n\n=== 구독형 상점 정보 조회 ===\n")
                addLog(String.format("IAPV4.getSubscriptionProductInfo(listener) Called\n"))

                IAPV4.getSubscriptionProductInfo(object : IAPV4.IAPV4ProductInfoListener {

                    override fun onIAPV4ProductInfo(result: ResultAPI, iapV4ProductList: ArrayList<IAPV4.IAPV4Product>?, balance: Int) {

                        addLog(String.format("IAPV4ProductInfoListener.getSubscriptionProductInfo() Callback\nresult = %s\nbalance = %d\n", result.toString(), balance))

                        if (result.isSuccess && iapV4ProductList != null) {

                            val txtSelectedMarket = this@IAPV4TestView.view.findViewById<View>(R.id.txtIAPV4SelectedMarket) as TextView

                            txtSelectedMarket.text = IAPV4.getSelectedMarket().name

                            Handler(Looper.getMainLooper()).post {
                                val iapProductView = IAPV4ProductInfoView(activity, iapV4ProductList, balance, additionalInfo, this@IAPV4TestView)
                                iapProductView.show()
                            }
                        } else {

                            showAlertDialog("IAPV4 getSubscriptionProductInfo Failed", result.toString())
                        }

                    }
                })
            }


            R.id.btnIAPV4RestoreSubscription -> {
                addLog("\n\n=== HIVE IAPV4 구독형 상품에 대한 미지급된 아이템 지급을 요청. ===\n")
                addLog("IAPV4.restoreSubscription(listener) Called\n")

                IAPV4.restoreSubscription(object : IAPV4.IAPV4RestoreListener {

                    override fun onIAPV4Restore(result: ResultAPI, iapv4ReceiptList: ArrayList<IAPV4.IAPV4Receipt>?) {

                        val builder = StringBuilder()
                        builder.append(String.format("IAPV4RestoreListener.restoreSubscription() Callback\nresult = %s\n", result.toString()))

                        if (result.isSuccess && iapv4ReceiptList != null) {

                            builder.append("\n" + iapv4ReceiptList.size + " item restored.\n\n")

                            val marketPidList = ArrayList<String>()

                            for (receipt in iapv4ReceiptList) {
                                if (receipt != null) {
                                    builder.append("\n").append(receipt.toString()).append("\n\n------------\n\n")

                                    marketPidList.add(receipt.product.marketPid)


                                    GameNetwork.receiptVerify(receipt, object : IGameListener{

                                        override fun onGameListener(result: Boolean, response: Response?) {

                                            if (result) {

                                                if (response?.isSuccess == true) {

                                                    IAPV4.transactionFinish(receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                        override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                            Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show()
                                                        }
                                                    })

                                                } else {

                                                    AlertDialog.Builder(activity)
                                                            .setTitle("Purchase Result")
                                                            .setMessage("Game Server Response Error\n" + response!!.resultMsg)
                                                            .setPositiveButton("OK") { dialog, which -> }
                                                            .setNegativeButton("Transaction Finish") { dialog, which ->
                                                                IAPV4.transactionFinish(receipt.product.marketPid, object : IAPV4.IAPV4TransactionFinishListener {
                                                                    override fun onIAPV4TransactionFinish(result: ResultAPI, marketPid: String) {

                                                                        Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show()
                                                                    }
                                                                })
                                                            }
                                                            .create().show()
                                                }

                                            }
                                        }
                                    })
                                }
                            }

//							IAPV4.transactionMultiFinish(marketPidList, new IAPV4.IAPV4TransactionMultiFinishListener() {
//								@Override
//								public void onIAPV4TransactionMultiFinish(List<ResultAPI> resultList, List<String> marketPidList) {
//
//									for (int i = 0; i < resultList.size(); i++) {
//										Logger.log("transactionFinish : " + resultList.get(i));
//									}
//								}
//							});
                        }

                        addLog(builder.toString())

                        showAlertDialog("IAPV4 restoreSubscription Restore result", result.toString() + builder.toString())
                    }
                })
            }


            R.id.btnIAPV4GetBalance -> {
                addLog("\n\n=== HIVE 러비 상점일 경우 잔액 정보 조회. ===\n")
                addLog("IAPV4.getBalance(listener) Called\n")

                IAPV4.getBalanceInfo(object : IAPV4.IAPV4BalanceInfoListener {
                    override fun onIAPV4Balance(result: ResultAPI, balance: Int) {

                        addLog(String.format("IAPV4BalanceInfoListener.onIAPV4Balance() Callback\nresult = %s\nbalance = %d\n", result.toString(), balance))

                        showAlertDialog("IAPV4 getBalance result", result.toString() + if (result.isSuccess) "\n잔액 : $balance" else "")
                    }
                })
            }


            R.id.btnIAPV4ShowPayment -> {
                addLog("\n\n=== 러비 상점이나 구글 상점을 선택하기 위한 창을 띄운다. ===\n")
                addLog("IAPV4.showPayment(listener) Called\n")

                IAPV4.showMarketSelection(object : IAPV4.IAPV4MarketInfoListener {

                    override fun onIAPV4MarketInfo(result: ResultAPI, iapV4TypeList: ArrayList<IAPV4Type>?) {

                        val log = String.format("IAPV4MarketInfoListener.onIAPV4MarketInfo() Callback\nresult = %s\niapTypeList = %s\n", result.toString(), iapV4TypeList?.toString()
                                ?: "null")

                        addLog(log)

                        val txtSelectedMarket = this@IAPV4TestView.view.findViewById<View>(R.id.txtIAPV4SelectedMarket) as TextView

                        if (iapV4TypeList != null) {
                            val buffer = StringBuffer()
                            for (iapType in iapV4TypeList) {
                                buffer.append(" " + iapType.name)
                            }

                            txtSelectedMarket.text = buffer.toString()
                        }

                        showAlertDialog("IAPV4 showPayment result", log)
                    }
                })
            }


            R.id.btnIAPV4ShowCharge -> {
                addLog("\n\n=== 러비 충전 페이지 호출. ===\n")
                addLog("IAPV4.showCharge(listener) Called\n")

                IAPV4.showCharge(object : IAPV4.IAPV4BalanceInfoListener {
                    override fun onIAPV4Balance(result: ResultAPI, balance: Int) {

                        val log = String.format(Locale.US, "IAPV4BalanceInfoListener.onIAPV4Balance() Callback\nresult = %s\nbalance = %d\n", result.toString(), balance)

                        addLog(log)

                        showAlertDialog("IAPV4 ShowCharge Result", log)
                    }
                })
            }
            R.id.btnIAPV4CheckPromote -> {
                IAPV4.checkPromotePurchase(object : IAPV4.IAPV4CheckPromotePurchaseListener {
                    override fun onIAPV4CheckPromotePurchase(result: ResultAPI, marketPid: String?) {
                        val log = String.format(Locale.US, "IAPV4CheckPromotePurchaseListener.onIAPV4CheckPromotePurchase() Callback\nresult = %s\nmarketPid = %s\n", result.toString(), marketPid)

                        addLog(log)

                        showAlertDialog("IAPV4 checkPromotePurchase Result", log)
                    }
                })
            }
        }

    }
}
