package com.hive.sample

import android.app.AlertDialog.Builder
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Color
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.os.Handler
import android.os.Looper
import androidx.core.content.FileProvider.getUriForFile
import android.text.TextUtils
import android.util.Log
import android.view.*
import android.widget.*
import android.widget.TableLayout.LayoutParams
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.*
import com.hive.Promotion.AppInvitationCampaign
import com.hive.Promotion.AppInvitationCampaignStage
import com.hive.Promotion.AppInvitationData
import com.hive.platformhelper.PlatformShare
import com.hive.ui.cache.HiveResourceLoader
import com.hive.ui.cache.HiveResourceLoadingListener
import org.json.JSONException
import org.json.JSONObject
import java.io.File
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException
import java.util.*


/**
 * 프로모션의 앱 초대 데이터 조회 (Promotion.getAppInvitationData()) 의 결과를 화면에 보여주기 위한 테스트 대화 상자.
 *
 * 앱 초대 데이터 정보는 앱 자신의 qrCode, inviteLink 정보등을 보함하고 있다.
 *
 * @author hife
 */
class PromotionAppInvitationDataDialog(context: Context, info: AppInvitationData) : Dialog(context) {

    internal var info: AppInvitationData? = null

    internal var imagebutton_qrcode: ImageButton
    internal var textview_commonlinkmsg: TextView
    internal var textview_hivelinkmsg: TextView
    internal var textview_facebooklinkmsg: TextView
    internal var textview_inviteMessage: TextView
    internal var button_uaShare: Button
    internal var listview_each: ListView
    internal var listview_stage: ListView

    internal var eachListView_adapter: ListViewAdapter
    internal var stageListView_adapter: ListViewAdapter

    internal var qrcode: Bitmap? = null
    internal var qrcodeFile: File

    internal var movieFile: File? = null

    //flag
    internal var isFileWriteSuccess = false


    init {

        this.requestWindowFeature(Window.FEATURE_NO_TITLE)

        setContentView(R.layout.test_view_promotion_appinvitation_result)

        imagebutton_qrcode = findViewById<View>(R.id.imagebutton_qrcode) as ImageButton
        textview_commonlinkmsg = findViewById<View>(R.id.textview_commonlinkmsg) as TextView
        textview_hivelinkmsg = findViewById<View>(R.id.textview_hivelinkmsg) as TextView
        textview_facebooklinkmsg = findViewById<View>(R.id.textview_facebooklinkmsg) as TextView
        textview_inviteMessage = findViewById<View>(R.id.textview_inviteMessagemsg) as TextView

        button_uaShare = findViewById<Button>(R.id.button_uaShare)
        button_uaShare.setOnClickListener(View.OnClickListener {
            Promotion.showUAShare(textview_inviteMessage.getText().toString(), textview_hivelinkmsg.text.toString(), object : Promotion.PromotionShareListener {
                override fun onPromotionShare(result: ResultAPI) {
                    Log.v("sample", String.format("PromotionShareListener.onPromotionShare() Callback\nresult = %s\n", result.toString()))
                }
            })
        })
        listview_each = findViewById<View>(R.id.listview_each) as ListView
        listview_stage = findViewById<View>(R.id.listview_stage) as ListView

        info.qrcode?.let {
            qrcode = BitmapFactory.decodeByteArray(it, 0, it.size)
        }

        //		qrcodeFile = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES), "qrcode.png");
        qrcodeFile = File(context.getExternalFilesDir(Environment.DIRECTORY_PICTURES), "qrcode.png")

        val listener = object : PlatformHelper.PlatformHelperListener {
            override fun onShare(isSuccess: Boolean?) {

                Logger.d("[UASample] onShare:: isSuccess : " + isSuccess!!)
            }
        }

        qrcode?.let { qrcode ->

            //		File imagePath = new File(Context.getFilesDir(), "images");
            //		File newFile = new File(imagePath, "default_image.jpg");
            val contentUri = getUriForFile(context, context.packageName + ".provider", qrcodeFile)

            Log.w("TESTLog", "contentUri=$contentUri")

            var fos: FileOutputStream? = null
            try {
                fos = FileOutputStream(qrcodeFile)
                qrcode.compress(Bitmap.CompressFormat.PNG, 100, fos)
                isFileWriteSuccess = true
            } catch (e: FileNotFoundException) {
                isFileWriteSuccess = false
                e.printStackTrace()
                Toast.makeText(context, "failed : gen QR code file.", Toast.LENGTH_SHORT).show()
            } finally {
                if (fos != null) try {
                    fos.close()
                } catch (ignore: IOException) {
                }

            }

            imagebutton_qrcode.setImageBitmap(qrcode)
            imagebutton_qrcode.setOnClickListener {
                Logger.d("[UASample] [ResultActivity] imagebutton_qrcode onClicked")

                val mediaPaths = ArrayList<String>()

                if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
                    mediaPaths.add(Uri.fromFile(qrcodeFile).toString())
                } else {
                    mediaPaths.add(contentUri.toString())
                }
                Logger.d("mediaPaths.toString() : $mediaPaths")

                var paths = mediaPaths.toTypedArray<String>()

                Logger.d("paths.toString() : $paths")

                val platformShare = PlatformShare()
                platformShare.shareType = PlatformShare.SHARE_MEDIA
                platformShare.subject = "See this QR Code!!"
                platformShare.media = paths

                PlatformHelper.share(platformShare, listener)
            }
        }

        imagebutton_qrcode.setOnLongClickListener {
            Logger.d("[UASample] [ResultActivity] imagebutton_qrcode onLongClicked")
            if (isFileWriteSuccess) {
                context.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.fromFile(qrcodeFile)))
                Toast.makeText(context, "QACode 이미지가 저장 되었습니다.", Toast.LENGTH_SHORT).show()
            } else {
                Toast.makeText(context, "QACode 이미지가 올바르지 않습니다.", Toast.LENGTH_SHORT).show()
            }
            true
        }

        textview_commonlinkmsg.text = info.inviteCommonLink
        textview_commonlinkmsg.setTextColor(Color.BLUE)
        textview_commonlinkmsg.setOnTouchListener { v, event ->
            if (event.action == MotionEvent.ACTION_UP) {
                Logger.d("[UASample] [ResultActivity] textview_commonlinkmsg onTouch")

                val platformShare = PlatformShare()
                platformShare.shareType = PlatformShare.SHARE_TEXT
                platformShare.subject = info.inviteMessage
                platformShare.text = info.inviteCommonLink

                PlatformHelper.share(platformShare, listener)

                false
            } else
                true
        }

        textview_hivelinkmsg.text = info.inviteHivemsgLink
        textview_hivelinkmsg.setTextColor(Color.BLUE)
        textview_hivelinkmsg.setOnTouchListener { v, event ->
            if (event.action == MotionEvent.ACTION_UP) {
                Logger.d("[UASample] [ResultActivity] textview_hivelinkmsg onTouch")

                val platformShare = PlatformShare()
                platformShare.shareType = PlatformShare.SHARE_TEXT
                platformShare.subject = info.inviteMessage
                platformShare.text = info.inviteHivemsgLink

                PlatformHelper.share(platformShare, listener)

                false
            } else
                true
        }

        textview_facebooklinkmsg.text = info.inviteFacebookLink
        textview_facebooklinkmsg.setTextColor(Color.BLUE)
        textview_facebooklinkmsg.setOnTouchListener { v, event ->
            if (event.action == MotionEvent.ACTION_UP) {
                Logger.d("[UASample] [ResultActivity] textview_facebooklinkmsg onTouch")

                val platformShare = PlatformShare()
                platformShare.shareType = PlatformShare.SHARE_TEXT
                platformShare.subject = info.inviteMessage
                platformShare.text = info.inviteFacebookLink

                PlatformHelper.share(platformShare, listener)

                false
            } else
                true
        }

        textview_inviteMessage.setText(info.inviteMessage)

        eachListView_adapter = ListViewAdapter()
        stageListView_adapter = ListViewAdapter()

        listview_each.adapter = eachListView_adapter
        listview_stage.adapter = stageListView_adapter

        for (campaign in info.eachCampaignList) {
            eachListView_adapter.add(campaign)
        }
        for (campaign in info.stageCampaignList) {
            stageListView_adapter.add(campaign)
        }

        listview_each.layoutParams = LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 100 * info.eachCampaignList.size)
        listview_stage.layoutParams = LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 100 * info.stageCampaignList.size)
    }


    override fun dismiss() {
        //TODO list bitmap recycle

        //		if (qrcodeFile != null) qrcodeFile.delete();
        //		if (qrcode != null) qrcode.recycle();

        super.dismiss()
    }


    private fun startActionView(context: Context, uri: String) {
        val marketIntent = Intent(Intent.ACTION_VIEW, Uri.parse(uri))
        marketIntent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        try {
            Toast.makeText(context, "Landing...", Toast.LENGTH_SHORT).show()
            context.startActivity(marketIntent)
        } catch (ignore: Exception) {
            ignore.printStackTrace()
            Toast.makeText(context, "Landing Exception!", Toast.LENGTH_SHORT).show()
        }

    }

    internal inner class ListViewAdapter : BaseAdapter() {

        private val m_List: ArrayList<AppInvitationCampaign>

        init {
            m_List = ArrayList()
        }

        override fun getCount(): Int {
            return m_List.size
        }

        override fun getItem(position: Int): Any {
            return m_List[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val localConvertView: View
            val context = parent.context

            val campaign = m_List[position]
            val holder: ViewHolder

            if (convertView == null) {
                val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
                localConvertView = inflater.inflate(R.layout.test_view_promotion_appinvitation_campaign, parent, false)

                holder = ViewHolder()
                holder.imageview_img = localConvertView.findViewById<View>(R.id.imageview_img) as ImageView
                holder.textview_title = localConvertView.findViewById<View>(R.id.textview_title) as TextView
                holder.textview_desc = localConvertView.findViewById<View>(R.id.textview_desc) as TextView
                holder.textview_count = localConvertView.findViewById<View>(R.id.textview_count) as TextView

                localConvertView.tag = holder

            } else {
                localConvertView = convertView
                holder = localConvertView.tag as ViewHolder
            }

            HiveResourceLoader.loadResource(campaign.imageUrl, object : HiveResourceLoadingListener {

                override fun onLoadingFailed(url: String, failReason: String) {
                    Logger.w("[UASample] [ResultActivity] onloadingFailed. url: $url, $failReason")
                }

                override fun onLoadingComplete(url: String, data: ByteArray) {
                    Handler(Looper.getMainLooper()).post {
                        val img = BitmapFactory.decodeByteArray(data, 0, data.size)
                        holder.imageview_img!!.setImageBitmap(img)
                    }
                }
            })

            holder.textview_title!!.text = campaign.title
            holder.textview_desc!!.text = campaign.description
            holder.textview_count!!.text = if (campaign is AppInvitationCampaignStage)
                "보상 지급 수 : (" + campaign.count + "/" + campaign.limit + ")\n목표 달성 인원 : (" + campaign.goalCount + "/" + campaign.goalTotal + ")"
            else
                "캠페인 완료 인원 : (" + campaign.count + "/" + campaign.limit + ")"

            val builder = StringBuilder()
            val item = campaign.item
            if (item != null) {
                val itemKeys = item.keys()
                var i = 0
                while (itemKeys.hasNext()) {
                    val key = itemKeys.next()
                    if (i++ != 0) builder.append("\n")
                    builder.append(key).append(" ").append(item.opt(key))
                }
            }
            holder.rewardInfo = builder.toString()

            var vid: String? = ""

            val account = Auth.account
            if (account != null)
                vid = account.vid

            if (TextUtils.equals("레벨10 달성", campaign.title)) {
                holder.isCPA = true
                holder.params["vid"] = vid!!
                holder.params["action"] = "CPA_LEVEL_10"
            } else if (TextUtils.equals("레벨20 달성", campaign.title)) {
                holder.isCPA = true
                holder.params["vid"] = vid!!
                holder.params["action"] = "CPA_LEVEL_20"
            } else {
                holder.isCPA = false
            }

            localConvertView.setOnClickListener {
                val builder = Builder(context)

                builder.setTitle(campaign.title + " 보상")
                builder.setMessage(holder.rewardInfo)
                builder.setInverseBackgroundForced(true)
                builder.setCancelable(true)
                builder.setPositiveButton(android.R.string.ok) { dialog, which -> dialog.cancel() }

                if (holder.isCPA) {
                    builder.setNegativeButton("CPA 완료하기") { dialog, which ->
                        Toast.makeText(context, campaign.title + " CPA 전송...", Toast.LENGTH_SHORT).show()


                        // Map<String, String> params
                        val bodyBuilder = StringBuilder()
                        val iterator = holder.params.entries.iterator()
                        // constructs the POST body using the parameters
                        while (iterator.hasNext()) {
                            val param = iterator.next()
                            bodyBuilder.append(param.key).append('=')
                                    .append(param.value)
                            if (iterator.hasNext()) {
                                bodyBuilder.append('&')
                            }
                        }
                        val httpBody = bodyBuilder.toString()


                        Thread {

                            val coreResponse = com.gcp.hivecore.Network.sync(com.gcp.hivecore.Request("http://test-misample.com2us.net/game/cpa"))

                            val builder = StringBuilder()
                            builder.append("CPA 전송 ")
                            if (coreResponse.isSuccess() && coreResponse.content != null) {
                                try {
                                    val o = JSONObject(String(coreResponse.content!!))
                                    val resultCode = o.getInt("result_code")
                                    val resultMsg = o.optString("result_message")
                                    if (resultCode == 0) {
                                        builder.append("성공 : ").append(resultMsg)
                                    } else {
                                        builder.append("실패 : ").append(resultMsg)
                                    }
                                } catch (ignore: JSONException) {
                                    builder.append("실패 : ").append(ignore.toString())
                                }

                            } else {
                                builder.append("실패 : ").append(coreResponse.result)
                            }

                            Toast.makeText(context, builder.toString(), Toast.LENGTH_LONG).show()

                        }.start()

                    }
                }
                builder.setOnCancelListener { dialog -> dialog.dismiss() }
                builder.create().show()
            }

            return localConvertView
        }

        fun add(campaign: AppInvitationCampaign) {
            m_List.add(campaign)
        }

        fun remove(position: Int) {
            m_List.removeAt(position)
        }

    }


    internal inner class ViewHolder {
        var imageview_img: ImageView? = null
        var textview_title: TextView? = null
        var textview_desc: TextView? = null
        var textview_count: TextView? = null
        var rewardInfo: String? = null
        var isCPA = false
        var params: MutableMap<String, String> = HashMap()
    }


}
