package com.hive.sample

import android.app.Dialog
import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Handler
import android.os.HandlerThread
import android.os.Looper
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.*
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.Promotion
import com.hive.Promotion.PromotionBanner
import com.hive.ResultAPI
import com.hive.ui.cache.HiveResourceLoader
import com.hive.ui.cache.HiveResourceLoadingListener
import com.hive.sample.uihelper.ViewPagerBase
import java.util.*


/**
 * HIVE SDK 의 IAP (In App Purchase) 상점 정보를 보여주는 대화 상자
 *
 * @author hife
 */
class PromotionBannerDialog(context: Context, bannerInfoList: ArrayList<PromotionBanner>, internal var forLog: ViewPagerBase) : Dialog(context) {

    private val listViewBanners: ListView
    private val listViewAdapterBanner: ListViewAdapter

    init {
        this.requestWindowFeature(Window.FEATURE_NO_TITLE)
        this.setCancelable(true)

        setContentView(R.layout.test_view_promotion_banner_info)

        listViewBanners = findViewById(R.id.promotion_listview_banners)

//        val testList: ArrayList<PromotionBanner> = ArrayList()
//
//        for (i in 0..200) {
//            for(banner in bannerInfoList) {
//                testList.add(banner)
//            }
//        }

        listViewAdapterBanner = ListViewAdapter(bannerInfoList)
        listViewBanners.adapter = listViewAdapterBanner
        listViewBanners.layoutParams = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
    }


    override fun dismiss() {
        //TODO list bitmap recycle
        for (bitmap in listViewAdapterBanner.bannerList.values) {
            bitmap.recycle()
        }
        listViewAdapterBanner.adapterThread.quit()

        super.dismiss()
    }


    internal inner class ListViewAdapter(private val bannerInfoList: ArrayList<PromotionBanner>) : BaseAdapter() {

        val adapterThread = HandlerThread("ListViewAdapter")
        val bannerList = mutableMapOf<String, Bitmap>()

        init {
            adapterThread.start()
            for (promotionBanner in bannerInfoList) {
                //banner image
                preLoadImage(promotionBanner.imageUrl)
            }
        }

        override fun getCount(): Int {
            return bannerInfoList.size
        }

        override fun getItem(position: Int): Any {
            return bannerInfoList[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val localConvertView: View

            val promotionBanner = bannerInfoList[position]

            val holder: ViewHolder

            if (convertView == null) {
                localConvertView = LayoutInflater.from(parent.context).inflate(R.layout.test_view_promotion_banner, parent, false) as LinearLayout
                holder = ViewHolder(
                        localConvertView.findViewById(R.id.promotion_textview_banner_desc),
                        localConvertView.findViewById(R.id.promotion_imageview_banner)
                )

                localConvertView.tag = holder

            } else {
                localConvertView = convertView
                holder = localConvertView.tag as ViewHolder
            }

            //banner desc
            val builder = StringBuilder()
            builder
                    .append("PID: ").append(promotionBanner.pid)
                    .append("\n")
                    .append("Start: ").append(promotionBanner.displayStartDate)
                    .append("\n")
                    .append("End: ").append(promotionBanner.displayEndDate)

            holder.textView.text = builder.toString()

            bannerList[promotionBanner.imageUrl]?.let {
                holder.imageView.setImageBitmap(it)
            } ?: HiveResourceLoader.loadResourceFirst(promotionBanner.imageUrl, object : HiveResourceLoadingListener {
                override fun onLoadingFailed(url: String, failReason: String) {
                    // TODO set fail default image
                    println("HiveImageLoader ----------------------------- loading failed: $url / $failReason")
                }
                override fun onLoadingComplete(url: String, data: ByteArray) {
                    println("HiveImageLoader -----------------------------")
                    BitmapFactory.decodeByteArray(data, 0, data.size)?.let {
                        Handler(Looper.getMainLooper()).post {
                            holder.imageView.setImageBitmap(it)
                            this@ListViewAdapter.notifyDataSetChanged()
                        }
                    }
                }
            }, Handler(adapterThread.looper))


            localConvertView.setOnClickListener {

                if ((promotionBanner.interworkData.isEmpty()) || "null".equals(promotionBanner.interworkData)) {
                    //or promotionBanner.typeLink != "interwork"
                    val pid = promotionBanner.pid.toString()

                    forLog.addLog("Promotion.banner(pid: $pid) opened.\n")

                    Log.v("sample", "\n\n=== HIVE Promotion DIRECT Banner. ($pid) ===\n")

                    Promotion.showCustomContents(Promotion.PromotionCustomType.DIRECT, pid, object : Promotion.PromotionViewListener {
                        override fun onPromotionView(result: ResultAPI, promotionEventType: Promotion.PromotionViewResultType) {

                            forLog.addLog(String.format("PromotionViewListener.onPromotionView() Callback\nresult = %s\ntype = %s\n", result.toString(), promotionEventType.toString()))
                        }
                    })
                }
                else {
                    forLog.addLog("interworkData exist. will not show CustomWebView.\n")

                }
            }


            return localConvertView
        }

        fun add(promotionBanner: PromotionBanner) {
            bannerInfoList.add(promotionBanner)
            preLoadImage(promotionBanner.imageUrl)
            this.notifyDataSetChanged()
        }

        fun remove(position: Int) {
            bannerList[bannerInfoList[position].imageUrl]?.recycle()
            bannerInfoList.removeAt(position)
            this.notifyDataSetChanged()
        }

        private fun preLoadImage(url: String) {
            HiveResourceLoader.loadResource(url, object : HiveResourceLoadingListener {

                override fun onLoadingFailed(url: String, failReason: String) {
                    // TODO set fail default image
                }

                override fun onLoadingComplete(url: String, data: ByteArray) {
                    BitmapFactory.decodeByteArray(data, 0, data.size)?.let {
                        bannerList[url] = it
                    }
                }
            }, Handler(adapterThread.looper))
        }

    }

    inner class ViewHolder(val textView: TextView, val imageView: ImageView)
}
