package com.hive.sample

import android.app.Activity
import android.text.Editable
import android.view.View
import android.widget.ArrayAdapter
import android.widget.EditText
import android.widget.Spinner
import android.widget.Toast
import androidx.annotation.StringRes
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.Configuration
import com.hive.Promotion
import com.hive.Promotion.AppInvitationData
import com.hive.Promotion.AppInvitationDataListener
import com.hive.Promotion.PromotionBadge
import com.hive.Promotion.PromotionBadgeInfoListener
import com.hive.Promotion.PromotionCustomType
import com.hive.Promotion.PromotionViewInfo
import com.hive.Promotion.PromotionViewInfoListener
import com.hive.Promotion.PromotionViewType
import com.hive.ResultAPI
import com.hive.sample.uihelper.ViewPagerBase
import com.hive.ui.promotion.news.NewsV2Activity
import org.json.JSONArray
import org.json.JSONObject
import java.util.*


/**
 * HIVE SDK 의 Promotion 기능 테스트 뷰
 *
 * @author ryuvsken
 */
class PromotionTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {


    // 프로모션 뷰의 결과 통지 리스너 - Promotion.showPromotion()
    private val promotionViewListener = object : com.hive.Promotion.PromotionViewListener {

        override fun onPromotionView(result: ResultAPI, type: Promotion.PromotionViewResultType) {

            when (type) {
                Promotion.PromotionViewResultType.OPENED -> addLog(String.format("PromotionViewListener.onPromotionView() Callback\nresult = %s\ntype = %s\n", result.toString(), type.toString()))
                Promotion.PromotionViewResultType.CLOSED -> addLog(String.format("PromotionViewListener.onPromotionView() Callback\nresult = %s\ntype = %s\n", result.toString(), type.toString()))
                Promotion.PromotionViewResultType.START_PLAYBACK -> {
                    addLog(String.format("PromotionViewListener.onPromotionView() Callback\nresult = %s\ntype = %s\n", result.toString(), type.toString()))
                    Toast.makeText(activity, "광고 영상 시작됨.\n게임 볼륨(배경음, 효과음 등) 줄이기", Toast.LENGTH_SHORT).show()
                }
                Promotion.PromotionViewResultType.FINISH_PLAYBACK -> {
                    addLog(String.format("PromotionViewListener.onPromotionView() Callback\nresult = %s\ntype = %s\n", result.toString(), type.toString()))
                    Toast.makeText(activity, "광고 영상 종료됨.\n게임 볼륨(배경음, 효과음 등) 원래대로 복구", Toast.LENGTH_SHORT).show()
                }
                Promotion.PromotionViewResultType.NEED_TO_EXIT -> addLog(String.format("PromotionViewListener.onPromotionView() Callback\nresult = %s\ntype = %s\n", result.toString(), type.toString()))
            }
        }
    }


    private val promotionViewInfoListener = object : PromotionViewInfoListener {

        override fun onReceiveInfo(result: ResultAPI, infos: ArrayList<PromotionViewInfo>?) {

            val sb = StringBuffer()

            if (result.isSuccess && infos != null) {

                sb.append("[")
                for (promotionViewInfo in infos) {
                    sb.append("{$promotionViewInfo}")
                }
                sb.append("]")
            }

            addLog(String.format("PromotionInfoListener.onViewInfoReceive() Callback\nresult = %s\nviewInfo = %s\n", result.toString(), sb.toString()))
        }
    }


    private val promotionBadgeInfoListener = object : PromotionBadgeInfoListener {

        override fun onReceiveInfo(result: ResultAPI, infos: ArrayList<PromotionBadge>?) {

            val sb = StringBuffer()

            if (result.isSuccess && infos != null) {

                for (promotionBadge in infos) {

                    sb.append("[ $promotionBadge] ")
                }
                sb.append("\n")
            }

            addLog(String.format("PromotionInfoListener.onViewInfoReceived() Callback\nresult = %s\npromotionBadges = %s", result.toString(), sb.toString()))
        }
    }


    private val promotionBannerInfoListener = object : Promotion.PromotionBannerInfoListener {

        override fun onReceiveInfo(result: ResultAPI, bannerInfoList: ArrayList<Promotion.PromotionBanner>?) {

            addLog("PromotionBannerInfoListener.onReceiveInfo() Callback\nresult = $result")

            val sb = StringBuffer()

            if (result.isSuccess && bannerInfoList != null) {

                for (promotionBanner in bannerInfoList) {

                    sb.append("[ $promotionBanner] ")
                    sb.append("[ ** INTERWORK DATA **" + promotionBanner.interworkData + "] ")
                }
                sb.append("\n")

                addLog("\nbanners = $sb")

                val promotionBannerDialog = PromotionBannerDialog(activity, bannerInfoList, this@PromotionTestView)
                promotionBannerDialog.show()


            } else {

                showAlertDialog("Promotion getBannerInfo Failed", result.toString())
            }
        }
    }


    private val appInvitationDataListener = object : AppInvitationDataListener {

        override fun onAppInvitationData(result: ResultAPI, info: AppInvitationData?) {

            addLog(String.format("AppInvitationDataListener.onAppInvitationData() Callback\nresult = %s\nappInvitationData = %s\n", result.toString(), info?.toString()
                    ?: "null"))

            if (result.isSuccess && info != null) {

                activity.runOnUiThread {

                    val appInvitationDataView = PromotionAppInvitationDataDialog(activity, info)
                    appInvitationDataView.setOnDismissListener {
                        // not used.
                    }

                    appInvitationDataView.show()
                }
            }

        }
    }

    private val languageMap = LinkedHashMap<String, LanguageDisplayContent>()

    init {

        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)

        var spAdapter: ArrayAdapter<CharSequence>
        var spinner: Spinner
        var edtText: EditText

        //ViewType
        spAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spAdapter.add("news")
        spAdapter.add("notice")
        spAdapter.add("banner")
        spAdapter.add("bannerlegacy")
        spAdapter.add("spot")
        spAdapter.add("board")
        spAdapter.add("view")
        spAdapter.add("direct")

        spinner = this.view.findViewById<View>(R.id.spPromotionViewType) as Spinner
        spinner.adapter = spAdapter


        //CampaignType
        spAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spAdapter.add("event")
        spAdapter.add("notice")

        spinner = this.view.findViewById<View>(R.id.spPromotionCampaignType) as Spinner
        spinner.adapter = spAdapter


        //BannerType
        spAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spAdapter.add("rolling")
        spAdapter.add("great")
        spAdapter.add("small")

        spinner = this.view.findViewById<View>(R.id.spPromotionBannerType) as Spinner
        spinner.adapter = spAdapter


        //isForced
        spAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spAdapter.add("false")
        spAdapter.add("true")

        spinner = this.view.findViewById<View>(R.id.spIsForced) as Spinner
        spinner.adapter = spAdapter


        //GameLanguage for Configuration.updateGameLanguage()
        languageMap["en"] = LanguageDisplayContent("en", Locale.ENGLISH, R.string.hive_language_en)
        languageMap["ko"] = LanguageDisplayContent("ko", Locale.KOREAN, R.string.hive_language_ko)
        languageMap["zh-Hans"] = LanguageDisplayContent("zh-Hans", Locale.SIMPLIFIED_CHINESE, R.string.hive_language_zh_Hans)
        languageMap["zh-Hant"] = LanguageDisplayContent("zh-Hant", Locale.TRADITIONAL_CHINESE, R.string.hive_language_zh_Hant)
        languageMap["ja"] = LanguageDisplayContent("ja", Locale.JAPANESE, R.string.hive_language_ja)
        languageMap["ru"] = LanguageDisplayContent("ru", Locale("ru"), R.string.hive_language_ru)
        languageMap["fr"] = LanguageDisplayContent("fr", Locale.FRENCH, R.string.hive_language_fr)
        languageMap["de"] = LanguageDisplayContent("de", Locale.GERMAN, R.string.hive_language_de)
        languageMap["es"] = LanguageDisplayContent("es", Locale("es"), R.string.hive_language_es)
        languageMap["pt"] = LanguageDisplayContent("pt", Locale("pt"), R.string.hive_language_pt)
        languageMap["id"] = LanguageDisplayContent("id", Locale("id"), R.string.hive_language_id)
        languageMap["th"] = LanguageDisplayContent("th", Locale("th"), R.string.hive_language_th)
        languageMap["vi"] = LanguageDisplayContent("vi", Locale("vi"), R.string.hive_language_vi)// vi_VN
        languageMap["it"] = LanguageDisplayContent("it", Locale.ITALIAN, R.string.hive_language_it)
        languageMap["tr"] = LanguageDisplayContent("tr", Locale("tr"), R.string.hive_language_tr)

        // HiveSDK 의 사용 가능 언어 선택
        val spLanguageAdapter: ArrayAdapter<LanguageDisplayContent> = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        for (content in languageMap.values) {
            spLanguageAdapter.add(content)
        }

        spinner = this.view.findViewById<View>(R.id.spGameLanguage) as Spinner
        spinner.adapter = spLanguageAdapter
        spinner.setSelection(spLanguageAdapter.getPosition(languageMap["en"]))


        edtText = this.view.findViewById<View>(R.id.edtHiveGameServerId) as EditText
        edtText.setText("kr")


        this.view.findViewById<View>(R.id.btnPromotionGetOfferwallState).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionShowPromotion).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionShowNews).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionShowCustomContents).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionShowOfferwall).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionShowReview).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionShowNativeReview).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionExit).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionGetViewInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionGetBadgeInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionGetBannerInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionGetAppInvitationData).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionSetAdditionalInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionShowEngagementTestPage).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionProcessURI).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPromotionGetAppInvitationSenderInfo).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnConfigurationUpdateGameLanguage).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnConfigurationUpdateServerId).setOnClickListener(this)
    }


    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {

            R.id.btnPromotionShowPromotion -> {

                var spinner = this.view.findViewById<View>(R.id.spPromotionViewType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String
                if (PromotionViewType[selectedValue] == null) {
                    return
                }

                val promotionViewType = PromotionViewType.valueOf(selectedValue.toUpperCase(Locale.US))

                spinner = this.view.findViewById<View>(R.id.spIsForced) as Spinner
                val selectedIsForced = spinner.adapter.getItem(spinner.selectedItemPosition) as String


                addLog("\n\n=== 이벤트, 프로모션 화면 노출. ===\n")
                addLog(String.format("Promotion.showPromotion(promotionType = %s, isForced = %s, listener) Called\n", promotionViewType.name, selectedIsForced))

                Promotion.showPromotion(promotionViewType, java.lang.Boolean.valueOf(selectedIsForced), promotionViewListener)

            }

            R.id.btnPromotionShowNews -> {

                val editText = this.view.findViewById<View>(R.id.edtNewsMenu) as EditText
                val strNewsMenu = editText.text.toString()

                addLog("\n\n=== 새소식 특정 탭 노출. ===\n")
                addLog("Promotion.showNews(menu = $strNewsMenu, listener) Called\n")

                Promotion.showNews(strNewsMenu, promotionViewListener)

            }


            R.id.btnPromotionShowCustomContents -> {

                val spinner = this.view.findViewById<View>(R.id.spPromotionViewType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String
                if (PromotionCustomType[selectedValue] == null) {
                    return
                }

                val promotionCustomType = PromotionCustomType.valueOf(selectedValue.toUpperCase(Locale.US))

                val editText = this.view.findViewById<View>(R.id.edtContentKey) as EditText
                val content_key = editText.text.toString()


                addLog("\n\n=== Custom 웹뷰 화면 노출. ===\n")
                addLog(String.format("Promotion.showCustomView(%s, listener) Called\n", content_key))

                Promotion.showCustomContents(promotionCustomType, content_key, promotionViewListener)
            }


            R.id.btnPromotionGetOfferwallState -> {

                val offerwallState = Promotion.offerwallState

                addLog("\n\n=== 게임내 오퍼월(무료 충전소) 노출 가능 여부 조회 (Android only) ===\n")
                addLog(String.format("Promotion.getOfferwallState() Called\nofferwallState = %s\n", offerwallState.name))

            }


            R.id.btnPromotionShowOfferwall -> {

                addLog("\n\n=== 게임내 오퍼월(무료 충전소) 화면 노출. (Android only) ===\n")
                addLog("Promotion.showOfferwall(listener) Called\n")

                Promotion.showOfferwall(promotionViewListener)
            }


            R.id.btnPromotionShowNativeReview -> {


                addLog("\n\n=== 게임 리뷰 팝업 대화 상자 노출. (HIVE SDK 4.10.0+) ===\n")
                addLog("Promotion.showNativeReview() Called\n")

                Promotion.showNativeReview()
            }


            R.id.btnPromotionExit -> {

                addLog("\n\n=== 게임 종료 대화 상자 노출 (Android only) ===\n")
                addLog("Promotion.showExit(listener) Called\n")

                Promotion.showExit(promotionViewListener)
            }


            R.id.btnPromotionGetViewInfo -> {

                val spinner = this.view.findViewById<View>(R.id.spPromotionViewType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String

                if (PromotionCustomType[selectedValue] == null) {
                    return
                }

                val promotionCustomType = PromotionCustomType.valueOf(selectedValue.toUpperCase(Locale.US))

                val editText = this.view.findViewById<View>(R.id.edtContentKey) as EditText
                val content_key = editText.text.toString()

                addLog("\n\n=== viewId에 해당하는 프로모션 컨텐츠 조회. ===\n")
                addLog(String.format("Promotion.getViewInfo(viewId = %s, listener) Called\n", content_key))

                Promotion.getViewInfo(promotionCustomType, content_key, promotionViewInfoListener)
            }


            R.id.btnPromotionGetBadgeInfo -> {

                addLog("\n\n=== 게임내 프로모션 버튼의 부각을 위한 정보 (Badge) 조회 ===\n")
                addLog("Promotion.getBadgeInfo(listener) Called\n")

                Promotion.getBadgeInfo(promotionBadgeInfoListener)
            }


            R.id.btnPromotionGetBannerInfo -> {

                val campaignTypeSpinner = this.view.findViewById<View>(R.id.spPromotionCampaignType) as Spinner
                val selectedCampaignValue = campaignTypeSpinner.adapter.getItem(campaignTypeSpinner.selectedItemPosition) as String

                if (Promotion.PromotionCampaignType[selectedCampaignValue] == null) {
                    addLog("Promotion.getBannerInfo(listener) CampaignType is null\n")
                    return
                }

                val bannerTypeSpinner = this.view.findViewById<View>(R.id.spPromotionBannerType) as Spinner
                val selectedBannerValue = bannerTypeSpinner.adapter.getItem(bannerTypeSpinner.selectedItemPosition) as String

                if (Promotion.PromotionBannerType[selectedBannerValue] == null) {
                    addLog("Promotion.getBannerInfo(listener) BannerType is null\n")
                    return
                }

                val promotionCampaignType = Promotion.PromotionCampaignType.valueOf(selectedCampaignValue.toUpperCase(Locale.US))
                val promotionBannerType = Promotion.PromotionBannerType.valueOf(selectedBannerValue.toUpperCase(Locale.US))

                addLog("\n\n=== 롤링 프로모션 배너 정보 조회 (Banner) 조회 ===\n")
                addLog("Promotion.getBannerInfo(listener) Called\n")

                Promotion.getBannerInfo(promotionCampaignType, promotionBannerType, promotionBannerInfoListener)
            }


            R.id.btnPromotionGetAppInvitationData -> {

                addLog("\n\n=== 앱 초대를 위한 데이터 조회 (qrCode, inviteLink) ===\n")
                addLog("Promotion.getAppInvitationData(listener) Called\n")

                Promotion.getAppInvitationData(appInvitationDataListener)
            }


            R.id.btnPromotionSetAdditionalInfo -> {

                val editText = this.view.findViewById<View>(R.id.edtAdditionalInfo) as EditText

                if(editText.text.toString().isBlank()) {
                    // News V2 선물상자 노출 여부를 구분하는 pid 기본 additionalInfo data 설정
                    val pidJSONArray = JSONArray()
                    NewsV2Activity.bannerLists.forEach { bannerList ->
                        bannerList.value.bannerList.forEach { banner ->
                            pidJSONArray.put(banner.pid)
                        }
                    }
                    val jsonObject = JSONObject()
                    jsonObject.put("complete", pidJSONArray)
                    editText.text = Editable.Factory.getInstance().newEditable(jsonObject.toString())
                }


                val additionalInfo = editText.text.toString()

                addLog("\n\n=== GMS 등을 통해 별도의 가공 없이 게임 서버 API 또는 게임 서버의 중계용 API 등 그에 준하는 API에 전달될 값 설정 ===\n")
                addLog(String.format("Promotion.setAdditionalInfo(additionalInfo = %s) Called\n", additionalInfo))

                Promotion.setAdditionalInfo(additionalInfo)
            }

            R.id.btnPromotionShowEngagementTestPage -> {

                val viewId = "310000"

                addLog("\n\n=== EngagementTestPage 노출. ===\n")
                addLog(String.format("Promotion.showCustomView(%s, listener) Called\n", viewId))

                Promotion.showCustomContents(PromotionCustomType.VIEW, viewId, promotionViewListener)
            }

            R.id.btnPromotionProcessURI -> {
                val uri = "interwork://hive/socialinquery"

                addLog("\n\n=== 인터워크 스키마 커맨드 입력 ===\n")
                addLog(String.format("Promotion.processURI(%s) Called\n", uri))

                val result = Promotion.processURI(uri)

                addLog("result = $result \n")
            }

            R.id.btnPromotionGetAppInvitationSenderInfo -> {

                addLog("\n\n=== UA 초대자 정보 조회 호출 ===\n")
                addLog("Promotion.getAppInvitationSenderInfo(listener) Called\n")

                val result = Promotion.getAppInvitationSenderInfo(object : Promotion.AppInvitationSenderInfoListener {

                    override fun onAppInvitationSenderInfo(resultApi: ResultAPI, appInvitationSenderInfo: Promotion.AppInvitationSenderInfo?) {

                        val result = appInvitationSenderInfo.toString()
                        addLog("resultAPI: $resultApi \n")
                        addLog("result = $result \n")
                        // result 데이터가 null 인 경우 result_code가 101(초대자 정보 없음) 혹은 기타 에러일 수 있으니
                        // 로그 확인을 통해 서버 응답 확인
                        // (https://test-promotion.qpyou.cn/ua/getSenderVid)
                    }
                })
            }

            R.id.btnConfigurationUpdateGameLanguage -> {
                val spinner = this.view.findViewById<View>(R.id.spGameLanguage) as Spinner
                val content = spinner.adapter.getItem(spinner.selectedItemPosition) as LanguageDisplayContent
                val gameLanguage = content.languageCode

                Configuration.updateGameLanguage(gameLanguage)
            }

            R.id.btnConfigurationUpdateServerId -> {
                val editText = this.view.findViewById<View>(R.id.edtHiveGameServerId) as EditText
                val serverId = editText.text.toString()

                Configuration.updateServerId(serverId)
            }


        }
    }

    private inner class LanguageDisplayContent internal constructor(languageCode: String, locale: Locale, @StringRes displayStringId: Int) {

        internal var languageCode = ""
        internal var locale: Locale? = null
        @StringRes
        internal var displayStringId = 0

        init {
            this.languageCode = languageCode
            this.locale = locale
            this.displayStringId = displayStringId
        }

        override fun toString(): String {

            return if (displayStringId != 0)
                activity.resources.getString(displayStringId) + "(" + languageCode + ")"
            else
                languageCode
        }
    }
}



