package com.hive.sample

import android.app.Activity
import android.view.View

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.ProviderGoogle
import com.hive.ResultAPI
import com.hive.sample.uihelper.ViewPagerBase

/**
 * HIVE SDK 의 AuthV4 (인증) Provider 기능 테스트 뷰
 *
 * Created by hife on 2017. 6. 7..
 */

class ProviderTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {

    internal var achievementScore = 0
    internal var leaderboardScore = 0

    init {

        initUi()
    }

    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)

        this.view.findViewById<View>(R.id.btnProviderGoogleAchievementsReveal).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnProviderGoogleAchievementsUnlock).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnProviderGoogleAchievementsIncrement).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnProviderGoogleShowAchievements).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnProviderGoogleLeaderboardsSubmitScore).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnProviderGoogleShowLeaderboards).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnProviderGetGooglePlayerId).setOnClickListener(this)
    }

    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {

            R.id.btnProviderGoogleAchievementsReveal -> {

                addLog("\n\n=== 업적 숨김 해제 요청 ===\n")
                addLog("ProviderGoogle.achievementsReveal() Called\n")

                ProviderGoogle.achievementsReveal("CgkIzev2g9MJEAIQAQ")
            }


            R.id.btnProviderGoogleAchievementsUnlock -> {

                addLog("\n\n=== 업적 획득 요청 ===\n")
                addLog("ProviderGoogle.achievementsUnlock() Called\n")

                ProviderGoogle.achievementsUnlock("CgkIzev2g9MJEAIQAw")
            }


            R.id.btnProviderGoogleAchievementsIncrement -> {

                addLog("\n\n=== 업적 단계 증가 (+1) / 5 ===\n")
                addLog("ProviderGoogle.achievementsIncrement() Called\n")

                achievementScore++

                ProviderGoogle.achievementsIncrement("CgkIzev2g9MJEAIQAg", 1)
                ProviderGoogle.achievementsIncrement("CgkIzev2g9MJEAIQBQ", 1)
                ProviderGoogle.achievementsIncrement("CgkIzev2g9MJEAIQBg", 1)
            }


            R.id.btnProviderGoogleShowAchievements -> {

                addLog("\n\n=== 업적 UI 표시 ===\n")
                addLog("ProviderGoogle.showAchievements() Called\n")

                ProviderGoogle.showAchievements(object : ProviderGoogle.GoogleAchievementsListener {
                    override fun onAchievementsResult(resultAPI: ResultAPI) {

                        addLog(resultAPI.toString())
                    }
                })
            }


            R.id.btnProviderGoogleLeaderboardsSubmitScore -> {

                addLog("\n\n=== 리더보드 점수 증가 (+10) ===\n")
                addLog("ProviderGoogle.achievementsIncrement() Called\n")

                leaderboardScore += 10
                ProviderGoogle.leaderboardsSubmitScore("CgkIzev2g9MJEAIQBw", leaderboardScore.toLong())
            }


            R.id.btnProviderGoogleShowLeaderboards -> {

                addLog("\n\n=== 리더보드 UI 표시 ===\n")
                addLog("ProviderGoogle.showLeaderboards() Called\n")

                ProviderGoogle.showLeaderboards(object : ProviderGoogle.GoogleLeaderboardsListener {
                    override fun onLeaderboardsResult(resultAPI: ResultAPI) {

                        addLog(resultAPI.toString())
                    }
                })
            }


            R.id.btnProviderGetGooglePlayerId -> {

                addLog("\n\n=== 구글 Player ID 표시 ===\n")
                addLog("ProviderGoogle.getGooglePlayerId() Called\n")

                ProviderGoogle.getGooglePlayerId(object : ProviderGoogle.GooglePlayerIdListener {

                    override fun onPlayerIdResult(resultAPI: ResultAPI, googlePlayerId: String, authCode: String) {

                        addLog("$resultAPI" +
                                "\ngooglePlayerId : $googlePlayerId" +
                                "\nauthCode : $authCode")
                    }
                })
            }

        }

    }
}
