package com.hive.sample


import android.app.Activity
import android.view.View
import android.widget.EditText
import android.widget.RadioGroup
import android.widget.ToggleButton
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.Push
import com.hive.Push.LocalPush
import com.hive.Push.LocalPushListener
import com.hive.Push.RemotePush
import com.hive.Push.RemotePushListener
import com.hive.ResultAPI
import com.hive.sample.uihelper.ViewPagerBase


/**
 * HIVE SDK 의 Push Notification 기능 테스트 뷰
 *
 * @author ryuvsken
 */
class PushTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {

    private var bucketType = 0

    init {

        initUi()
    }

    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)

        this.view.findViewById<View>(R.id.btnPushGetPushSetting).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushSetPushSetting).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushGetRemote).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushSetRemote).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushRegisterLocal).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushUnregisterLocal).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushUnregisterLocalPushes).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushUnregisterAllLocalPushes).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushRegisterLocal100).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnPushUnregisterLocal100).setOnClickListener(this)

        val tbRemoteEnabledWhileRunning = this.view.findViewById<View>(R.id.tbRemoteEnabledWhileRunning) as ToggleButton
        val tbLocalEnabledWhileRunning = this.view.findViewById<View>(R.id.tbLocalEnabledWhileRunning) as ToggleButton

        Push.getForegroundPush(object : Push.PushSettingListener {
            override fun onPushSetting(result: ResultAPI, pushSetting: Push.PushSetting?) {
                if (result.isSuccess) {
                    tbRemoteEnabledWhileRunning.isChecked = pushSetting!!.isRemotePushEnabledWhileRunning
                    tbLocalEnabledWhileRunning.isChecked = pushSetting.isLocalPushEnabledWhileRunning
                }
            }
        })

    }


    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {

            R.id.btnPushGetPushSetting -> {

                val tbRemoteEnabledWhileRunning = this.view.findViewById<View>(R.id.tbRemoteEnabledWhileRunning) as ToggleButton
                val tbLocalEnabledWhileRunning = this.view.findViewById<View>(R.id.tbLocalEnabledWhileRunning) as ToggleButton

                addLog("\n\n=== 푸시 설정값 조회 ===\n")
                addLog("Push.getForegroundPush(listener) called\n")

                Push.getForegroundPush(object : Push.PushSettingListener {
                    override fun onPushSetting(result: ResultAPI, pushSetting: Push.PushSetting?) {

                        addLog(String.format("PushSettingListener.onPushSetting() Callback\nresult = %s\npushSetting = %s\n", result.toString(), pushSetting?.toString()
                                ?: "null"))

                        if (result.isSuccess) {
                            activity.runOnUiThread {
                                tbRemoteEnabledWhileRunning.isChecked = pushSetting!!.isRemotePushEnabledWhileRunning
                                tbLocalEnabledWhileRunning.isChecked = pushSetting.isLocalPushEnabledWhileRunning
                            }
                        }
                    }
                })
            }

            R.id.btnPushSetPushSetting -> {

                val tbRemoteEnabledWhileRunning = this.view.findViewById<View>(R.id.tbRemoteEnabledWhileRunning) as ToggleButton
                val tbLocalEnabledWhileRunning = this.view.findViewById<View>(R.id.tbLocalEnabledWhileRunning) as ToggleButton

                addLog("\n\n=== 푸시 설정값 저장 ===\n")
                addLog("Push.setForegroundPush(pushSetting, listener) called\n")

                val pushSetting = Push.PushSetting(tbRemoteEnabledWhileRunning.isChecked, tbLocalEnabledWhileRunning.isChecked)

                Push.setForegroundPush(pushSetting, object : Push.PushSettingListener {
                    override fun onPushSetting(result: ResultAPI, pushSetting: Push.PushSetting?) {
                        addLog(String.format("PushSettingListener.onPushSetting() Callback\nresult = %s\npushSetting = %s\n", result.toString(), pushSetting?.toString()
                                ?: "null"))


                    }
                })
            }

            R.id.btnPushGetRemote -> {

                val tbUseNotice = this.view.findViewById<View>(R.id.tbUseNotice) as ToggleButton
                val tbUseNoticeNight = this.view.findViewById<View>(R.id.tbUseNoticeNight) as ToggleButton

                addLog("\n\n=== 리모트 푸시 정보 조회 ===\n")
                addLog("Push.getRemotePush(listener) called\n")

                Push.getRemotePush(object : RemotePushListener {

                    override fun onPushToken(result: ResultAPI, remotePush: RemotePush?) {

                        addLog(String.format("RemotePushListener.onPushToken() Callback\nresult = %s\nremotePush = %s\n", result.toString(), remotePush?.toString()
                                ?: "null"))

                        if (result.isSuccess && remotePush != null) {

                            activity.runOnUiThread {
                                tbUseNotice.isChecked = remotePush.isAgreeNotice
                                tbUseNoticeNight.isChecked = remotePush.isAgreeNight
                            }

                        } else {
//    						AlertDialog.Builder alertDialog = new AlertDialog.Builder(HiveActivity.getRecentActivity());
//    						alertDialog.setTitle("setPushToken 결과 받음");
//    						alertDialog.setMessage("에러가 발생했으며 내용은 다음과 같습니다.\n"
//    								+ result.message);
//    						alertDialog.setPositiveButton("확인", null);
//    						alertDialog.show();
                        }
                    }
                })
            }


            R.id.btnPushSetRemote -> {

                val tbUseNotice = this.view.findViewById<View>(R.id.tbUseNotice) as ToggleButton
                val tbUseNoticeNight = this.view.findViewById<View>(R.id.tbUseNoticeNight) as ToggleButton

                val remotePush = RemotePush(tbUseNotice.isChecked, tbUseNoticeNight.isChecked)


                addLog("\n\n=== 리모트 푸시 정보 설정 ===\n")
                addLog(String.format("Push.setRemotePush(remotePush, listener) called\nremotePush = %s\n", remotePush.toString()))

                Push.setRemotePush(remotePush, object : RemotePushListener {

                    override fun onPushToken(result: ResultAPI, remotePush: RemotePush?) {

                        addLog(String.format("RemotePushListener.onPushToken() Callback\nresult = %s\nremotePush = %s\n", result.toString(), remotePush?.toString()
                                ?: "null"))

                        if (!result.isSuccess) {

                            //						AlertDialog.Builder alertDialog = new AlertDialog.Builder(HiveActivity.getRecentActivity());
                            //						alertDialog.setTitle("getPushToken 결과 받음");
                            //						alertDialog.setMessage("에러가 발생했으며 내용은 다음과 같습니다.\n"
                            //								+ result.message);
                            //						alertDialog.setPositiveButton("확인", null);
                            //						alertDialog.show();
                        } else {

                            //						AlertDialog.Builder alertDialog = new AlertDialog.Builder(HiveActivity.getRecentActivity());
                            //						alertDialog.setTitle("setPushToken 결과 받음");
                            //						alertDialog.setMessage("다음의 PushToken이 정상적으로 서버에 전송되었습니다.\n"
                            //								+ remotePush.toString());
                            //						alertDialog.setPositiveButton("확인", null);
                            //						alertDialog.show();
                        }
                    }
                })
            }


            R.id.btnPushRegisterLocal -> {

                val etTitle = this.view.findViewById<View>(R.id.etPushTitle) as EditText
                val etMessage = this.view.findViewById<View>(R.id.etPushMessage) as EditText
                val etBigMessage = this.view.findViewById<View>(R.id.etPushBigMessage) as EditText
                val etNo = this.view.findViewById<View>(R.id.etPushNo) as EditText
                val etAfter = this.view.findViewById<View>(R.id.etPushAfter) as EditText
                val etBucketSize = this.view.findViewById<View>(R.id.etBucketSize) as EditText
                val rgBucketType = this.view.findViewById<View>(R.id.rgBucketType) as RadioGroup


                var after = 0L
                var noticeId = 1
                var bucketSize = 0
                var title: String = ""
                var msg: String = ""
                var bigMsg = ""
                try {
                    after = java.lang.Long.valueOf(etAfter.text.toString())
                    noticeId = Integer.valueOf(etNo.text.toString())
                    title = etTitle.text.toString()
                    msg = etMessage.text.toString()
                    bigMsg = etBigMessage.text.toString()
                    bucketSize = etBucketSize.text.toString().toInt()

                } catch (e: Exception) {
                    e.printStackTrace()
                }

                rgBucketType.setOnCheckedChangeListener { _, checkedId ->
                    bucketType = when (checkedId) {
                        R.id.rbOverWriteType -> 0
                        R.id.rbInboxingType -> 1
                        R.id.rbBigTextType -> 2
                        else -> 0
                    }
                }
                val localPush = LocalPush(noticeId, title, msg, after,
                        bigMsg, "푸시 티커",
                        "", "", "", "",
                        "", bucketType, bucketSize, "", "")


                addLog("\n\n=== 로컬 푸시 등록 ===\n")
                addLog(String.format("Push.registerLocalPush(localPush, listener) called\nlocalPush = %s\n", localPush.toString()))

                Push.registerLocalPush(localPush, object : LocalPushListener {

                    override fun onRegisterLocalPush(result: ResultAPI, localPush: LocalPush?) {

                        addLog(String.format("RemotePushListener.onPushToken() Callback\nresult = %s\nlocalPush = %s\n", result.toString(), localPush?.toString()
                                ?: "null"))
                    }
                })
            }


            R.id.btnPushUnregisterLocal -> {

                val etNo = this.view.findViewById<View>(R.id.etPushNo) as EditText
                var noticeID = 1
                try {
                    noticeID = Integer.valueOf(etNo.text.toString())
                } catch (e: Exception) {
                    e.printStackTrace()
                }


                addLog("\n\n=== 로컬 푸시 해제 ===\n")
                addLog(String.format("Push.unregisterLocalPush(noticeID = %d) called\n", noticeID))

                Push.unregisterLocalPush(noticeID)
            }


            R.id.btnPushUnregisterLocalPushes -> {

                val etNo = this.view.findViewById<View>(R.id.etPushNo) as EditText
                var noticeID = 1
                try {
                    noticeID = Integer.valueOf(etNo.text.toString())
                } catch (e: Exception) {
                    e.printStackTrace()
                }

                val noticeIDs = ArrayList<Int>()
                noticeIDs.add(noticeID)

                addLog("\n\n=== 로컬 푸시 리스트 해제 ===\n")
                addLog(String.format("Push.unregisterLocalPushes(noticeID = %d) called\n", noticeID))

                Push.unregisterLocalPushes(noticeIDs)
            }


            R.id.btnPushUnregisterAllLocalPushes -> {

                addLog("\n\n=== 로컬 푸시 모두 해제 ===\n")
                addLog(String.format("Push.unregisterAllLocalPushes called\n"))

                Push.unregisterAllLocalPushes()
            }


            R.id.btnPushRegisterLocal100 -> {

                val etTitle = this.view.findViewById<View>(R.id.etPushTitle) as EditText
                val etMessage = this.view.findViewById<View>(R.id.etPushMessage) as EditText
                val etBigMessage = this.view.findViewById<View>(R.id.etPushBigMessage) as EditText
                val etNo = this.view.findViewById<View>(R.id.etPushNo) as EditText
                val etAfter = this.view.findViewById<View>(R.id.etPushAfter) as EditText
                val etBucketSize = this.view.findViewById<View>(R.id.etBucketSize) as EditText
                val rgBucketType = this.view.findViewById<View>(R.id.rgBucketType) as RadioGroup


                var after = 0L
                var noticeId = 1
                var bucketSize = 0
                var title: String = ""
                var msg: String = ""
                var bigMsg = ""
                try {
                    after = java.lang.Long.valueOf(etAfter.text.toString())
                    noticeId = Integer.valueOf(etNo.text.toString())
                    title = etTitle.text.toString()
                    msg = etMessage.text.toString()
                    bigMsg = etBigMessage.text.toString()
                    bucketSize = etBucketSize.text.toString().toInt()

                } catch (e: Exception) {
                    e.printStackTrace()
                }

                rgBucketType.setOnCheckedChangeListener { _, checkedId ->
                    bucketType = when (checkedId) {
                        R.id.rbOverWriteType -> 0
                        R.id.rbInboxingType -> 1
                        R.id.rbBigTextType -> 2
                        else -> 0
                    }
                }

                addLog("\n\n=== 로컬 푸시 100개 등록 시작 ===\n")
                val startTime = System.nanoTime()

                for (i in 1..100) {
                    val localPush = LocalPush(1000+i, "$title($i)", msg, after,
                            bigMsg, "푸시 티커",
                            "", "", "", "",
                            "", bucketType, bucketSize, "", "")

                    Push.registerLocalPush(localPush, object : LocalPushListener {
                        override fun onRegisterLocalPush(result: ResultAPI, localPush: LocalPush?) {
                            addLog(String.format("RemotePushListener.onPushToken() Callback\nresult = %s\nlocalPush = %s\n", result.toString(), localPush?.toString()
                                    ?: "null"))
                        }
                    })
                }

                val endTime = System.nanoTime()
                val latencyMs = (endTime - startTime) / 1000000
                addLog("\n\n=== 로컬 푸시 100개 등록 완료 ($latencyMs milliseconds) ===\n")
            }


            R.id.btnPushUnregisterLocal100 -> {
                addLog("\n\n=== 로컬 푸시 100개 해제 시작 ===\n")
                val startTime = System.nanoTime()

                for (i in 1..100) {
                    Push.unregisterLocalPush(1000+i)
                }

                val endTime = System.nanoTime()
                val latencyMs = (endTime - startTime) / 1000000
                addLog("\n\n=== 로컬 푸시 100개 해제 완료 ($latencyMs milliseconds)===\n")
            }
        }

    }
}



