package com.hive.sample

import android.app.Activity
import android.view.View
import android.widget.ArrayAdapter
import android.widget.EditText
import android.widget.Spinner
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.ResultAPI
import com.hive.SocialFacebook.ProfileFacebook
import com.hive.SocialFacebook.SocialFacebookMessage
import com.hive.SocialGoogle
import com.hive.SocialHive
import com.hive.SocialHive.MessageContent
import com.hive.SocialHive.ProfileHive
import com.hive.SocialHive.SocialBadge
import com.hive.base.Android
import com.hive.sample.uihelper.SelectDialog
import com.hive.sample.uihelper.ViewPagerBase
import org.json.JSONObject


/**
 * HIVE SDK 의 Social 기능 테스트 뷰
 *
 * @author ryuvsken
 */
class SocialTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {


    private val hiveFriends: MutableList<ProfileHive>
    private val facebookFriends: List<ProfileFacebook>
    private val defaultSampleImageURL = "http://gw.com2us.com/gw_resource/css/gw_skin01/images_gw/logo.png"

    init {

        initUi()
        hiveFriends = ArrayList()
        facebookFriends = ArrayList()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)


        var spAdapter: ArrayAdapter<String>

        // Social Hive
        this.view.findViewById<View>(R.id.btnSocialHiveGetMyProfile).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnSocialHiveSetMyProfile).setOnClickListener(this)

        spAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spAdapter.add("IN_GAME")
        spAdapter.add("OUT_GAME")
        spAdapter.add("INVITED")
        spAdapter.add("ALL_GAME")
        (this.view.findViewById<View>(R.id.spFriendType) as Spinner).adapter = spAdapter

        this.view.findViewById<View>(R.id.btnSocialHiveGetFriends).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnSocialHiveSendMessage).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnSocialHiveSendInvitationMessage).setOnClickListener(this)

        spAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        for (type in SocialHive.HiveDialogType.values())
            spAdapter.add(type.name)

        (this.view.findViewById<View>(R.id.spHiveDialogType) as Spinner).adapter = spAdapter

        val editText = this.view.findViewById<View>(R.id.etHiveDialogVid) as EditText
        editText.isFocusable = false
        editText.setOnClickListener(this)

        this.view.findViewById<View>(R.id.btnSocialHiveShowHiveDialog).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnSocialHiveCheckNotification).setOnClickListener(this)


        // Social Facebook
        this.view.findViewById<View>(R.id.btnSocialFacebookGetMyProfile).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnSocialFacebookGetFriends).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnSocialFacebookSendMessage).setOnClickListener(this)
        this.view.findViewById<View>(R.id.btnSocialFacebookShowInvitationDialog).setOnClickListener(this)


        // Social Google
        this.view.findViewById<View>(R.id.btnSocialGoogleGetMyProfile).setOnClickListener(this)
    }


    override fun onClick(v: View) {

        val rid = v.id

        when (rid) {

            // SocialHive

            R.id.btnSocialHiveGetMyProfile -> {

                addLog("\n\n=== HIVE 인증 사용자의 자기 프로필 조회 ===\n")
                addLog("SocialHive.getMyProfile(listener) Called\n")

                SocialHive.getMyProfile(object : com.hive.SocialHive.ProfileListener {

                    override fun onProfile(result: ResultAPI, profileList: ArrayList<ProfileHive>?) {

                        addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n", result.toString(), profileList?.toString()
                                ?: "null"))

                        var profile: ProfileHive? = null
                        if (result.isSuccess) {
                            profile = profileList!![0]
                        }
                    }
                })
            }


            R.id.btnSocialHiveSetMyProfile -> {

                SelectDialog.Builder(this.activity).setTitle("SetMyProfile").addInputText("displayName").setDialogResultListener(

                    object : SelectDialog.OnDialogResultListener {

                        override fun onDialogResult(completed: Boolean, result: Map<String, Any>?) {

                            if (completed) {
                                val displayName = result!!["displayName"] as String?


                                addLog("\n\n=== HIVE 인증 사용자의 자기 대화명 수정 ===\n")
                                addLog(String.format("SocialHive.setMyProfile(\"%s\", listener) Called\n", displayName))

                                SocialHive.setMyProfile(displayName!!, object : com.hive.SocialHive.ProfileListener {

                                    override fun onProfile(result: ResultAPI, profileList: ArrayList<ProfileHive>?) {

                                        addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n", result.toString(), profileList?.toString()
                                                ?: "null"))
                                    }
                                })
                            }
                        }
                    }

                ).build().show()
            }


            R.id.btnSocialHiveGetFriends -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spFriendType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String
                val friendType = SocialHive.FriendType.valueOf(selectedValue.toUpperCase())


                addLog("\n\n=== HIVE 인증 사용자의 친구 목록 조회 ===\n")
                addLog(String.format("SocialHive.getFriends(friendType = %s, listener) Called\n", friendType.name))

                SocialHive.getFriends(friendType, object : com.hive.SocialHive.ProfileListener {

                    override fun onProfile(result: ResultAPI, profileList: ArrayList<ProfileHive>?) {

                        addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n", result.toString(), profileList?.toString()
                                ?: "null"))

                        if (result.isSuccess) {
                            hiveFriends.clear()
                            hiveFriends.addAll(profileList!!)


                            val vidList = ArrayList<String>()

                            for (profile in profileList) {

                                profile.vid?.let {
                                    vidList.add(it)
                                }
                            }

                            var vids = vidList.toTypedArray<String>()

                            SocialHive.getProfiles(vids, object : com.hive.SocialHive.ProfileListener {

                                override fun onProfile(result: ResultAPI, profileList: ArrayList<ProfileHive>?) {

                                    addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n", result.toString(), profileList?.toString()
                                            ?: "null"))

                                }
                            })


                        } else {

                            val vids = arrayOf("61217583")
                            SocialHive.getProfiles(vids, object : com.hive.SocialHive.ProfileListener {

                                override fun onProfile(result: ResultAPI, profileList: ArrayList<ProfileHive>?) {

                                    addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n", result.toString(), profileList?.toString()
                                            ?: "null"))

                                }
                            })

                        }
                    }
                })
            }


            R.id.btnSocialHiveSendMessage -> {

                // 입력 창 띄우기
                val builder = SelectDialog.Builder(this.activity).setTitle("SendMessage").addInputText("vid")
                if (hiveFriends.isEmpty()) {
                    builder.addInputText("uid", "")
                } else {
                    val uidLists = ArrayList<String>(hiveFriends.size)
                    for (profile in hiveFriends) {
                        profile.uid?.let {uidLists.add(it)}
                    }
                    builder.addSelectList("uid", uidLists)
                }
                builder.addInputText("message", "메세지").addInputText("imageUrl", "이미지 URL").addInputText("thumbnailUrl", "썸네일 URL").addSwitch("usePush", true)
                        .setDialogResultListener(

                                object : SelectDialog.OnDialogResultListener {

                                    override fun onDialogResult(completed: Boolean, result: Map<String, Any>?) {

                                        if (completed && result != null) {
                                            val messageContent = MessageContent()
                                            messageContent.vid = result["vid"] as String
                                            messageContent.uid = result["uid"] as String
                                            messageContent.message = result["message"] as String
                                            messageContent.imageUrl = result["imageUrl"] as String
                                            messageContent.thumbnailUrl = result["thumbnailUrl"] as String
                                            messageContent.usePush = result["usePush"] as Boolean

                                            addLog("\n\n=== HIVE 친구에게 메시지 전송 요청 ===\n")
                                            addLog(String.format("SocialHive.sendMessage(messageContent, listener) Called\nmessageContent = %s\n", messageContent.toString()))

                                            SocialHive.sendMessageEx(messageContent, object : com.hive.SocialHive.SendMessageListenerEx {

                                                override fun onSendMessage(result: ResultAPI, responseJsonData: JSONObject?) {

                                                    addLog(String.format("SocialHive.SendMessageListener.onSendMessage() Callback\nresult = %s\n", result.toString()))
                                                }
                                            })
                                        }
                                    }
                        }).build().show()

            }


            R.id.btnSocialHiveSendInvitationMessage -> {

                // 입력 창 띄우기
                val builder = SelectDialog.Builder(this.activity).setTitle("SendInvitationMessage").addInputText("vid")
                if (hiveFriends.isEmpty()) {
                    builder.addInputText("uid")
                } else {
                    val uidLists = ArrayList<String>(hiveFriends.size)
                    for (profile in hiveFriends) {
                        profile.uid?.let {uidLists.add(it)}
                    }
                    builder.addSelectList("uid", uidLists)
                }
                builder.addInputText("message", "메세지").addInputText("imageUrl", "이미지 URL").addInputText("thumbnailUrl", "썸네일 URL").addSwitch("usePush", true)
                        .setDialogResultListener(

                            object : SelectDialog.OnDialogResultListener {

                                override fun onDialogResult(completed: Boolean, result: Map<String, Any>?) {

                                    if (completed && result != null) {
                                        val messageContent = MessageContent()
                                        messageContent.vid = result["vid"] as String
                                        messageContent.uid = result["uid"] as String
                                        messageContent.message = result["message"] as String
                                        messageContent.imageUrl = result["imageUrl"] as String
                                        messageContent.thumbnailUrl = result["thumbnailUrl"] as String
                                        messageContent.usePush = result["usePush"] as Boolean


                                        addLog("\n\n=== HIVE 친구에게 초대 메시지 전송 요청 ===\n")
                                        addLog(String.format("SocialHive.sendInvitationMessage(messageContent, listener) Called\nmessageContent = %s\n", messageContent.toString()))

                                        SocialHive.sendInvitationMessage(messageContent, object : com.hive.SocialHive.SendMessageListener {

                                            override fun onSendMessage(result: ResultAPI) {

                                                addLog(String.format("SocialHive.sendInvitationMessage.onSendMessage() Callback\nresult = %s\n", result.toString()))
                                            }
                                        })
                                    }
                                }

                        }).build().show()
            }

            R.id.etHiveDialogVid -> {
                val builder = SelectDialog.Builder(this.activity).setTitle("InputVid")
                if (hiveFriends.isEmpty()) {
                    builder.addInputText("vid")
                } else {
                    val vidLists = ArrayList<String>(hiveFriends.size)
                    for (profile in hiveFriends) {
                        profile.vid?.let {vidLists.add(it)}
                    }
                    builder.addSelectList("vid", vidLists)
                }
                builder.setDialogResultListener(

                        object : SelectDialog.OnDialogResultListener {

                            override fun onDialogResult(completed: Boolean, result: Map<String, Any>?) {
                                if (completed) {
                                    val editText = view.findViewById<View>(R.id.etHiveDialogVid) as EditText
                                    val vid = result!!["vid"] as String?
                                    editText.setText(vid)
                                }
                            }
                }).build().show()

            }

            R.id.btnSocialHiveShowHiveDialog -> {

                val spinner: Spinner

                spinner = this.view.findViewById<View>(R.id.spHiveDialogType) as Spinner
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String
                val hiveDialogType = SocialHive.HiveDialogType.valueOf(selectedValue.toUpperCase())


                val editText: EditText

                editText = this.view.findViewById<View>(R.id.etHiveDialogVid) as EditText
                val vid = editText.text.toString()
                // String vid = "";

                addLog("\n\n=== HIVE 웹뷰 대화상자 호출 ===\n")
                addLog(String.format("SocialHive.showHiveDialog(hiveDialogType = %s, vid = %s, listener) Called\n", hiveDialogType.name, vid))

                val showHiveDialogListener = object : com.hive.SocialHive.ShowHiveDialogListener {

                    override fun onShowHiveDialog(result: ResultAPI) {
                        addLog(String.format("SocialHive.ShowHiveDialogListener.onShowHiveDialog() Callback\nresult = %s\n", result.toString()))
                    }
                }

                when(hiveDialogType){
                    SocialHive.HiveDialogType.CHATBOT -> {
                        val additionalInfo = JSONObject().apply {
                            this.put("evt_code",1000)
                            this.put("p1","random")
                        }

                        SocialHive.showHiveDialog(hiveDialogType, vid, additionalInfo.toString(), showHiveDialogListener)
                    }

                    else -> {
                        SocialHive.showHiveDialog(hiveDialogType, vid, showHiveDialogListener)
                    }
                }

            }


            R.id.btnSocialHiveCheckNotification -> {
                addLog("\n\n=== HIVE 인증 사용자의 쪽지 조회 ===\n")
                addLog("SocialHive.getBadgeInfo(listener) Called\n")

                SocialHive.getBadgeInfo(object : com.hive.SocialHive.SocialBadgeListener {

                    override fun onSocialBadge(result: ResultAPI, badge: SocialBadge?) {

                        addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n", result.toString(), badge?.messageCount
                                ?: "null"))

                    }
                })
            }

            // SocialFacebook

            R.id.btnSocialFacebookGetMyProfile -> {

                addLog("\n\n=== Facebook 프로필 정보 조회 ===\n")
                addLog("SocialFacebook.getMyProfile(listener) Called\n")

                com.hive.SocialFacebook.getMyProfile { result, profileList ->
                    addLog(String.format("SocialFacebook.ProfileListener.onProfile() Callback\nresult = %s\nprofileList = %s\n", result.toString(), profileList?.toString()
                            ?: "null"))
                }
            }


            R.id.btnSocialFacebookGetFriends -> {

                addLog("\n\n=== Facebook 친구 목록 조회 ===\n")
                addLog("SocialFacebook.getFriends(listener) Called\n")

                com.hive.SocialFacebook.getFriends { result, profileList ->
                    addLog(String.format("SocialFacebook.ProfileListener.onProfile() Callback\nresult = %s\nprofileList = %s\n", result.toString(), profileList?.toString()
                            ?: "null"))
                }
            }


            R.id.btnSocialFacebookSendMessage -> {

                val builder = SelectDialog.Builder(this.activity)
                        .setTitle("SocialFacebookSendMessage")
                        .addInputText("message", "전달할 메세지.")
                        .addInputText("dialogTitle", "메세지 전달할 다이얼로그 제목.")

                if (facebookFriends.isEmpty())
                    builder.addInputText("recipients", "메세지를 보낼 친구가 없거나, 친구목록을 업데이트 하지 않았습니다.")
                else {
                    val idLists = ArrayList<String>()
                    for (pf in facebookFriends) {
                        idLists.add(pf.uid)
                    }
                    if (!idLists.isEmpty())
                        builder.addSelectList("recipients", idLists)
                }

                builder.setDialogResultListener(

                        object : SelectDialog.OnDialogResultListener {

                            override fun onDialogResult(completed: Boolean, result: Map<String, Any>?) {

                                if (completed) {

                                    val content = Android.fromMap(result!!, SocialFacebookMessage::class.java)

                                    content!!.data = ""

                                    addLog("\n\n=== Facebook 인증 사용자의 친구에게 메시지를 전송 ===\n")
                                    addLog(String.format("SocialFacebook.sendMessageFacebook(content, listener) Called\ncontent = %s\n", content?.toString()
                                            ?: "null"))

                                    com.hive.SocialFacebook.sendMessageFacebook(content) { result -> addLog(String.format("SocialFacebook.SendMessageFacebookListener.onSendMessageFacebook() Callback\nresult = %s\n", result.toString())) }
                                }
                            }
                }).build().show()

            }


            R.id.btnSocialFacebookShowInvitationDialog -> {

                val builder = SelectDialog.Builder(this.activity)
                        .setTitle("SocialFacebookAppInvite")
                        .addInputText("message", "초대 메세지.")
                        .addInputText("dialogTitle", "초대 다이얼로그 제목.")
                builder.setDialogResultListener(

                        object : SelectDialog.OnDialogResultListener {

                            override fun onDialogResult(completed: Boolean, result: Map<String, Any>?) {

                                if (completed) {
                                    val content = Android.fromMap(result!!, SocialFacebookMessage::class.java)

                                    content!!.data = ""
                                    content.recipients = null

                                    addLog("\n\n=== Facebook 인증 사용자의 친구에게 초대 요청을 위한 대화상자 호출. ===\n")
                                    addLog(String.format("SocialFacebook.showInvitationDialog(content, listener) Called\ncontent = %s\n", content?.toString()
                                            ?: "null"))

                                    com.hive.SocialFacebook.showInvitationDialog(content) { result, invitedUserList ->
                                        addLog(String.format("SocialFacebook.ShowInvitationDialogListener.onShowInvitationDialog() Callback\nresult = %s\ninvitedUserList - %s\n", result.toString(), invitedUserList?.toString()
                                                ?: "null"))
                                    }
                                }
                            }

                }).build().show()

            }

        }
    }
}



