package com.hive.sample.game

import com.gcp.hivecore.Network
import com.gcp.hivecore.Request
import com.hive.*
import com.hive.base.Android
import com.hive.standalone.HiveLifecycle
import org.json.JSONArray
import org.json.JSONObject
import java.util.*

object GameNetwork {


    fun login(listener: IGameListener) {

        request(getUrl("account/login"), listener)
    }

    fun userInfo(listener: IGameListener) {

        request(getUrl("account/user"), listener)
    }

    fun posts(listener: IGameListener) {

        request(getUrl("postbox/posts"), listener)
    }


    fun postReceive(listener: IGameListener, vararg posts: PostBoxObject) {

        request(getUrl("postbox/receive"), listener, *posts)
    }

    fun postReceive(posts: ArrayList<Post>, listener: IGameListener) {

        request(getUrl("postbox/receive"), listener, posts)
    }

    fun receiptVerify(iapv4Receipt: IAPV4.IAPV4Receipt, listener: IGameListener) {

        request(getUrl("hive/iap/verify"), listener, iapv4Receipt)
    }

    private fun request(url: String, listener: IGameListener, vararg posts: PostBoxObject) {

        val account = HiveLifecycle.account

        val vid = account.vid
        val uid = account.uid
        val did = account.did
        val lc_sessionkey = account.accessToken
        val vidType = account.vidType


        val params = JSONObject()
        try {
            params.put("session_token", GameUser.sessionToken)
            params.put("world", Configuration.serverId)
            params.put("vid", vid)
            params.put("did", did)
            params.put("uid", uid)
            params.put("appid", Configuration.appId)
            params.put("country", Configuration.hiveCountry)
            params.put("language", Configuration.gameLanguage) //샘플앱이라서 Config 에서 세팅한 값을 읽음
            params.put("appver", Android.getAppVersion())
            params.put("lc_sessionkey", lc_sessionkey)
            params.put("vid_type", vidType)

            if (posts.isNotEmpty()) {
                val postIdList = JSONArray()

                for (post in posts) {
                    postIdList.put(post.getPostId())
                }

                params.put("post_id_list", postIdList)
            }

        } catch (e: Exception) {
            e.printStackTrace()
        }

        val request = Request(url = url, body = params.toString().toByteArray())

        Thread {

            val coreResponse = Network.sync(request)
            println("====Game Network Response : $coreResponse")

            if (coreResponse.isSuccess()) {

                coreResponse.content?.let {
                    val gameResponse = com.hive.sample.game.Response(String(it))
                    GameUser.reflesh(gameResponse)
                    listener.toOnGameListener(true, gameResponse)
                }

            } else {
                listener.toOnGameListener(false, null)
            }

        }.start()
    }

    private fun request(url: String, listener: IGameListener, posts: ArrayList<Post>?) {

        val account = HiveLifecycle.account

        val vid = account.vid
        val uid = account.uid
        val did = account.did
        val lc_sessionkey = account.accessToken
        val vidType = account.vidType

        val params = JSONObject()
        try {
            params.put("session_token", GameUser.sessionToken)
            params.put("world", Configuration.serverId)
            params.put("vid", vid)
            params.put("did", did)
            params.put("uid", uid)
            params.put("appid", Configuration.appId)
            params.put("country", Configuration.hiveCountry)
            params.put("language", Configuration.gameLanguage) //샘플앱이라서 Config 에서 세팅한 값을 읽음
            params.put("appver", Android.getAppVersion())
            params.put("lc_sessionkey", lc_sessionkey)
            params.put("vid_type", vidType)

            if (posts != null && posts.size > 0) {
                val postIdList = JSONArray()

                for (post in posts) {
                    postIdList.put(post.getPostId())
                }

                params.put("post_id_list", postIdList)
            }

        } catch (e: Exception) {
            e.printStackTrace()
        }

        val request = Request(url = url, body = params.toString().toByteArray())

        Thread {

            val coreResponse = Network.sync(request)
            println("====Game Network Response : $coreResponse")

            if (coreResponse.isSuccess()) {

                coreResponse.content?.let {
                    val gameResponse = com.hive.sample.game.Response(String(it))
                    GameUser.reflesh(gameResponse)
                    listener.toOnGameListener(true, gameResponse)
                }

            } else {
                listener.toOnGameListener(false, null)
            }

        }.start()
    }

    private fun request(url: String, listener: IGameListener, iapv4Receipt: IAPV4.IAPV4Receipt) {

        val account = HiveLifecycle.account

        val vid = account.vid
        val uid = account.uid
        val did = account.did
        val lc_sessionkey = account.accessToken
        val vidType = account.vidType

        val params = JSONObject()
        try {

            params.put("session_token", GameUser.sessionToken)
            params.put("world", Configuration.serverId)
            params.put("vid", vid)
            params.put("purchase_bypass_info", iapv4Receipt.bypassInfo)
            params.put("game_info", iapv4Receipt.additionalInfo)
            params.put("item_type", iapv4Receipt.product.productType)


        } catch (e: Exception) {
            e.printStackTrace()
        }

        val request = Request(url = url, body = params.toString().toByteArray())

        Thread {

            val coreResponse = Network.sync(request)
            println("====Game Network Response : $coreResponse")

            if (coreResponse.isSuccess()) {

                coreResponse.content?.let {
                    val gameResponse = com.hive.sample.game.Response(String(it))
                    GameUser.reflesh(gameResponse)
                    listener.toOnGameListener(true, gameResponse)
                }

            } else {
                listener.toOnGameListener(false, null)
            }

        }.start()
    }

    private fun getUrl(value: String): String {

        val buffer = StringBuffer()
        buffer.append("http://")

        when (Configuration.zone) {

            com.gcp.hivecore.Configuration.ZoneType.REAL -> {}

            com.gcp.hivecore.Configuration.ZoneType.SANDBOX -> buffer.append("sandbox-")

            com.gcp.hivecore.Configuration.ZoneType.TEST -> buffer.append("test-")

            com.gcp.hivecore.Configuration.ZoneType.DEV -> buffer.append("dev-")
        }

        buffer.append("misample.com2us.net/").append(value)

        return buffer.toString()
    }

    private fun IGameListener.toOnGameListener(result: Boolean, response: Response?) {
        HiveActivity.recentActivity.runOnUiThread {
            this.onGameListener(result, response)
        }
    }

}
