package com.hive.sample.game


import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.*
import com.hive.Logger
import com.hive.ui.Resource
import com.hive.sample.IAPTestView
import com.hive.sample.IAPV4TestView
import java.util.*


/**
 * HIVE SDK 의 가상 게임서버와 연결된 우편함을 보여주는 대화 상자
 *
 * @author hife
 */
class GamePostboxDialog : Dialog {

    internal var listview_postbox: ListView

    internal lateinit var listviewadapter_postbox: ListViewAdapter

    internal var parantsView: IAPTestView? = null
    internal var parantsViewV4: IAPV4TestView? = null


    constructor(context: Context, parantsView: IAPTestView?) : super(context) {

        this.parantsView = parantsView

        this.requestWindowFeature(Window.FEATURE_NO_TITLE)
        this.setCancelable(true)

        setContentView(Resource.getLayoutId(context, "test_view_iap_postbox"))

        listview_postbox = findViewById<View>(Resource.getViewId(context, "listview_postbox")) as ListView


        GameNetwork.posts(object : IGameListener{

            override fun onGameListener(result: Boolean, response: Response?) {

                if (result && response?.isSuccess == true) {

                    listviewadapter_postbox = ListViewAdapter()

                    listview_postbox.adapter = listviewadapter_postbox

                    try {
                        response.resultData?.optJSONArray("post_list")?.let { postList ->

                            if (postList.length() == 0) {
                                val textView_postboxtitle = findViewById<View>(Resource.getViewId(context, "textView_postboxtitle")) as TextView
                                textView_postboxtitle.text = "Postbox is empty."
                            }

                            for (i in 0 until postList.length()) {
                                listviewadapter_postbox.add(Post(postList.optJSONObject(i)))
                            }
                        }

                        listview_postbox.layoutParams = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)


                    } catch (e: Exception) {
                        Toast.makeText(context, "Postbox Error : $e", Toast.LENGTH_LONG).show()
                        dismiss()
                    }


                } else {
                    val msg = "Postbox Error : " + if (response == null) "Invaild Game Session or Network Not Available." else response.resultMsg
                    Toast.makeText(context, msg, Toast.LENGTH_LONG).show()
                    dismiss()
                }
            }
        })
    }

    constructor(context: Context, parantsView: IAPV4TestView?) : super(context) {

        this.parantsViewV4 = parantsView

        this.requestWindowFeature(Window.FEATURE_NO_TITLE)
        this.setCancelable(true)

        setContentView(Resource.getLayoutId(context, "test_view_iap_postbox"))

        listview_postbox = findViewById<View>(Resource.getViewId(context, "listview_postbox")) as ListView


        GameNetwork.posts(object : IGameListener{

            override fun onGameListener(result: Boolean, response: Response?) {

                if (result && response!!.isSuccess) {

                    listviewadapter_postbox = ListViewAdapter()

                    listview_postbox.adapter = listviewadapter_postbox

                    try {
                        response.resultData?.getJSONArray("post_list")?.let { postList ->

                            if (postList.length() == 0) {
                                val textView_postboxtitle = findViewById<View>(Resource.getViewId(context, "textView_postboxtitle")) as TextView
                                textView_postboxtitle.text = "Postbox is empty."
                            }

                            for (i in 0 until postList.length()) {
                                listviewadapter_postbox.add(Post(postList.optJSONObject(i)))
                            }
                        }

                        response.resultData?.optJSONArray("post_subscription_list")?.let { postList ->
                            //reset GameUser.getInstance().subscrptionMarketPidList ( ArrayList )
                            GameUser.subscrptionMarketPidList.clear();
                            for (i in 0 until postList.length()) {
                                listviewadapter_postbox.add(PostSubscription(postList.optJSONObject(i)))
                                GameUser.subscrptionMarketPidList.add(PostSubscription(postList.optJSONObject(i)).product_id)
                            }
                        }

                        listview_postbox.layoutParams = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)


                    } catch (e: Exception) {
                        Toast.makeText(context, "Postbox Error : $e", Toast.LENGTH_LONG).show()
                        dismiss()
                    }


                } else {
                    val msg = "Postbox Error : " + if (response == null) "Invaild Game Session or Network Not Available." else response.resultMsg
                    Toast.makeText(context, msg, Toast.LENGTH_LONG).show()
                    dismiss()
                }
            }
        })

    }


    override fun dismiss() {
        parantsView?.onGameUserUiUpdate()

        parantsViewV4?.onGameUserUiUpdate()

        super.dismiss()
    }


    internal inner class ListViewAdapter : BaseAdapter() {

        private val gamePostList: ArrayList<PostBoxObject>

        init {
            gamePostList = ArrayList()
        }

        override fun getCount(): Int {
            return gamePostList.size
        }

        override fun getItem(position: Int): Any {
            return gamePostList[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            var convertView = convertView

            val context = parent.context

            val gamePost = gamePostList[position]
            val holder: ViewHolder

            if (convertView == null) {

                val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
                convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_iap_post"), parent, false)

                holder = ViewHolder()

                holder.textview_post_assetamount = convertView!!.findViewById<View>(Resource.getViewId(context, "textview_post_assetamount")) as TextView
                holder.textview_post_assetcode = convertView.findViewById<View>(Resource.getViewId(context, "textview_post_assetcode")) as TextView
                holder.textview_post_character_key = convertView.findViewById<View>(Resource.getViewId(context, "textview_post_character_key")) as TextView
                holder.textview_post_usermsg = convertView.findViewById<View>(Resource.getViewId(context, "textview_post_usermsg")) as TextView
                holder.textview_post_date_added = convertView.findViewById<View>(Resource.getViewId(context, "textview_post_date_added")) as TextView
                holder.gamePost = gamePost

                convertView.tag = holder
            } else {

                holder = convertView.tag as ViewHolder
            }

            holder.textview_post_assetamount!!.text = gamePost.getAssetAmount().toString()
            holder.textview_post_assetcode!!.text = gamePost.getAssetCode()
            holder.textview_post_character_key!!.text = gamePost.getCharacterKey()
            holder.textview_post_usermsg!!.text = gamePost.getUserMsg()
            holder.textview_post_date_added!!.text = gamePost.getDate()

            convertView.setOnClickListener {
                if (gamePost is PostSubscription)
                    return@setOnClickListener

                val postId = gamePost.getPostId()

                if (parantsView != null)
                    parantsView!!.addLog("GameUser Receive Post (postId: $postId)\n")
                if (parantsViewV4 != null)
                    parantsViewV4!!.addLog("GameUser Receive Post (postId: $postId)\n")

                Logger.i("\n\n=== Game 우편 수신 ===\n")
                Logger.i(String.format("GameUser Receive Post (postId: $postId)\n", postId))

                GameNetwork.postReceive(object : IGameListener {

                    override fun onGameListener(result: Boolean, response: Response?) {

                        var msg: String? = null

                        if (result && response != null) {

                            if (response.isSuccess) {
                                msg = "GameUser Receive Post Success."

                                listviewadapter_postbox.remove(position)

                            } else {
                                msg = "GameUser Receive Post Failure. : " + response.resultMsg
                            }

                        } else {
                            msg = "GameUser Receive Post Failure."
                        }

                        parantsView?.addLog(msg)

                        parantsViewV4?.addLog(msg)

                        Logger.i(msg)


                        AlertDialog.Builder(context)
                                .setTitle("Receive Post Result")
                                .setMessage(msg)
                                .create().show()

//                        Toast.makeText(context, msg, Toast.LENGTH_SHORT).show();
                    }

                }, gamePost)
            }


            return convertView
        }

        fun add(Post : PostBoxObject) {
            gamePostList.add(Post)
            this.notifyDataSetChanged()
        }

        fun remove(position: Int) {
            gamePostList.removeAt(position)
            this.notifyDataSetChanged()
        }
    }


    internal inner class ViewHolder {

        var textview_post_assetamount: TextView? = null
        var textview_post_assetcode: TextView? = null
        var textview_post_character_key: TextView? = null
        var textview_post_usermsg: TextView? = null
        var textview_post_date_added: TextView? = null

        var gamePost: PostBoxObject? = null
    }

}
