package com.hive.sample.game

import android.app.Activity
import android.app.AlertDialog
import android.content.Intent
import android.net.Uri
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import com.hive.HiveActivity
import java.util.ArrayList

object GameUser {

    var sessionToken: String? = null //that's mean game session token. not lc_session(loginCenter)
    var nickname: String? = null
    var gold: Int = 0
    var gem: Int = 0
    var subscrptionMarketPid: String? = null
    var subscrptionMarketPidList = ArrayList<String>()

    var isLogin = false


    fun login(listener: IGameListener?) {

        GameNetwork.login(object : IGameListener {

            override fun onGameListener(result: Boolean, response: Response?) {

                if (result && response != null && response.isSuccess) {

                    if (reflesh(response)) {

//                        response.subscriptionStatusMsg = "There is a problem with your subscription. Click here to go to the Google Play subscription settings to fix your payment method."
//                        response.subscriptionStoreUrl = "https://play.google.com/store/account/subscriptions?sku=sub01&package=${Android.getPackageName()}"
                        showSubscriptionState(HiveActivity.recentActivity, response) {

                            listener?.onGameListener(true, response)
                        }

                        return
                    }
                }

                listener?.onGameListener(false, null)
            }
        })
    }

    fun logout(listener: IGameListener?) {

        sessionToken = null
        nickname = null
        gold = -1
        gem = -1
        subscrptionMarketPid = null

        isLogin = false

        Handler(Looper.getMainLooper()).post { listener?.onGameListener(true, null) }
    }


    fun update(listener: IGameListener?) {

        GameNetwork.userInfo(object : IGameListener {

            override fun onGameListener(result: Boolean, response: Response?) {

                if (result && response != null && response.isSuccess) {

                    if (reflesh(response)) {

                        listener?.onGameListener(true, response)
                        return
                    }
                }

                listener?.onGameListener(false, null)
            }
        })

    }


    fun reflesh(response: Response?): Boolean {

        return if (response != null) {

            try {
                sessionToken = if (TextUtils.isEmpty(response.sessionToken)) sessionToken else response.sessionToken
                nickname = response.resultData?.optString("nickname") ?: ""
                gold = response.resultData?.optInt("gold") ?: 0
                gem = response.resultData?.optInt("gem") ?: 0

                isLogin = !TextUtils.isEmpty(sessionToken)

                true

            } catch (ignore: Exception) {
                false
            }

        } else
            false
    }


    fun showSubscriptionState(activity: Activity, response: Response, listener: () -> Unit) {
        activity.runOnUiThread {

            response.subscriptionStatusMsg.takeIf { it.isNotBlank() }?.let { msg ->
                val builder = AlertDialog.Builder(activity)
                builder.setMessage(msg)

                response.subscriptionStoreUrl.takeIf { it.isNotBlank() }?.let { url ->
                    builder.setPositiveButton("Google Play") { _, _ ->
                        val intent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
                        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                        try {
                            activity.startActivity(intent)
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                }
                builder.setNegativeButton(android.R.string.ok) { _, _ -> }
                builder.setOnDismissListener {
                    listener()
                }
                .create().show()

            } ?: listener()
        }
    }

}
