/**
 *
 */
package com.hive.sample.uihelper

import java.util.ArrayList
import java.util.HashMap
import java.util.LinkedList

import android.app.Dialog
import android.content.Context
import android.os.Build
import android.os.Bundle
import android.util.Pair
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.EditorInfo
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.Spinner
import android.widget.TextView
import android.widget.ToggleButton

/**
 * @author Red
 *
 * @param context
 */
class SelectDialog(context: Context) : Dialog(context) {

    private var resultCallback: OnDialogResultListener? = null
    private var titleString: String? = null

    private var inputLayout: LinearLayout? = null
    private val layoutViewList = LinkedList<InputContent>()
    private var inputIndex = 0

    private var buttonLayout: LinearLayout? = null
    private var cancelButton: Button? = null
    private var prevButton: Button? = null
    private var dummyView: View? = null
    private var nextButton: Button? = null
    private var finshButton: Button? = null

    private val clickListener = View.OnClickListener { v ->
        inputLayout!!.getChildAt(inputIndex).isEnabled = false

        if (v === cancelButton)
            inputIndex--
        else if (v === prevButton)
            inputIndex--
        else if (v === nextButton)
            inputIndex++
        else if (v === finshButton)
            inputIndex++
        updateLayout()
    }

    override fun onCreate(savedInstanceState: Bundle?) {

        super.onCreate(savedInstanceState)

        val dialogLayout = LinearLayout(context)// 상중하 3등분
        dialogLayout.orientation = LinearLayout.VERTICAL
        // dialogLayout.setWeightSum(100);


        if (titleString != null) {
            // 상단타이틀
            val titleText = TextView(context)
            titleText.text = titleString
            dialogLayout.addView(titleText, LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT))
        }

        // 중앙
        // 입력
        inputLayout = LinearLayout(context).apply {
            this.setPadding(50, 50, 50, 50)
            this.orientation = LinearLayout.VERTICAL
            this.isScrollContainer = true
            for (v in layoutViewList) {
                v.isEnabled = false
                this.addView(v, LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT))
            }
        }
        dialogLayout.addView(inputLayout, LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT))

        // 하단
        // 버튼들.
        buttonLayout = LinearLayout(context).apply {
            this.orientation = LinearLayout.HORIZONTAL
            this.weightSum = 3f
        }

        cancelButton = Button(context).apply {
            this.text = "cancel"
            this.setOnClickListener(clickListener)
        }
        buttonLayout?.addView(cancelButton, LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.MATCH_PARENT, 1f))

        prevButton = Button(context).apply {
            this.text = "prev"
            this.setOnClickListener(clickListener)
        }
        buttonLayout?.addView(prevButton, LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.MATCH_PARENT, 1f))

        // DummyArea
        dummyView = View(context)
        buttonLayout?.addView(dummyView, LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.MATCH_PARENT, 1f))

        nextButton = Button(context).apply {
            this.text = "next"
            this.setOnClickListener(clickListener)
        }
        buttonLayout?.addView(nextButton, LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.MATCH_PARENT, 1f))

        finshButton = Button(context).apply {
            this.text = "finish"
            this.setOnClickListener(clickListener)
        }
        buttonLayout?.addView(finshButton, LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.MATCH_PARENT, 1f))

        dialogLayout.addView(buttonLayout, LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT))

        setContentView(dialogLayout, LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT))

        updateLayout()

        // titleText.setBackgroundColor(Color.BLACK);
        // inputLayout.setBackgroundColor(Color.YELLOW);
        // cancelButton.setBackgroundColor(Color.CYAN);// for Debug
        // prevButton.setBackgroundColor(Color.BLUE);// for Debug
        // dummyView.setBackgroundColor(Color.RED);// for Debug
        // nextButton.setBackgroundColor(Color.GREEN);// for Debug
        // finshButton.setBackgroundColor(Color.MAGENTA);// for Debug
    }

    internal inner class InputContent
    /**
     * @param context
     */
    (context: Context, key: String, var input: View) : LinearLayout(context) {

        var keyTextView: TextView

        init {
            // TODO Auto-generated constructor stub
            keyTextView = TextView(context)
            keyTextView.text = key

            orientation = LinearLayout.HORIZONTAL
            addView(keyTextView, LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.WRAP_CONTENT, 0.3f))
            addView(input)
        }

        /*
		 * (non-Javadoc)
		 *
		 * @see android.view.ViewGroup#onLayout(boolean, int, int, int, int)
		 */
        override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {

            // TODO Auto-generated method stub
            super.onLayout(changed, l, t, r, b)
        }
    }


    fun addInputContent(key: String, v: View) {

        val itemLayout = InputContent(context, key, v)

        layoutViewList.add(itemLayout)
    }

    fun setTitle(titleString: String?) {

        this.titleString = titleString
    }

    fun setDialogResultListener(listener: OnDialogResultListener?) {

        this.resultCallback = listener
    }

    private fun updateLayout() {

        val childCount = inputLayout!!.childCount
        if (inputIndex == childCount) {
            if (resultCallback != null) {
                val result = HashMap<String, Any>()
                for (ic in layoutViewList) {
                    val key = ic.keyTextView.text.toString()
                    val v = ic.input
                    if (v is EditText) {
                        result[key] = v.text.toString()
                    } else if (v is Spinner) {
                        result[key] = v.selectedItem.toString()
                    } else if (v is ToggleButton) {
                        result[key] = if (v.isChecked) java.lang.Boolean.TRUE else java.lang.Boolean.FALSE
                    }
                }
                resultCallback!!.onDialogResult(true, result)
            }
            dismiss()
        } else if (inputIndex < 0) {
            if (resultCallback != null)
                resultCallback!!.onDialogResult(false, null)
            cancel()
        } else {

            if (inputIndex == 0) {
                // hide prev & show cancel
                prevButton!!.visibility = View.GONE
                cancelButton!!.visibility = View.VISIBLE
            } else {
                // show prev & hide cancel
                prevButton!!.visibility = View.VISIBLE
                cancelButton!!.visibility = View.GONE
            }

            if (inputIndex + 1 == childCount) {
                // show finish & hide next
                nextButton!!.visibility = View.GONE
                finshButton!!.visibility = View.VISIBLE
            } else {
                // show next & hide finish
                nextButton!!.visibility = View.VISIBLE
                finshButton!!.visibility = View.GONE
            }

            val nextInput = inputLayout!!.getChildAt(inputIndex)
            nextInput.isEnabled = true
            nextInput.requestFocus()

            inputLayout!!.invalidate()
            buttonLayout!!.invalidate()
        }
    }

    interface OnDialogResultListener {
        fun onDialogResult(completed: Boolean, result: Map<String, Any>?)
    }


    class Builder(private val context: Context) {
        private val inputList = LinkedList<Pair<String, Any>>()
        private var titleString: String? = null
        private var listener: OnDialogResultListener? = null

        fun addInputText(key: String): Builder {

            inputList.add(Pair<String, Any>(key, null))
            return this
        }

        fun addInputText(key: String, defaultValue: String): Builder {

            inputList.add(Pair(key, defaultValue))
            return this
        }

        fun addSelectList(key: String, selectList: List<String>): Builder {

            inputList.add(Pair(key, selectList.toTypedArray()))
            return this
        }

        fun addSelectList(key: String, selectList: Array<String>): Builder {

            inputList.add(Pair(key, selectList))
            return this
        }

        fun addSwitch(key: String, defaultValue: Boolean): Builder {

            inputList.add(Pair(key, java.lang.Boolean.valueOf(defaultValue)))
            return this
        }

        fun setTitle(titleString: String): Builder {

            this.titleString = titleString
            return this
        }

        fun setDialogResultListener(listener: OnDialogResultListener): Builder {

            this.listener = listener
            return this
        }

        fun autobuild(buildType: Class<*>): SelectDialog {

            if (this.titleString == null)
                setTitle(buildType.simpleName)

            val memberFields = buildType.fields

            for (field in memberFields) {
                try {
                    // if value == null , don't put it. cause make "null" string
                    val name = field.name
                    val type = field.type

                    if (type.isEnum) {
                        val enumType = type as Class<out Enum<*>>

                        enumType.enumConstants?.let { enumValues ->
                            val enumValueNames = ArrayList<String>()
                            for (enumValue in enumValues) {
                                enumValueNames.add(enumValue.name)

                            }
                            addSelectList(name, enumValueNames)
                        }
                    } else if (type == Boolean::class.java) {
                        addSwitch(name, false)
                    } else {
                        addInputText(name)
                    }

                } catch (e: IllegalArgumentException) {
                    // TODO Auto-generated catch block
                    e.printStackTrace()
                }

            }
            return build()
        }

        fun build(): SelectDialog {

            val dialog = SelectDialog(context)

            if (inputList.size == 0)
                return dialog
            else {

                var step: Pair<String, Any>? = inputList.poll()
                while (step != null) {

                    if (step.second == null) {
                        // InputText
                        val inputText = EditText(context)
                        inputText.inputType = EditorInfo.TYPE_TEXT_FLAG_NO_SUGGESTIONS
                        inputText.hint = step.first
                        dialog.addInputContent(step.first, inputText)
                    } else if (step.second is String) {
                        val inputText = EditText(context)
                        inputText.inputType = EditorInfo.TYPE_TEXT_FLAG_NO_SUGGESTIONS
                        inputText.hint = step.second as String
                        dialog.addInputContent(step.first, inputText)
                    } else if (step.second is Array<*>) {
                        // SelectInput
                        val list = step.second as Array<String>
                        val spAdapter = ArrayAdapter<String>(context, android.R.layout.simple_spinner_item)
                        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
                        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.HONEYCOMB)
                            spAdapter.addAll(*list)
                        else {
                            for (item in list)
                                spAdapter.add(item)
                        }

                        val spinText = Spinner(context)
                        spinText.adapter = spAdapter
                        spinText.setSelection(0)
                        spinText.prompt = step.first
                        dialog.addInputContent(step.first, spinText)

                    } else if (step.second is Boolean) {
                        val tb = ToggleButton(context)
                        tb.text = step.first
                        tb.isChecked = step.second as Boolean
                        dialog.addInputContent(step.first, tb)
                    }

                    step = inputList.poll()
                }

                dialog.setTitle(titleString)
                dialog.setDialogResultListener(listener)

                if (Build.VERSION.SDK_INT >= 21)
                    dialog.create()
                return dialog
            }
        }
    }

    // N개의 텍스트입력 혹은 콤보박스 선택입력을 이전< , 다음> or 완성 & 취소
    // 프리뷰를 실시간 혹은 입력후 선택.

}



