package com.hive.sample.uihelper


import android.app.Activity
import android.app.AlertDialog
import android.content.DialogInterface
import android.content.Intent
import android.os.Handler
import android.os.Looper
import android.text.method.ScrollingMovementMethod
import android.view.View
import android.view.ViewGroup.LayoutParams
import android.widget.Button
import android.widget.LinearLayout
import android.widget.ScrollView
import android.widget.TextView
import com.hive.Const
import com.hive.Logger
import org.json.JSONObject
import java.text.SimpleDateFormat
import java.util.*


/**
 * 테스터 UI 를 각 기능별로 좌/우로 스크롤 해서 구성하기 위한 Android ViewPager 의 기반 클래스
 *
 * @author ryuvsken
 */
open class ViewPagerBase(activity: Activity, view: View, title: String) {


    protected var TAG = Const.TAG + "_sample"


    var activity: Activity
        protected set

    var view: View
        protected set

    var title: String
        protected set


    // 로그 뷰
    protected var logView: LogTextView? = null


    init {
        this.activity = activity
        this.view = view
        this.title = title
    }


    // View Pager Event
    fun onAddLog(log: String) {}

    open fun onUpdateView() {}

    fun onBackPressed() {}

    fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        return false
    }


    // 이 클래스를 상속받은 Activity 는 onCreate() 함수에서 반드시 이 메서드를 호출해주어야 한다.
    //
    // @param id Layout 에 배치된 LogTextView 의 id
    protected fun createLogView(viewId: Int, btnIdLogDialog: Int, btnIdLogClear: Int) {

        val logView = this.view.findViewById<View>(viewId) as LogTextView
        this.logView = logView
        if (logView != null) {
            logView.movementMethod = ScrollingMovementMethod()
            //logView.setOnCreateContextMenuListener(this);
        }

        var button = this.view.findViewById<View>(btnIdLogDialog) as Button
        button.setOnClickListener { showLogDialog(activity, logView.text.toString()) }

        button = this.view.findViewById<View>(btnIdLogClear) as Button
        button.setOnClickListener { logView.setText("") }

        //button.requestFocus();
    }

    fun addLogS(message: String?) {

        addLog("")

        if (message != null) {
            addLog("$divideString Start $message $divideString")
        } else {
            addLog("$divideString Start $divideString")
        }
    }

    fun addLogE(message: String?) {

        if (message != null) {
            addLog("$divideString End $message $divideString")
        } else {
            addLog("$divideString End $divideString")
        }

        addLog("")
    }

    fun addLogD(message: String) {

        addLog(divideString)
        addLog(message)
        addLog(divideString)
    }


    fun addLog(message: String?) {
        var message = message

        if (message == null) {
            message = "(null)"
        }

        //Log.dB(TAG, message);
        Logger.i(message + "\n")

        val pageViewList = ViewPagerManager.getPageViewList()

        for (view in pageViewList) {
            view.onAddLog(message)
        }

        //		Time now = new Time();
        //		now.setToNow();
        val sdf = SimpleDateFormat("HHmmss")
        val currentDateandTime = sdf.format(Date())
        val finalMessage = currentDateandTime + message
        if (logView != null) {

            activity.runOnUiThread { logView!!.append(finalMessage + "\n") }
        }
    }


    fun addLog(map: Map<*, *>) {

        val sb = StringBuffer()

        val keys = map.keys
        for (key in keys) {

            val obj = map[key]
            if (obj is List<*>) {

                val values = obj as List<String>?
                for (value in values!!) {
                    sb.append(key).append("=").append(value).append("\r\n")
                }
            } else {
                val value = obj as String?
                sb.append(key).append("=").append(value).append("\r\n")
            }
        }

        val text = sb.toString()
        addLog(text)
    }


    fun addLog(json: JSONObject) {

        var text = ""
        try {
            text = json.toString(2)
        } catch (e: Exception) {
            //e.printStackTrace();
            return
        }

        addLog(text)
    }


    // 샘플 앱의 Alert 대화 상자를 노출 한다.
    fun showAlertDialog(title: String?, msg: String?) {
        var title = title
        var msg = msg

        if (title == null) {
            title = "(null)"
        }

        if (msg == null) {
            msg = "(null)"
        }

        val fTitle = title
        val fMsg = msg

        Handler(Looper.getMainLooper()).post {
            AlertDialog.Builder(activity)
                    .setTitle(fTitle).setMessage(fMsg).create().show()
        }
    }


    // 샘플 앱의 로그 대화 상자를 노출 한다.
    fun showLogDialog(resourceId: Int) {

        val message = activity.getString(resourceId) as String
        ViewPagerBase.showLogDialog(activity, message)
    }


    fun updateView() {

        val pageViewList = ViewPagerManager.getPageViewList()

        for (view in pageViewList) {
            view.onUpdateView()
        }
    }

    companion object {


        private val divideString = "======"


        fun showLogDialog(activity: Activity, message: String) {

            val dialogBuilder = AlertDialog.Builder(activity)

            val rootView = buildCustomView(activity, message) ?: return
            dialogBuilder.setView(rootView)
            dialogBuilder.setPositiveButton("Clipboard") { dialog, which ->
                if (which == DialogInterface.BUTTON_POSITIVE) {

                    val txtMessage = rootView.findViewById<View>(1) as TextView
                    com.hive.base.Android.copyToClipboard(activity, "logText", txtMessage.text.toString())

                } else {
                }
            }

            dialogBuilder.setNegativeButton(activity.getText(android.R.string.cancel)) { dialog, which -> }

            val alertDialog = dialogBuilder.create()
            alertDialog.setCanceledOnTouchOutside(false)
            alertDialog.setOnDismissListener { }
            alertDialog.show()
        }

        protected fun buildCustomView(activity: Activity, message: String): View {

            val root = LinearLayout(activity)
            root.orientation = LinearLayout.VERTICAL
            root.setPadding(10, 10, 10, 10)
            root.layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT)
            root.isFocusable = true
            root.isFocusableInTouchMode = true

            val scroll = ScrollView(activity)
            root.addView(scroll, LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT, 1.0f))
            val scrollable = LinearLayout(activity)
            scrollable.orientation = LinearLayout.VERTICAL
            scroll.addView(scrollable)

            val text = TextView(activity)
            text.id = 1
            text.text = message
            scrollable.addView(text)

            return root
        }
    }
}



