package com.hive.sample.uihelper

import android.annotation.SuppressLint
import android.app.Activity
import android.os.Parcelable
import androidx.viewpager.widget.PagerAdapter
import androidx.viewpager.widget.ViewPager
import android.view.View
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import java.util.*


/**
 * 테스터 UI 를 각 기능별로 좌/우로 스크롤 해서 구성하기 위한 Android ViewPager 의 관리 클래스
 *
 * @author ryuvsken
 */
@SuppressLint("StaticFieldLeak")
object ViewPagerManager : androidx.viewpager.widget.PagerAdapter() {

    private var viewPager: androidx.viewpager.widget.ViewPager? = null

    private val pageViewList = ArrayList<ViewPagerBase>()

    private var currentPosition = 0

    private var viewPagerIndicator: ViewPagerIndicator? = null


    fun getPageViewList(): List<ViewPagerBase> {
        return this.pageViewList
    }


    fun getViewPager(): androidx.viewpager.widget.ViewPager? {
        return this.viewPager
    }


    fun setViewPager(activity: Activity, viewPager: androidx.viewpager.widget.ViewPager) {

        this.viewPager = viewPager


        // 현재 페이지의 위치를 표시하기 위한 인디케이터 초기화
        this.viewPagerIndicator = (activity.findViewById<View>(R.id.viewPagerIndicator) as ViewPagerIndicator).apply {
            //원사이의 간격
            this.setItemMargin(15)
            //애니메이션 속도
            this.setAnimDuration(300)
            //indecator 생성
            this.createDotPanel(pageViewList.size, 0, 0)
        }
    }

    fun releaseViewPager() {

        this.viewPager = null
        clearViewList()
    }


    fun addView(v: ViewPagerBase) {

        this.pageViewList.add(v)
    }

    private fun clearViewList() {

        this.pageViewList.clear()
    }


    fun getViewByPosition(i: Int): ViewPagerBase {

        return pageViewList[i]
    }


    override fun getCount(): Int {

        return pageViewList.size
    }

    override fun getPageTitle(position: Int): CharSequence? {
        return pageViewList[position].title
    }


    override fun instantiateItem(pager: View, position: Int): Any {

        val v = pageViewList[position].view

        (pager as androidx.viewpager.widget.ViewPager).addView(v, 0)

        return v
    }


    override fun destroyItem(pager: View, position: Int, view: Any) {
        (pager as androidx.viewpager.widget.ViewPager).removeView(view as View)
    }


    override fun isViewFromObject(pager: View, obj: Any): Boolean {
        return pager === obj
    }


    override fun restoreState(arg0: Parcelable?, arg1: ClassLoader?) {}


    override fun saveState(): Parcelable? {
        return null
    }


    override fun startUpdate(arg0: View) {}


    override fun finishUpdate(arg0: View) {}


    override fun getItemPosition(`object`: Any): Int {
        return androidx.viewpager.widget.PagerAdapter.POSITION_NONE
    }


    fun getCurrentPosition(): Int {
        return this.currentPosition
    }


    fun setCurrentPosition(currentPosition: Int) {
        if (currentPosition < 0 || currentPosition >= pageViewList.size) {
            return
        }

        this.currentPosition = currentPosition
        this.viewPager!!.currentItem = currentPosition
    }


    fun onPageChange(position: Int) {
        this.currentPosition = position

        this.viewPagerIndicator!!.selectDot(position)

        this.pageViewList[position].onUpdateView()
    }

}



